@@ -4,7 +4,7 @@ OOJSConsole.m
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -125,6 +125,7 @@ enum
 	kConsole_reducedDetailMode,					// reduced detail mode, boolean, read/write
 	kConsole_displayFPS,						// display FPS (and related info), boolean, read/write
 	kConsole_platformDescription,				// Information about system we're running on in unspecified format, string, read-only
+	kConsole_ignoreDroppedPackets,				// boolean (default false), read/write
 	kConsole_pedanticMode,						// JS pedantic mode (JS_STRICT flag, not the same as "use strict"), boolean (default true), read/write
 	kConsole_showErrorLocations,				// Show error/warning source locations, boolean (default true), read/write
 	kConsole_dumpStackForErrors,				// Write stack dump when reporting error/exception, boolean (default false), read/write
@@ -161,6 +162,7 @@ static JSPropertySpec sConsoleProperties[] =
 	{ "displayFPS",							kConsole_displayFPS,						OOJS_PROP_READWRITE_CB },
 	{ "platformDescription",				kConsole_platformDescription,				OOJS_PROP_READONLY_CB },
 	{ "pedanticMode",						kConsole_pedanticMode,						OOJS_PROP_READWRITE_CB },
+	{ "ignoreDroppedPackets",				kConsole_ignoreDroppedPackets,				OOJS_PROP_READWRITE_CB },
 	{ "__showErrorLocations",				kConsole_showErrorLocations,				OOJS_PROP_HIDDEN_READWRITE_CB },
 	{ "__dumpStackForErrors",				kConsole_dumpStackForErrors,				OOJS_PROP_HIDDEN_READWRITE_CB },
 	{ "__dumpStackForWarnings",				kConsole_dumpStackForWarnings,				OOJS_PROP_HIDDEN_READWRITE_CB },
@@ -313,7 +315,7 @@ static JSBool ConsoleGetProperty(JSContext *context, JSObject *this, jsid propID
 	{
 #ifndef NDEBUG
 		case kConsole_debugFlags:
-			*value = INT_TO_JSVAL(gDebugFlags);
+			*value = INT_TO_JSVAL((uint32_t)gDebugFlags);
 			break;
 #endif		
 			
@@ -345,6 +347,10 @@ static JSBool ConsoleGetProperty(JSContext *context, JSObject *this, jsid propID
 			}
 			break;
 			
+		case kConsole_ignoreDroppedPackets:
+			*value = OOJSValueFromBOOL([[OODebugMonitor sharedDebugMonitor] TCPIgnoresDroppedPackets]);
+			break;
+			
 		case kConsole_showErrorLocations:
 			*value = OOJSValueFromBOOL([[OOJavaScriptEngine sharedEngine] showErrorLocations]);
 			break;
@@ -457,6 +463,13 @@ static JSBool ConsoleSetProperty(JSContext *context, JSObject *this, jsid propID
 			}
 			break;
 			
+		case kConsole_ignoreDroppedPackets:
+			if (JS_ValueToBoolean(context, *value, &bValue))
+			{
+				[[OODebugMonitor sharedDebugMonitor] setTCPIgnoresDroppedPackets:bValue];
+			}
+			break;
+			
 		case kConsole_showErrorLocations:
 			if (JS_ValueToBoolean(context, *value, &bValue))
 			{
@@ -619,7 +632,7 @@ static JSBool ConsoleSettingsSetProperty(JSContext *context, JSObject *this, jsi
 		}
 		else
 		{
-			OOJSReportWarning(context, @"debugConsole.settings: could not convert %@ to native object.", [NSString stringWithJavaScriptValue:*value inContext:context]);
+			OOJSReportWarning(context, @"debugConsole.settings: could not convert %@ to native object.", OOStringFromJSValue(context, *value));
 		}
 	}
 	OOJSResumeTimeLimiter();
@@ -762,7 +775,7 @@ static JSBool ConsoleCallObjCMethod(JSContext *context, uintN argc, jsval *vp)
 	object = OOJSNativeObjectFromJSObject(context, OOJS_THIS);
 	if (object == nil)
 	{
-		OOJSReportError(context, @"Attempt to call __callObjCMethod() for non-Objective-C object %@.", [NSString stringWithJavaScriptValue:JS_THIS(context, vp) inContext:context]);
+		OOJSReportError(context, @"Attempt to call __callObjCMethod() for non-Objective-C object %@.", OOStringFromJSValueEvenIfNull(context, JS_THIS(context, vp)));
 		return NO;
 	}
 	
@@ -821,7 +834,7 @@ static JSBool ConsoleIsExecutableJavaScript(JSContext *context, uintN argc, jsva
 	
 	OOJSResumeTimeLimiter();
 	
-	OOJS_RETURN_BOOL(YES);
+	OOJS_RETURN_BOOL(result);
 	
 	OOJS_NATIVE_EXIT
 }
@@ -834,8 +847,8 @@ static JSBool ConsoleDisplayMessagesInClass(JSContext *context, uintN argc, jsva
 	
 	NSString				*messageClass = nil;
 	
-	messageClass = [NSString stringWithJavaScriptValue:OOJS_ARGV[0] inContext:context];
-	OOJS_RETURN_BOOL(OOLogWillDisplayMessagesInClass(messageClass));
+	messageClass = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	OOJS_RETURN_BOOL(messageClass != nil && OOLogWillDisplayMessagesInClass(messageClass));
 	
 	OOJS_NATIVE_EXIT
 }
@@ -911,10 +924,12 @@ typedef struct
 
 static void DumpCallback(const char *name, void *rp, JSGCRootType type, void *datap)
 {
+	assert(type == JS_GC_ROOT_VALUE_PTR || type == JS_GC_ROOT_GCTHING_PTR);
+	
 	DumpCallbackData *data = datap;
 	
 	const char *typeString = "unknown type";
-	jsval value = JSVAL_VOID;
+	jsval value;
 	switch (type)
 	{
 		case JS_GC_ROOT_VALUE_PTR:
