@@ -3,7 +3,7 @@
 Entity.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -28,7 +28,6 @@ MA 02110-1301, USA.
 #import "OOPlanetEntity.h"
 
 #import "OOMaths.h"
-#import "Geometry.h"
 #import "Universe.h"
 #import "GameController.h"
 #import "ResourceManager.h"
@@ -106,7 +105,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	
 #ifndef NDEBUG
 	gLiveEntityCount--;
-	gTotalEntityMemory += [self oo_objectSize];
+	gTotalEntityMemory -= [self oo_objectSize];
 #endif
 	
 	[super dealloc];
@@ -119,7 +118,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (OOUInteger) sessionID
+- (NSUInteger) sessionID
 {
 	return _sessionID;
 }
@@ -131,6 +130,12 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (BOOL)isDock
+{
+	return NO;
+}
+
+
 - (BOOL)isStation
 {
 	return isStation;
@@ -184,10 +189,16 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (BOOL) isVisualEffect
+{
+	return NO;
+}
+
+
 - (BOOL) validForAddToUniverse
 {
-	OOUInteger mySessionID = [self sessionID];
-	OOUInteger currentSessionID = [UNIVERSE sessionID];
+	NSUInteger mySessionID = [self sessionID];
+	NSUInteger currentSessionID = [UNIVERSE sessionID];
 	if (EXPECT_NOT(mySessionID != currentSessionID))
 	{
 		OOLogERR(@"entity.invalidSession", @"Entity %@ from session %lu cannot be added to universe in session %lu. This is an internal error, please report it.", [self shortDescription], mySessionID, currentSessionID);
@@ -582,6 +593,19 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (Vector) position
+{
+	return position;
+}
+
+
+// Exposed to uniform bindings.
+- (Vector) relativePosition
+{
+	return vector_subtract([self position], [PLAYER position]);
+}
+
+
 - (void) setPosition:(Vector) posn
 {
 	position = posn;
@@ -607,12 +631,10 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	Vector		abspos = vector_add(position, OOVectorMultiplyMatrix(offset, rotMatrix));
 	Entity		*last = nil;
 	Entity		*father = [self parentEntity];
-	OOMatrix	r_mat;
 	
-	while ((father)&&(father != last)  && (father != NO_TARGET))
+	while (father != nil && father != last)
 	{
-		r_mat = [father drawRotationMatrix];
-		abspos = vector_add(OOVectorMultiplyMatrix(abspos, r_mat), [father position]);
+		abspos = vector_add(OOVectorMultiplyMatrix(abspos, [father drawRotationMatrix]), [father position]);
 		last = father;
 		if (![last isSubEntity]) break;
 		father = [father owner];
@@ -627,6 +649,12 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
+- (double) camZeroDistance
+{
+	return cam_zero_distance;
+}
+
+
 - (NSComparisonResult) compareZeroDistance:(Entity *)otherEntity
 {
 	if ((otherEntity)&&(zero_distance > otherEntity->zero_distance))
@@ -788,13 +816,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (void) moveForward:(double) amount
+- (void) moveForward:(double)amount
 {
-	Vector		forward = vector_forward_from_quaternion(orientation);
+	Vector forward = vector_multiply_scalar(vector_forward_from_quaternion(orientation), amount);
+	position = vector_add(position, forward);
 	distanceTravelled += amount;
-	position.x += amount * forward.x;
-	position.y += amount * forward.y;
-	position.z += amount * forward.z;
 }
 
 
@@ -824,12 +850,6 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (Vector) position
-{
-	return position;
-}
-
-
 - (BOOL) canCollide
 {
 	return YES;
@@ -858,13 +878,28 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 {
 	if (_status != STATUS_COCKPIT_DISPLAY)
 	{
-		zero_distance = distance2(PLAYER->position, position);
+		if ([self isSubEntity])
+		{
+			zero_distance = [[self owner] zeroDistance];
+			cam_zero_distance = [[self owner] camZeroDistance];
+		}
+		else
+		{
+			zero_distance = distance2(PLAYER->position, position);
+			cam_zero_distance = distance2([PLAYER viewpointPosition], position);
+		}
 	}
 	else
 	{
 		zero_distance = magnitude2(position);
+		cam_zero_distance = zero_distance;
 	}
 	
+	if ([self status] != STATUS_COCKPIT_DISPLAY)
+	{
+		position = vector_add(position, vector_multiply_scalar(velocity, delta_t));
+	}
+
 	hasMoved = !vector_equal(position, lastPosition);
 	hasRotated = !quaternion_equal(orientation, lastOrientation);
 	lastPosition = position;
@@ -872,12 +907,6 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (void) applyVelocityWithTimeDelta:(OOTimeDelta)delta_t
-{
-	position = vector_add(position, vector_multiply_scalar(velocity, delta_t));
-}
-
-
 - (BOOL) checkCloseCollisionWith:(Entity *)other
 {
 	return other != nil;
@@ -891,14 +920,7 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 }
 
 
-- (Geometry *)geometry
-{
-	OOLogGenericSubclassResponsibility();
-	return nil;
-}
-
-
-- (void) drawEntity:(BOOL)immediate :(BOOL)translucent
+- (void) drawImmediate:(bool)immediate translucent:(bool)translucent
 {
 	OOLogGenericSubclassResponsibility();
 }
@@ -917,10 +939,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 		OOLog(@"dumpState", @"State for %@:", self);
 		OOLogPushIndent();
 		OOLogIndent();
-		NS_DURING
+		@try
+		{
 			[self dumpSelfState];
-		NS_HANDLER
-		NS_ENDHANDLER
+		}
+		@catch (id exception) {}
 		OOLogPopIndent();
 	}
 }
@@ -955,10 +978,11 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 	ADD_FLAG_IF_SET(hasMoved);
 	ADD_FLAG_IF_SET(hasRotated);
 	ADD_FLAG_IF_SET(isSunlit);
-	ADD_FLAG_IF_SET(collisionTestFilter);
 	ADD_FLAG_IF_SET(throw_sparks);
 	flagsString = [flags count] ? [flags componentsJoinedByString:@", "] : (NSString *)@"none";
 	OOLog(@"dumpState.entity", @"Flags: %@", flagsString);
+	OOLog(@"dumpState.entity", @"Collision Test Filter: %u", collisionTestFilter);
+
 }
 
 
@@ -1017,7 +1041,13 @@ static NSString * const kOOLogEntityUpdateError				= @"entity.linkedList.update.
 
 - (BOOL) isVisible
 {
-	return zero_distance <= ABSOLUTE_NO_DRAW_DISTANCE2;
+	return cam_zero_distance <= ABSOLUTE_NO_DRAW_DISTANCE2;
+}
+
+
+- (BOOL) isImmuneToBreakPatternHide
+{
+	return isImmuneToBreakPatternHide;
 }
 
 @end
