@@ -4,7 +4,7 @@ OOExhaustPlumeEntity.m
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,9 +30,20 @@ MA 02110-1301, USA.
 #import "Universe.h"
 #import "OOMacroOpenGL.h"
 
+#import "OOTexture.h"
+#import "OOGraphicsResetManager.h"
 
-#define kOverallAlpha		0.5f
-#define kTimeStep			0.05
+
+#define kOverallAlpha		1.0f
+#define kTimeStep			0.05f
+#define kFadeLevel1  0.4f
+#define kFadeLevel2  0.2f
+#define kFadeLevel3  0.02f
+#define kScaleLevel1  1.0f
+#define kScaleLevel2  0.8f
+#define kScaleLevel3  0.6f
+
+static OOTexture *sPlumeTexture = nil;
 
 
 @interface OOExhaustPlumeEntity (Private)
@@ -86,9 +97,12 @@ MA 02110-1301, USA.
 
 - (void) update:(OOTimeDelta) delta_t
 {
-	// don't draw if there's no ship, or if we're just jumping out of whitchspace/docked at a station!
+// Profiling: this function and subfunctions are expensive - CIM
+
+	// don't draw if there's no ship, or if we're just jumping out of witchspace/docked at a station!
 	ShipEntity  *ship = [self owner];
-	if (EXPECT_NOT(ship == nil || ([ship isPlayer] && [ship suppressFlightNotifications]))) return;
+// also don't draw if the ship isn't visible
+	if (EXPECT_NOT(ship == nil || ![ship isVisible] || ([ship isPlayer] && [ship suppressFlightNotifications]))) return;
 
 	OOTimeAbsolute now = [UNIVERSE getTime];
 	if ([UNIVERSE getTime] > _trackTime + kTimeStep)
@@ -96,8 +110,8 @@ MA 02110-1301, USA.
 		[self saveToLastFrame];
 		_trackTime = now;
 	}
-	
-	GLfloat ex_emissive[4]	= {0.6f, 0.8f, 1.0f, 0.9f * kOverallAlpha};   // pale blue
+
+	GLfloat ex_emissive[4]	= {0.7f, 0.9, 1.0f, 0.9f * kOverallAlpha};   // pale blue
 	const GLfloat s1[8] = { 0.0, M_SQRT1_2, 1.0, M_SQRT1_2, 0.0, -M_SQRT1_2, -1.0, -M_SQRT1_2};
 	const GLfloat c1[8] = { 1.0, M_SQRT1_2, 0.0, -M_SQRT1_2, -1.0, -M_SQRT1_2, 0.0, M_SQRT1_2};
 	
@@ -107,17 +121,17 @@ MA 02110-1301, USA.
 	{
 		.timeframe = [UNIVERSE getTime],
 		.orientation = shipQrotation,
-		.k = vector_forward_from_quaternion(shipQrotation)
+		.k = [ship forwardVector]
 	};
 	int dam = [ship damage];
+
 	GLfloat speed = [ship speedFactor];
-	
 	// don't draw if not moving.
 	if (EXPECT_NOT(speed <= 0.001f)) return;
 	
 	GLfloat hyper_fade = 8.0f / (8.0f + speed * speed * speed);
 	
-	GLfloat flare_factor = speed * ex_emissive[3] * hyper_fade;
+	GLfloat flare_factor = fmaxf(speed,1.0) * ex_emissive[3] * hyper_fade;
 	GLfloat red_factor = speed * ex_emissive[0] * (ranrot_rand() % 11) * 0.1;	// random fluctuations
 	GLfloat green_factor = speed * ex_emissive[1] * hyper_fade;
 	
@@ -134,23 +148,34 @@ MA 02110-1301, USA.
 		flare_factor = 0.0;
 	
 	Vector currentPos = ship->position;
-	Vector vfwd = vector_forward_from_quaternion(shipQrotation);
+	Vector vfwd = [ship forwardVector];
 	GLfloat	spd = 0.5 * [ship flightSpeed];
 	vfwd = vector_multiply_scalar(vfwd, spd);
-	Vector master_i = vector_right_from_quaternion(shipQrotation);
+	Vector master_i = [ship rightVector];
 	Vector vi,vj,vk;
 	vi = master_i;
-	vj = vector_up_from_quaternion(shipQrotation);
-	vk = cross_product(vi, vj);
+	vj = [ship upVector];
+	vk = [ship forwardVector];
 	zero.position = make_vector(currentPos.x + vi.x * position.x + vj.x * position.y + vk.x * position.z,
 								currentPos.y + vi.y * position.x + vj.y * position.y + vk.y * position.z,
 								currentPos.z + vi.z * position.x + vj.z * position.y + vk.z * position.z);
 	
-	GLfloat i01 = -0.03 * hyper_fade;
-	GLfloat i03 = -0.12;
-	GLfloat i06 = -0.25;
-	GLfloat i08 = -0.32;
-	GLfloat i10 = -0.40;
+	GLfloat speedScale = fminf(1.0,speed*5.0);
+
+	GLfloat exhaust_factor = _exhaustScale.z;
+	if (exhaust_factor < 0.5)
+	{
+		exhaust_factor = 1.0;
+	}
+	else if (exhaust_factor > 2.0)
+	{
+		exhaust_factor = 1.0;
+	}
+	GLfloat i01 = -0.00 * hyper_fade;
+	GLfloat i03 = -0.12 * exhaust_factor;
+	GLfloat i06 = -0.25 * exhaust_factor;
+	GLfloat i08 = -0.32 * exhaust_factor;
+	GLfloat i10 = -0.40 * exhaust_factor;
 	GLfloat q01 = i01/i10;	// factor for trail
 	GLfloat q03 = i03/i10;
 	GLfloat q06 = i06/i10;
@@ -173,29 +198,27 @@ MA 02110-1301, USA.
 	int iv = 0;
 	int i;
 	float r1;
-	
+
+//	f01.position = vector_subtract(zero.position, vk); // 1m out from zero
+	f01.position = zero.position;
+
 	ex_emissive[3] = flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
 	ex_emissive[1] = green_factor;	// diminish green part towards rear of exhaust
 	ex_emissive[0] = red_factor;		// diminish red part towards rear of exhaust
-	_vertices[iv++] = f03.position.x + b03.x;// + zero.k.x * flare_factor * 4.0;
-	_vertices[iv++] = f03.position.y + b03.y;// + zero.k.y * flare_factor * 4.0;
-	_vertices[iv++] = f03.position.z + b03.z;// + zero.k.z * flare_factor * 4.0;
+	_vertices[iv++] = f01.position.x + b01.x;// + zero.k.x * flare_factor * 4.0;
+	_vertices[iv++] = f01.position.y + b01.y;// + zero.k.y * flare_factor * 4.0;
+	_vertices[iv++] = f01.position.z + b01.z;// + zero.k.z * flare_factor * 4.0;
 	_exhaustBaseColors[ci++] = ex_emissive[0];
 	_exhaustBaseColors[ci++] = ex_emissive[1];
 	_exhaustBaseColors[ci++] = ex_emissive[2];
 	_exhaustBaseColors[ci++] = ex_emissive[3];
-	
-	ex_emissive[3] = 0.9 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
-	ex_emissive[1] = 0.9 * green_factor;	// diminish green part towards rear of exhaust
-	ex_emissive[0] = 0.9 * red_factor;		// diminish red part towards rear of exhaust
+
+
 	Vector k1 = f01.k;
 	Vector j1 = cross_product(master_i, k1);
-	Vector i1 = cross_product(j1, k1);
-	
-	f01.position = vector_subtract(zero.position, vk); // 1m out from zero
-	
-	i1.x *= _exhaustScale.x;	i1.y *= _exhaustScale.x;	i1.z *= _exhaustScale.x;
-	j1.x *= _exhaustScale.y;	j1.y *= _exhaustScale.y;	j1.z *= _exhaustScale.y;
+	Vector i1 = vector_multiply_scalar(cross_product(j1, k1), _exhaustScale.x * speedScale);
+	j1 = vector_multiply_scalar(j1, _exhaustScale.y * speedScale);
+
 	for (i = 0; i < 8; i++)
 	{
 		_vertices[iv++] = f01.position.x + b01.x + s1[i] * i1.x + c1[i] * j1.x;
@@ -206,13 +229,14 @@ MA 02110-1301, USA.
 		_exhaustBaseColors[ci++] = ex_emissive[2];
 		_exhaustBaseColors[ci++] = ex_emissive[3];
 	}
-	
-	ex_emissive[3] = 0.6 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
-	ex_emissive[1] = 0.6 * green_factor;	// diminish green part towards rear of exhaust
-	ex_emissive[0] = 0.6 * red_factor;		// diminish red part towards rear of exhaust
+
+	ex_emissive[3] = kFadeLevel1 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
+	ex_emissive[1] = kFadeLevel1 * green_factor;	// diminish green part towards rear of exhaust
+	ex_emissive[0] = kFadeLevel1 * red_factor;		// diminish red part towards rear of exhaust
+
 	k1 = f03.k;
-	i1 = vector_multiply_scalar(cross_product(j1, k1), _exhaustScale.x);
-	j1 = vector_multiply_scalar(cross_product(master_i, k1), _exhaustScale.y);
+	i1 = vector_multiply_scalar(cross_product(j1, k1), _exhaustScale.x * kScaleLevel1 * speedScale);
+	j1 = vector_multiply_scalar(cross_product(master_i, k1), _exhaustScale.y * kScaleLevel1 * speedScale);
 	for (i = 0; i < 8; i++)
 	{
 		r1 = randf();
@@ -225,12 +249,13 @@ MA 02110-1301, USA.
 		_exhaustBaseColors[ci++] = ex_emissive[3];
 	}
 	
-	ex_emissive[3] = 0.4 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
-	ex_emissive[1] = 0.4 * green_factor;	// diminish green part towards rear of exhaust
-	ex_emissive[0] = 0.4 * red_factor;		// diminish red part towards rear of exhaust
+	ex_emissive[3] = kFadeLevel2 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
+	ex_emissive[1] = kFadeLevel2 * green_factor;	// diminish green part towards rear of exhaust
+	ex_emissive[0] = kFadeLevel2 * red_factor;		// diminish red part towards rear of exhaust
+
 	k1 = f06.k;
-	i1 = vector_multiply_scalar(cross_product(j1, k1), 0.8f * _exhaustScale.x);
-	j1 = vector_multiply_scalar(cross_product(master_i, k1), 0.8f * _exhaustScale.y);
+	i1 = vector_multiply_scalar(cross_product(j1, k1), 0.8f * _exhaustScale.x * kScaleLevel2 * speedScale);
+	j1 = vector_multiply_scalar(cross_product(master_i, k1), 0.8f * _exhaustScale.y * kScaleLevel2 * speedScale);
 	for (i = 0; i < 8; i++)
 	{
 		r1 = randf();
@@ -243,12 +268,12 @@ MA 02110-1301, USA.
 		_exhaustBaseColors[ci++] = ex_emissive[3];
 	}
 	
-	ex_emissive[3] = 0.2 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
-	ex_emissive[1] = 0.2 * green_factor;	// diminish green part towards rear of exhaust
-	ex_emissive[0] = 0.2 * red_factor;		// diminish red part towards rear of exhaust
+	ex_emissive[3] = kFadeLevel3 * flare_factor * kOverallAlpha;	// fade alpha towards rear of exhaust
+	ex_emissive[1] = kFadeLevel3 * green_factor;	// diminish green part towards rear of exhaust
+	ex_emissive[0] = kFadeLevel3 * red_factor;		// diminish red part towards rear of exhaust
 	k1 = f08.k;
-	i1 = vector_multiply_scalar(cross_product(j1, k1), 0.5f * _exhaustScale.x);
-	j1 = vector_multiply_scalar(cross_product(master_i, k1), 0.5f * _exhaustScale.y);
+	i1 = vector_multiply_scalar(cross_product(j1, k1), 0.5f * _exhaustScale.x * kScaleLevel3 * speedScale);
+	j1 = vector_multiply_scalar(cross_product(master_i, k1), 0.5f * _exhaustScale.y * kScaleLevel3 * speedScale);
 	for (i = 0; i < 8; i++)
 	{
 		r1 = randf();
@@ -275,52 +300,145 @@ MA 02110-1301, USA.
 	(void)iv; (void)ci;	// Suppress Clang static analyzer warnings.
 }
 
-
 GLuint tfan1[10] =    {	0,	1,	2,	3,	4,	5,	6,	7,	8,	1 };		// initial fan 0..9
+/*
 GLuint qstrip1[18] =  {	1,	9,	2,	10,	3,	11,	4,	12,	5,	13,	6,	14,	7,	15,	8,	16,	1,	9 };		// first quadstrip 10..27
 GLuint qstrip2[18] =  {	9,	17,	10,	18,	11,	19,	12,	20,	13,	21,	14,	22,	15,	23,	16,	24,	9,	17 };	// second quadstrip 28..45
 GLuint qstrip3[18] =  {	17,	25,	18,	26,	19,	27,	20,	28,	21,	29,	22,	30,	23,	31,	24,	32,	17,	25 };	// third quadstrip 46..63
-GLuint tfan2[10] =    {	33,	25,	26,	27,	28,	29,	30,	31,	32,	25 };	// final fan 64..73
+GLuint tfan2[10] =    {	33,	25,	26,	27,	28,	29,	30,	31,	32,	25 };	// final fan 64..73  */
+
+// normal polys
+GLuint tstr1[9] = {  1, 5, 9, 13, 17, 21, 25, 29, 33 };
+GLuint tstr2[9] = {  2, 6, 10, 14, 18, 22, 26, 30, 33 };
+GLuint tstr3[9] = {  3, 7, 11, 15, 19, 23, 27, 31, 33 };
+GLuint tstr4[9] = {  4, 8, 12, 16, 20, 24, 28, 32, 33 }; 
+
+// aft-view special polys
+GLuint afttstr1[4] = {  1, 5, 25, 29 };
+GLuint afttstr2[4] = {  2, 6, 26, 30 };
+GLuint afttstr3[4] = {  3, 7, 27, 31 };
+GLuint afttstr4[4] = {  4, 8, 28, 32 }; 
 
 
-- (void) drawSubEntity:(BOOL) immediate:(BOOL) translucent
+GLfloat pA[6] = { 0.01, 0.0, 2.0, 4.0, 6.0, 10.0 }; // phase adjustments
+
+
+- (void) drawSubEntityImmediate:(bool)immediate translucent:(bool)translucent
 {
 	if (!translucent)  return;
 	
 	ShipEntity *ship = [self owner];
-	if ([ship speedFactor] <= 0.0)  return;	// don't draw if there's no fire!
-	
+	if ([ship speedFactor] <= 0.001f)  return;	// don't draw if not moving according to 'update' calculation
+
 	OO_ENTER_OPENGL();
+	OOSetOpenGLState(OPENGL_STATE_ADDITIVE_BLENDING);
 	
 	OOGL(glPopMatrix());	// restore absolute positioning
 	OOGL(glPushMatrix());	// avoid stack underflow
 	
 	OOGL(glPushAttrib(GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT));
 	
+	OOGL(glDisable(GL_LIGHTING));
 	OOGL(glEnable(GL_BLEND));
-	OOGL(glDisable(GL_TEXTURE_2D));
-	OOGL(glDisable(GL_CULL_FACE));		// face culling
+	OOGL(glDepthMask(GL_FALSE));
+	OOGL(glEnableClientState(GL_TEXTURE_COORD_ARRAY));
+	OOGL(glEnable(GL_TEXTURE_2D));
+	[[self texture] apply];
+
+//	OOGL(glDisable(GL_CULL_FACE));		// face culling
 	OOGL(glShadeModel(GL_SMOOTH));
-	OOGL(glBlendFunc(GL_SRC_ALPHA, GL_ONE));
 	
-	OOGL(glEnableClientState(GL_VERTEX_ARRAY));
-	OOGL(glVertexPointer(3, GL_FLOAT, 0, _vertices));
 	OOGL(glEnableClientState(GL_COLOR_ARRAY));
+	OOGL(glVertexPointer(3, GL_FLOAT, 0, _vertices));
 	OOGL(glColorPointer(4, GL_FLOAT, 0, _exhaustBaseColors));
-	OOGL(glDisableClientState(GL_NORMAL_ARRAY));
-	OOGL(glDisableClientState(GL_TEXTURE_COORD_ARRAY));
-	OOGL(glDisableClientState(GL_EDGE_FLAG_ARRAY));
+
+	double intpart, dphase = 1.0-modf((double)[UNIVERSE getTime]*2.5,&intpart);
+	GLfloat phase = (GLfloat)dphase;
+
+	GLfloat texCoords[68] = {
+		0.5, phase+pA[0],
+
+		0.1, phase+pA[1], 0.1, phase+pA[1], 
+		0.1, phase+pA[1], 0.1, phase+pA[1], 
+		0.9, phase+pA[1], 0.9, phase+pA[1], 
+		0.9, phase+pA[1], 0.9, phase+pA[1],
+
+		0.1, phase+pA[2], 0.1, phase+pA[2], 
+		0.1, phase+pA[2], 0.1, phase+pA[2], 
+		0.9, phase+pA[2], 0.9, phase+pA[2], 
+		0.9, phase+pA[2], 0.9, phase+pA[2],
+
+		0.1, phase+pA[3], 0.1, phase+pA[3], 
+		0.1, phase+pA[3], 0.1, phase+pA[3], 
+		0.9, phase+pA[3], 0.9, phase+pA[3], 
+		0.9, phase+pA[3], 0.9, phase+pA[3],
+
+		0.1, phase+pA[4], 0.1, phase+pA[4], 
+		0.1, phase+pA[4], 0.1, phase+pA[4], 
+		0.9, phase+pA[4], 0.9, phase+pA[4], 
+		0.9, phase+pA[4], 0.9, phase+pA[4],
+
+		0.5, phase+pA[5],
+	};
+	OOGL(glTexCoordPointer(2, GL_FLOAT, 0, texCoords));
+
+	// reduced detail for internal view to avoid rendering artefacts
+	if ([[self owner] isPlayer] && [UNIVERSE viewDirection] != VIEW_CUSTOM)
+	{
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_INT, afttstr1));
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_INT, afttstr2));
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_INT, afttstr3));
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_INT, afttstr4));
+	} 
+	else
+	{
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 9, GL_UNSIGNED_INT, tstr1));
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 9, GL_UNSIGNED_INT, tstr2));
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 9, GL_UNSIGNED_INT, tstr3));
+		OOGL(glDrawElements(GL_TRIANGLE_STRIP, 9, GL_UNSIGNED_INT, tstr4));
+	}
+
+	/* Need a different texture and color array for this segment */
+	GLfloat fanTextures[18] = {
+		0.5, 0.0+phase,
+		0.2, 0.0+phase,
+		0.2, 0.1+phase,
+		0.2, 0.2+phase,
+		0.2, 0.3+phase,
+		0.2, 0.4+phase,
+		0.2, 0.3+phase,
+		0.2, 0.2+phase,
+		0.2, 0.1+phase
+	};
+	OOGL(glTexCoordPointer(2, GL_FLOAT, 0, fanTextures));
 	
+	GLfloat fanColors[36];
+	GLfloat fr = _exhaustBaseColors[0], fg = _exhaustBaseColors[1], fb = _exhaustBaseColors[2];
+	unsigned i = 0;
+	fanColors[i++] = fr;
+	fanColors[i++] = fg;
+	fanColors[i++] = fb;
+	fanColors[i++] = 1.0;
+	for (;i<36;)
+	{
+		fanColors[i++] = fr;
+		fanColors[i++] = fg;
+		fanColors[i++] = fb;
+		fanColors[i++] = 0.5;
+	}
+	OOGL(glColorPointer(4, GL_FLOAT, 0, fanColors));
+
 	OOGL(glDrawElements(GL_TRIANGLE_FAN, 10, GL_UNSIGNED_INT, tfan1));
-	OOGL(glDrawElements(GL_QUAD_STRIP, 18, GL_UNSIGNED_INT, qstrip1));
-	OOGL(glDrawElements(GL_QUAD_STRIP, 18, GL_UNSIGNED_INT, qstrip2));
-	OOGL(glDrawElements(GL_QUAD_STRIP, 18, GL_UNSIGNED_INT, qstrip3));
-	OOGL(glDrawElements(GL_TRIANGLE_FAN, 10, GL_UNSIGNED_INT, tfan2));
-	
-	OOGL(glDisableClientState(GL_VERTEX_ARRAY));
+
+	OOGL(glDisableClientState(GL_TEXTURE_COORD_ARRAY));
+	OOGL(glDisable(GL_TEXTURE_2D));
+
 	OOGL(glDisableClientState(GL_COLOR_ARRAY));
 	
+
 	OOGL(glPopAttrib());
+	
+	OOVerifyOpenGLState();
 }
 
 
@@ -425,6 +543,42 @@ GLuint tfan2[10] =    {	33,	25,	26,	27,	28,	29,	30,	31,	32,	25 };	// final fan 6
 	_exhaustScale = vector_multiply_scalar(_exhaustScale, factor);
 }
 
+
+- (OOTexture *) texture
+{
+	return [OOExhaustPlumeEntity plumeTexture];
+}
+
+
++ (void) setUpTexture
+{
+	if (sPlumeTexture == nil)
+	{
+		sPlumeTexture = [[OOTexture textureWithName:@"oolite-exhaust-blur.png"
+										  inFolder:@"Textures"
+										   options:kOOTextureMinFilterMipMap | kOOTextureMagFilterLinear | kOOTextureAlphaMask | kOOTextureRepeatT | kOOTextureRepeatS
+										anisotropy:kOOTextureDefaultAnisotropy / 2.0
+										   lodBias:0.0] retain];
+		[[OOGraphicsResetManager sharedManager] registerClient:(id<OOGraphicsResetClient>)[OOExhaustPlumeEntity class]];
+
+	}
+}
+
+
++ (OOTexture *) plumeTexture
+{
+	if (sPlumeTexture == nil)  [self setUpTexture];
+	return sPlumeTexture;
+}
+
+
++ (void) resetGraphicsState
+{
+	[sPlumeTexture release];
+	sPlumeTexture = nil;
+}
+
+
 @end
 
 
