@@ -6,7 +6,7 @@ Flashing light attached to ships.
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,8 +35,9 @@ MA 02110-1301, USA.
 	float					_frequency;
 	float					_phase;
 	float					_wave;
+	float         _brightfraction;
 	NSArray					*_colors;
-	OOUInteger				_activeColor;
+	NSUInteger				_activeColor;
 	
 	OOTimeDelta				_time;
 	
@@ -44,7 +45,7 @@ MA 02110-1301, USA.
 	BOOL					_justSwitched;
 }
 
-+ (id) flasherWithDictionary:(NSDictionary *)dictionary;
++ (instancetype) flasherWithDictionary:(NSDictionary *)dictionary;
 - (id) initWithDictionary:(NSDictionary *)dictionary;
 
 - (BOOL) isActive;
