@@ -3,7 +3,7 @@
 OOPlanetEntity.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -33,6 +33,7 @@ MA 02110-1301, USA.
 #import "AI.h"
 #import "Universe.h"
 #import "ShipEntity.h"
+#import "PlayerEntity.h"
 #import "ShipEntityAI.h"
 #import "OOCharacter.h"
 
@@ -44,6 +45,7 @@ MA 02110-1301, USA.
 #import "OOPlanetTextureGenerator.h"
 #import "OOSingleTextureMaterial.h"
 #import "OOShaderMaterial.h"
+#import "OOEntityFilterPredicate.h"
 
 
 @interface OOPlanetEntity (Private)
@@ -68,6 +70,9 @@ MA 02110-1301, USA.
 }
 
 
+const double kMesosphere = 10.0 * ATMOSPHERE_DEPTH;	// atmosphere effect starts at 10x the height of the clouds
+
+
 - (id) initFromDictionary:(NSDictionary *)dict withAtmosphere:(BOOL)atmosphere andSeed:(Random_Seed)seed
 {
 	if (dict == nil)  dict = [NSDictionary dictionary];
@@ -93,12 +98,14 @@ MA 02110-1301, USA.
 	[planetInfo autorelease];
 	
 	int radius_km = [dict oo_intForKey:KEY_RADIUS defaultValue:[planetInfo oo_intForKey:KEY_RADIUS]];
+	collision_radius = radius_km * 10.0;	// Scale down by a factor of 100
 	OOTechLevelID techLevel = [dict oo_intForKey:KEY_TECHLEVEL defaultValue:[planetInfo oo_intForKey:KEY_TECHLEVEL]];
 	
 	if (techLevel > 14)  techLevel = 14;
 	_shuttlesOnGround = 1 + techLevel / 2;
 	_shuttleLaunchInterval = 3600.0 / (double)_shuttlesOnGround;	// All are launched in one hour.
-	_lastLaunchTime = 30.0 - _shuttleLaunchInterval;				// debug - launch 30s after player enters universe	 FIXME: is 0 the correct non-debug value?
+	_lastLaunchTime = [UNIVERSE getTime] + 30.0 - _shuttleLaunchInterval;	// launch 30s after player enters universe.
+																			// make delay > 0 to allow scripts adding a station nearby.
 	
 	int percent_land = [planetInfo oo_intForKey:@"percent_land" defaultValue:24 + (gen_rnd_number() % 48)];
 	[planetInfo setObject:[NSNumber numberWithFloat:0.01 * percent_land] forKey:@"land_fraction"];
@@ -112,6 +119,8 @@ MA 02110-1301, USA.
 	[planetInfo setObject:[NSValue valueWithBytes:&planetNoiseSeed objCType:@encode(RANROTSeed)] forKey:@"noise_map_seed"];
 	[self setUpLandParametersWithSourceInfo:dict targetInfo:planetInfo];
 	
+	_airColor = nil;	// default to no air
+	
 #if NEW_ATMOSPHERE
 	if (atmosphere)
 	{
@@ -121,31 +130,37 @@ MA 02110-1301, USA.
 		percent_land = 100 - [dict oo_intForKey:@"percent_cloud" defaultValue:100 - (3 + (gen_rnd_number() & 31)+(gen_rnd_number() & 31))];
 		[planetInfo setObject:[NSNumber numberWithFloat:0.01 * percent_land] forKey:@"cloud_fraction"];
 		[self setUpAtmosphereParametersWithSourceInfo:dict targetInfo:planetInfo];
-		
-		_materialParameters = [planetInfo dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"cloud_fraction", @"air_color", @"polar_air_color", @"cloud_color", @"polar_cloud_color", @"cloud_alpha",
-																								@"land_fraction", @"land_color", @"sea_color", @"polar_land_color", @"polar_sea_color", @"noise_map_seed", @"economy", nil]];
+		// planetInfo now contains a valid air_color
+		_airColor = [planetInfo objectForKey:@"air_color"];
+		//OOLog (@"kaks",@" translated air colour:%@ cloud colour:%@ polar cloud color:%@", [_airColor rgbaDescription],[(OOColor *)[planetInfo objectForKey:@"cloud_color"] rgbaDescription],[(OOColor *)[planetInfo objectForKey:@"polar_cloud_color"] rgbaDescription]);
+
+		_materialParameters = [planetInfo dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"cloud_fraction", @"air_color",  @"cloud_color", @"polar_cloud_color", @"cloud_alpha",
+															@"land_fraction", @"land_color", @"sea_color", @"polar_land_color", @"polar_sea_color", @"noise_map_seed", @"economy", nil]];
 	}
 	else
 #else
-	// differentiate between normal planets and moons.
+	// NEW_ATMOSPHERE is 0? still differentiate between normal planets and moons.
 	if (atmosphere)
 	{
 		_atmosphereDrawable = [[OOPlanetDrawable atmosphereWithRadius:collision_radius + ATMOSPHERE_DEPTH] retain];
+		_airColor = [[OOColor colorWithRed:0.8f green:0.8f blue:0.9f alpha:1.0f] retain];
 	}
+	if (YES) // create _materialParameters when NEW_ATMOSPHERE is set to 0
 #endif
-	
-	_materialParameters = [planetInfo dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"land_fraction", @"land_color", @"sea_color", @"polar_land_color", @"polar_sea_color", @"noise_map_seed", @"economy", nil]];
+	{
+		_materialParameters = [planetInfo dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"land_fraction", @"land_color", @"sea_color", @"polar_land_color", @"polar_sea_color", @"noise_map_seed", @"economy", nil]];
+	}
 	[_materialParameters retain];
 	
+	_mesopause2 = (atmosphere) ? (kMesosphere + collision_radius) * (kMesosphere + collision_radius) : 0.0;
+	
 	NSString *textureName = [dict oo_stringForKey:@"texture"];
 	[self setUpPlanetFromTexture:textureName];
-	
-	collision_radius = radius_km * 10.0;	// Scale down by a factor of 100
-	_rotationAxis = kBasisYVector;
-	orientation = (Quaternion){ M_SQRT1_2, M_SQRT1_2, 0, 0 };	// do we want to do something more interesting here?
-										// EW: NO, setting orientation should be handled by the code that adds the planet, not by planetEntity itself.
-										// just start with a default value.
-	[_planetDrawable setRadius:collision_radius];
+	[_planetDrawable setRadius:collision_radius];	
+		
+	// Orientation should be handled by the code that calls this planetEntity. Starting with a default value anyway.
+	orientation = (Quaternion){ M_SQRT1_2, M_SQRT1_2, 0, 0 };
+	_rotationAxis = vector_up_from_quaternion(orientation);
 	
 	// set speed of rotation.
 	if ([dict objectForKey:@"rotational_velocity"])
@@ -164,6 +179,10 @@ MA 02110-1301, USA.
 	setRandomSeed(savedRndSeed);
 	RANROTSetFullSeed(savedRanrotSeed);
 	
+	// rotate planet based on current time, needs to be done here - backported from PlanetEntity.
+	int		deltaT = floor(fmod([PLAYER clockTimeAdjusted], 86400));
+	quaternion_rotate_about_axis(&orientation, _rotationAxis, _rotationalVelocity * deltaT);
+	
 	[self setStatus:STATUS_ACTIVE];
 	
 	return self;
@@ -201,7 +220,7 @@ static Vector HSBColorWithColor(OOColor *color)
 
 static OOColor *ColorWithHSBColor(Vector c)
 {
-	return [OOColor colorWithCalibratedHue:c.x saturation:c.y brightness:c.z alpha:1.0];
+	return [OOColor colorWithHue:c.x saturation:c.y brightness:c.z alpha:1.0];
 }
 
 
@@ -226,29 +245,37 @@ static OOColor *ColorWithHSBColor(Vector c)
 		gen_rnd_number();
 	}
 	
-//	float polarColorFactor = [sourceInfo oo_floatForKey:@"polar_color_factor" defaultValue:0.5];
-	
 	Vector	landHSB, seaHSB, landPolarHSB, seaPolarHSB;
 	OOColor	*color;
 	
 	landHSB = RandomHSBColor();
-	do
-	{
-		seaHSB = RandomHSBColor();
-	}
-	while (dot_product(landHSB, seaHSB) > .80); // make sure land and sea colors differ significantly
 	
 	if (!isAtmosphere)
 	{
-		// possibly get landHSB and seaHSB from planetinfo.plist entry
-		ScanVectorFromString([sourceInfo oo_stringForKey:@"land_hsb_color"], &landHSB);
-		ScanVectorFromString([sourceInfo oo_stringForKey:@"sea_hsb_color"], &seaHSB);
+		do
+		{
+			seaHSB = RandomHSBColor();
+		}
+		while (dot_product(landHSB, seaHSB) > .80); // make sure land and sea colors differ significantly
+		
+		// saturation bias - avoids really grey oceans
+		if (seaHSB.y < 0.22f) seaHSB.y = seaHSB.y * 0.3f + 0.2f;
+		// brightness bias - avoids really bright landmasses
+		if (landHSB.z > 0.66f) landHSB.z = 0.66f;
+		
+		// planetinfo.plist overrides
+		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"land_color"]];
+		if (color != nil) landHSB = HSBColorWithColor(color);
+		else ScanVectorFromString([sourceInfo oo_stringForKey:@"land_hsb_color"], &landHSB);
+		
+		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"sea_color"]];
+		if (color != nil) seaHSB = HSBColorWithColor(color);
+		else ScanVectorFromString([sourceInfo oo_stringForKey:@"sea_hsb_color"], &seaHSB);
 		
 		// polar areas are brighter but have less colour (closer to white)
 		landPolarHSB = LighterHSBColor(landHSB);
 		seaPolarHSB = LighterHSBColor(seaHSB);
 		
-		// TODO: use all dictionary overrides
 		[targetInfo setObject:ColorWithHSBColor(landHSB) forKey:@"land_color"];
 		[targetInfo setObject:ColorWithHSBColor(seaHSB) forKey:@"sea_color"];
 		[targetInfo setObject:ColorWithHSBColor(landPolarHSB) forKey:@"polar_land_color"];
@@ -256,27 +283,31 @@ static OOColor *ColorWithHSBColor(Vector c)
 	}
 	else
 	{
+		landHSB = RandomHSBColor();	// NB: randomcolor is called twice to make the cloud colour similar to the old one.
+
+		// add a cloud_color tinge to sky blue({0.66, 0.3, 1}).
+		seaHSB = vector_add(landHSB,((Vector){1.333, 0.6, 2}));	// 1 part cloud, 2 parts sky blue
+		scale_vector(&seaHSB, 0.333);
+				
 		float cloudAlpha = OOClamp_0_1_f([sourceInfo oo_floatForKey:@"cloud_alpha" defaultValue:1.0f]);
 		[targetInfo setObject:[NSNumber numberWithFloat:cloudAlpha] forKey:@"cloud_alpha"];
 		
+		// planetinfo overrides
+		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"atmosphere_color"]];
+		if (color != nil) seaHSB = HSBColorWithColor(color);
 		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"cloud_color"]];
 		if (color != nil) landHSB = HSBColorWithColor(color);
-		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"clear_sky_color"]];
-		if (color != nil) seaHSB = HSBColorWithColor(color);
-
-		// polar areas are brighter but have less colour (closer to white)
-		landPolarHSB = LighterHSBColor(landHSB);
-		seaPolarHSB = LighterHSBColor(seaHSB);
+		
+		// polar areas: brighter, less saturation
+		landPolarHSB = vector_add(landHSB,LighterHSBColor(landHSB));
+		scale_vector(&landPolarHSB, 0.5);
 		
 		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"polar_cloud_color"]];
 		if (color != nil) landPolarHSB = HSBColorWithColor(color);
-		color = [OOColor colorWithDescription:[sourceInfo objectForKey:@"polar_clear_sky_color"]];
-		if (color != nil) seaPolarHSB = HSBColorWithColor(color);
 		
-		[targetInfo setObject:ColorWithHSBColor(landHSB) forKey:@"air_color"];
-		[targetInfo setObject:ColorWithHSBColor(seaHSB) forKey:@"cloud_color"];
-		[targetInfo setObject:ColorWithHSBColor(landPolarHSB) forKey:@"polar_air_color"];
-		[targetInfo setObject:ColorWithHSBColor(seaPolarHSB) forKey:@"polar_cloud_color"];
+		[targetInfo setObject:ColorWithHSBColor(seaHSB) forKey:@"air_color"];
+		[targetInfo setObject:ColorWithHSBColor(landHSB) forKey:@"cloud_color"];
+		[targetInfo setObject:ColorWithHSBColor(landPolarHSB) forKey:@"polar_cloud_color"];
 	}
 }
 
@@ -321,6 +352,7 @@ static OOColor *ColorWithHSBColor(Vector c)
 {
 	DESTROY(_planetDrawable);
 	DESTROY(_atmosphereDrawable);
+	//DESTROY(_airColor);	// this CTDs on loading savegames.. :(
 	DESTROY(_materialParameters);
 	
 	[super dealloc];
@@ -354,7 +386,7 @@ static OOColor *ColorWithHSBColor(Vector c)
 }
 
 
-- (id) miniatureVersion
+- (instancetype) miniatureVersion
 {
 	return [[[[self class] alloc] initAsMiniatureVersionOfPlanet:self] autorelease];
 }
@@ -364,32 +396,99 @@ static OOColor *ColorWithHSBColor(Vector c)
 {
 	[super update:delta_t];
 	
-	if (!_miniature)
+	if (EXPECT(!_miniature))
 	{
+		if (EXPECT_NOT(_atmosphereDrawable && cam_zero_distance < _mesopause2))
+		{
+			NSAssert(_airColor != nil, @"Expected a non-nil air colour for normal planet. Exiting.");
+			double		alt = (sqrt(cam_zero_distance) - collision_radius) / kMesosphere;
+			if (EXPECT_NOT(alt > 0 && alt <= 1.0))	// ensure aleph is clamped between 0 and 1
+			{
+				double	aleph = 1.0 - alt;
+				double	aleph2 = aleph * aleph;
+				
+				// night sky, reddish flash on entering the atmosphere, low light pollution otherwhise
+				OOColor	*mixColor = [OOColor colorWithRed:(EXPECT_NOT(alt > 0.98) ? 30.0f : 0.1f)
+													green:0.1f
+													 blue:0.1f
+													alpha:aleph];
+															  
+				// occlusion rate: .9 is 18 degrees after the terminus, where twilight ends.
+				// 1 is the terminus, 1.033 is 6 degrees before the terminus, where the sky begins to redden
+				double rate = ([PLAYER occlusionLevel] - 0.97)/0.06; // from 0.97 to 1.03
+
+				if (EXPECT(rate <= 1.0 && rate > 0.0))
+				{
+					mixColor = [mixColor blendedColorWithFraction:rate ofColor:_airColor];
+					// TODO: properly calculated pink sky - needs to depend on sun's angular size,
+					// and its angular height on the horizon.
+					/*
+					rate -= 0.7;
+					if (rate >= 0.0) // pink sky!
+					{
+						rate = 0.5 - (fabs(rate - 0.15) / 0.3);	// at most a 50% blend!
+						mixColor = [mixColor blendedColorWithFraction:rate ofColor:[OOColor colorWithRed:0.6
+																								   green:0.1
+																									blue:0.0
+																								   alpha:aleph]];
+					}
+					*/
+				}
+				else
+				{
+					if (PLAYER->isSunlit && _airColor != nil) mixColor = _airColor;
+				}
+				[UNIVERSE setSkyColorRed:[mixColor redComponent] * aleph2
+								   green:[mixColor greenComponent] * aleph2
+									blue:[mixColor blueComponent] * aleph
+								   alpha:aleph];
+				[_atmosphereDrawable setRadius:collision_radius + (ATMOSPHERE_DEPTH * alt)];
+			}
+		}
+		else if (EXPECT_NOT([_atmosphereDrawable radius] < collision_radius + ATMOSPHERE_DEPTH))
+		{
+			[_atmosphereDrawable setRadius:collision_radius + ATMOSPHERE_DEPTH];
+		}
+		
 		double time = [UNIVERSE getTime];
 		
 		if (_shuttlesOnGround > 0 && time > _lastLaunchTime + _shuttleLaunchInterval)  [self launchShuttle];
-		
-		// FIXME: update atmosphere
 	}
 	
 	quaternion_rotate_about_axis(&orientation, _rotationAxis, _rotationalVelocity * delta_t);
 
 	[self orientationChanged];
 	
-	// FIXME: update atmosphere
+	// FIXME: update atmosphere rotation
 }
 
 
-- (void) drawEntity:(BOOL)immediate :(BOOL)translucent
+- (BOOL) isFinishedLoading
+{
+	OOMaterial *material = [self material];
+	if (material != nil && ![material isFinishedLoading])  return NO;
+	material = [self atmosphereMaterial];
+	if (material != nil && ![material isFinishedLoading])  return NO;
+	return YES;
+}
+
+
+- (void) drawImmediate:(bool)immediate translucent:(bool)translucent
 {
 	if (translucent || [UNIVERSE breakPatternHide])   return; // DON'T DRAW
+	if (_miniature && ![self isFinishedLoading])  return; // For responsiveness, don't block to draw as miniature.
 	
-	if ([UNIVERSE wireframeGraphics])  GLDebugWireframeModeOn();
+	if (![UNIVERSE viewFrustumIntersectsSphereAt:position withRadius:([self radius] + ATMOSPHERE_DEPTH)])
+	{
+		// Don't draw
+		return;
+	}
+	
+	if ([UNIVERSE wireframeGraphics])  OOGLWireframeModeOn();
 	
 	if (!_miniature)
 	{
-		[_planetDrawable calculateLevelOfDetailForViewDistance:zero_distance];
+		[_planetDrawable calculateLevelOfDetailForViewDistance:cam_zero_distance];
 		[_atmosphereDrawable setLevelOfDetail:[_planetDrawable levelOfDetail]];
 	}
 	
@@ -401,7 +500,40 @@ static OOColor *ColorWithHSBColor(Vector c)
 	}
 #endif
 	
-	if ([UNIVERSE wireframeGraphics])  GLDebugWireframeModeOff();
+	if ([UNIVERSE wireframeGraphics])  OOGLWireframeModeOff();
+}
+
+
+- (BOOL) checkCloseCollisionWith:(Entity *)other
+{
+	if (!other)
+		return NO;
+	if (other->isShip)
+	{
+		ShipEntity *ship = (ShipEntity *)other;
+		if ([ship behaviour] == BEHAVIOUR_LAND_ON_PLANET)
+		{
+			return NO;
+		}
+	}
+	
+	return YES;
+}
+
+
+- (BOOL) planetHasStation
+{
+	// find the nearest station...
+	ShipEntity	*station =  nil;
+	station = [UNIVERSE nearestShipMatchingPredicate:IsStationPredicate
+										   parameter:nil
+									relativeToEntity:self];
+	
+	if (station && distance([station position], position) < 4 * collision_radius) // there is a station in range.
+	{
+		return YES;
+	}
+	return NO;
 }
 
 
@@ -409,6 +541,13 @@ static OOColor *ColorWithHSBColor(Vector c)
 {
 	if (_shuttlesOnGround == 0)  return;
 	
+	if (self != [UNIVERSE planet] && ![self planetHasStation])
+	{
+		// don't launch shuttles when no station is nearby.
+		_shuttlesOnGround = 0;
+		return;
+	}
+	
 	Quaternion  q1;
 	quaternion_set_random(&q1);
 	float start_distance = collision_radius + 125.0f;
@@ -457,6 +596,22 @@ static OOColor *ColorWithHSBColor(Vector c)
 }
 
 
+- (double) rotationalVelocity
+{
+	return _rotationalVelocity;
+}
+
+
+- (void) setRotationalVelocity:(double) v
+{
+	if ([self hasAtmosphere])
+	{
+		// FIXME: change atmosphere rotation speed proportionally
+	}
+	_rotationalVelocity = v;
+}
+
+
 - (BOOL) hasAtmosphere
 {
 	return _atmosphereDrawable != nil;
@@ -492,6 +647,7 @@ static OOColor *ColorWithHSBColor(Vector c)
 		diffuseMap = [OOTexture textureWithConfiguration:spec];
 		if (diffuseMap == nil)  return;		// OOTexture will have logged a file-not-found warning.
 		if (shadersOn)  macros = [materialDefaults oo_dictionaryForKey:isMoon ? @"moon-customized-macros" : @"planet-customized-macros"];
+		else textureName = @"dynamic";
 	}
 	else
 	{
@@ -519,7 +675,6 @@ static OOColor *ColorWithHSBColor(Vector c)
 	OOMaterial *material = nil;
 	
 #if OO_SHADERS
-
 	if (shadersOn)
 	{
 		NSMutableDictionary *config = [[[materialDefaults oo_dictionaryForKey:@"planet-material"] mutableCopy] autorelease];
