@@ -6,7 +6,7 @@ Droppings of a Q-mine, or one of its victims.
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,7 +35,7 @@ MA 02110-1301, USA.
 	GLfloat				_color[4];
 }
 
-+ (id) quiriumCascadeFromShip:(ShipEntity *)ship;
++ (instancetype) quiriumCascadeFromShip:(ShipEntity *)ship;
 
 @end
 
