@@ -3,7 +3,7 @@
 PlanetEntity.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -47,7 +47,7 @@ MA 02110-1301, USA.
 #define kOOLogUnconvertedNSLog @"unclassified.PlanetEntity"
 
 
-#if !OOLITE_MAC_OS_X || !OOLITE_LEOPARD
+#if !OOLITE_MAC_OS_X
 #define NSIntegerMapKeyCallBacks	NSIntMapKeyCallBacks
 #define NSIntegerMapValueCallBacks	NSIntMapValueCallBacks
 #endif
@@ -56,7 +56,7 @@ MA 02110-1301, USA.
 // straight C
 static Vector base_vertex_array[MAX_PLANET_VERTICES];
 static int base_terrain_array[MAX_PLANET_VERTICES];
-static OOUInteger next_free_vertex;
+static unsigned next_free_vertex;
 static NSMapTable *sEdgeToVertex;
 
 static int n_triangles[MAX_SUBDIVIDE];
@@ -76,7 +76,7 @@ static GLfloat	texture_uv_array[MAX_PLANET_VERTICES * 2];
 
 
 - (void) initialiseBaseTerrainArray:(int) percent_land;
-- (void) paintVertex:(int) vi :(int) seed;
+- (void) paintVertex:(unsigned) vi :(int) seed;
 - (void) scaleVertices;
 
 - (id) initAsAtmosphereForPlanet:(PlanetEntity *)planet dictionary:(NSDictionary *)dict;
@@ -241,25 +241,25 @@ static const BaseFace kTexturedFaces[][3] =
 	float		cloudAlpha;
 	
 	
-	clearSkyColor = [OOColor colorWithDescription:[dict objectForKey:@"clear_sky_color"]];
+	clearSkyColor = [OOColor colorWithDescription:[dict objectForKey:@"atmosphere_color"]];
 	cloudColor = [OOColor colorWithDescription:[dict objectForKey:@"cloud_color"]];
-	polarClearSkyColor = [OOColor colorWithDescription:[dict objectForKey:@"polar_clear_sky_color"]];
+	polarClearSkyColor = [OOColor colorWithDescription:[dict objectForKey:@"polar_atmosphere_color"]];
 	polarCloudColor = [OOColor colorWithDescription:[dict objectForKey:@"polar_cloud_color"]];
 	cloudAlpha = OOClamp_0_1_f([dict oo_floatForKey:@"cloud_alpha" defaultValue:1.0]);
 	
 	if (clearSkyColor != nil)
 	{
-		[clearSkyColor getGLRed:&amb_land[0] green:&amb_land[1] blue:&amb_land[2] alpha:&amb_land[3]];
+		[clearSkyColor getRed:&amb_land[0] green:&amb_land[1] blue:&amb_land[2] alpha:&amb_land[3]];
 	}
 	
 	if (cloudColor != nil)
 	{
-		[cloudColor getGLRed:&amb_sea[0] green:&amb_sea[1] blue:&amb_sea[2] alpha:&amb_sea[3]];
+		[cloudColor getRed:&amb_sea[0] green:&amb_sea[1] blue:&amb_sea[2] alpha:&amb_sea[3]];
 	}
 	
 	if (polarClearSkyColor != nil)
 	{
-		[polarClearSkyColor getGLRed:&amb_polar_land[0] green:&amb_polar_land[1] blue:&amb_polar_land[2] alpha:&amb_polar_land[3]];
+		[polarClearSkyColor getRed:&amb_polar_land[0] green:&amb_polar_land[1] blue:&amb_polar_land[2] alpha:&amb_polar_land[3]];
 	}
 	else if (clearSkyColor != nil)
 	{
@@ -269,7 +269,7 @@ static const BaseFace kTexturedFaces[][3] =
 	
 	if (polarCloudColor != nil)
 	{
-		[polarCloudColor getGLRed:&amb_polar_sea[0] green:&amb_polar_sea[1] blue:&amb_polar_sea[2] alpha:&amb_polar_sea[3]];
+		[polarCloudColor getRed:&amb_polar_sea[0] green:&amb_polar_sea[1] blue:&amb_polar_sea[2] alpha:&amb_polar_sea[3]];
 	}
 	else if (cloudColor != nil)
 	{
@@ -291,7 +291,7 @@ static const BaseFace kTexturedFaces[][3] =
 	{
 		RANROTSeed ranrotSavedSeed = RANROTGetFullSeed();
 		RNG_Seed saved_seed = currentRandomSeed();
-		cloudColor = [OOColor colorWithCalibratedRed: amb_sea[0] green: amb_sea[1] blue: amb_sea[2] alpha: amb_sea[3]];
+		cloudColor = [OOColor colorWithRed: amb_sea[0] green: amb_sea[1] blue: amb_sea[2] alpha: amb_sea[3]];
 		float cloud_bias = -0.01 * (float)percent_land;
 		float cloud_impress = 1.0 - cloud_bias;
 		
@@ -346,7 +346,7 @@ static const BaseFace kTexturedFaces[][3] =
 	[self scaleVertices];
 
 	// set speed of rotation
-	rotational_velocity = [dict oo_floatForKey:@"atmosphere_rotational_velocity" defaultValue:0.01f + 0.02f * randf()]; // 0.01 .. 0.03 avr 0.02
+	rotational_velocity = [dict oo_floatForKey:@"atmosphere_rotational_velocity" defaultValue:[planet rotationalVelocity]*(0.9+(randf()*0.2))]; // 90-110% of planet rotation speed
 	
 	root_planet = planet;
 	
@@ -406,11 +406,13 @@ static const BaseFace kTexturedFaces[][3] =
 		{
 			[self loadTexture:textureSpec];
 		}
-		if (textureSpec == nil && !procGen && !atmo)
+
+// CIM: nothing actually uses textureSpec after the assignment below, so skip it
+/*		if (textureSpec == nil && !procGen && !atmo)
 		{
 			// Moons use metal.png by default.
 			textureSpec = OOTextureSpecFromObject(@"metal.png", nil);
-		}
+			} */
 		
 		NSString *seedStr = [dict oo_stringForKey:@"seed"];
 		if (seedStr != nil)
@@ -439,7 +441,7 @@ static const BaseFace kTexturedFaces[][3] =
 	last_launch_time = 0.0;
 	shuttle_launch_interval = 3600.0 / shuttles_on_ground; // all are launched in an hour
 
-	last_launch_time = 30.0 - shuttle_launch_interval;   // debug - launch 30s after player enters universe
+	last_launch_time = [UNIVERSE getTime] + 30.0 - shuttle_launch_interval;   // debug - launch 30s after player enters universe
 
 	collision_radius = radius_km * 10.0; // scale down by a factor of 100 !
 	
@@ -494,10 +496,10 @@ static const BaseFace kTexturedFaces[][3] =
 		land_polar_hsb.x = land_hsb.x;  land_polar_hsb.y = (land_hsb.y / 4.0);  land_polar_hsb.z = 1.0 - (land_hsb.z / 10.0);
 		sea_polar_hsb.x = sea_hsb.x;  sea_polar_hsb.y = (sea_hsb.y / 4.0);  sea_polar_hsb.z = 1.0 - (sea_hsb.z / 10.0);
 		
-		OOColor *amb_land_color = [OOColor colorWithCalibratedHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0];
-		OOColor *amb_sea_color = [OOColor colorWithCalibratedHue:sea_hsb.x saturation:sea_hsb.y brightness:sea_hsb.z alpha:1.0];
-		OOColor *amb_polar_land_color = [OOColor colorWithCalibratedHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0];
-		OOColor *amb_polar_sea_color = [OOColor colorWithCalibratedHue:sea_polar_hsb.x saturation:sea_polar_hsb.y brightness:sea_polar_hsb.z alpha:1.0];
+		OOColor *amb_land_color = [OOColor colorWithHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0];
+		OOColor *amb_sea_color = [OOColor colorWithHue:sea_hsb.x saturation:sea_hsb.y brightness:sea_hsb.z alpha:1.0];
+		OOColor *amb_polar_land_color = [OOColor colorWithHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0];
+		OOColor *amb_polar_sea_color = [OOColor colorWithHue:sea_polar_hsb.x saturation:sea_polar_hsb.y brightness:sea_polar_hsb.z alpha:1.0];
 		
 		amb_land[0] = [amb_land_color redComponent];
 		amb_land[1] = [amb_land_color blueComponent];
@@ -545,13 +547,13 @@ static const BaseFace kTexturedFaces[][3] =
 	}
 	else
 	{
-		rotational_velocity = [planetInfo oo_floatForKey:@"rotation_speed" defaultValue:0.005 * randf()]; // 0.0 .. 0.005 avr 0.0025
+		rotational_velocity = [planetInfo oo_floatForKey:@"rotation_speed" defaultValue:0.002 * (0.5+0.5*randf())]; // 0.001 .. 0.002 avr 0.0015
 		rotational_velocity *= [planetInfo oo_floatForKey:@"rotation_speed_factor" defaultValue:1.0f];
 	}
 
 	// do atmosphere
 	NSDictionary *atmoDict = dict;
-	if (_texture != nil)  atmoDict = [NSDictionary dictionaryWithObject:@"0" forKey:@"percent_cloud"];
+	if (_texture != nil)  atmoDict = 	[NSDictionary dictionaryWithObjectsAndKeys:@"0", @"percent_cloud", [NSNumber numberWithFloat:[planetInfo oo_floatForKey:@"cloud_alpha" defaultValue:1.0]], @"cloud_alpha", nil];
 	if (atmo)  atmosphere = [[PlanetEntity alloc] initAsAtmosphereForPlanet:self dictionary:atmoDict];
 	
 	setRandomSeed(saved_seed);
@@ -642,9 +644,8 @@ static const BaseFace kTexturedFaces[][3] =
 	if (other->isShip)
 	{
 		ShipEntity *ship = (ShipEntity *)other;
-		if ([ship isShuttle])
+		if ([ship behaviour] == BEHAVIOUR_LAND_ON_PLANET)
 		{
-			[ship landOnPlanet:self];
 			return NO;
 		}
 #ifndef NDEBUG
@@ -663,7 +664,7 @@ static const BaseFace kTexturedFaces[][3] =
 - (void) update:(OOTimeDelta) delta_t
 {
 	[super update:delta_t];
-	sqrt_zero_distance = sqrt(zero_distance);
+	sqrt_zero_distance = sqrt(cam_zero_distance);
 
 	switch (planet_type)
 	{
@@ -707,7 +708,6 @@ static const BaseFace kTexturedFaces[][3] =
 		case STELLAR_TYPE_ATMOSPHERE:
 			{
 				// atmospheric rotation
-				//quaternion_rotate_about_y(&orientation, rotational_velocity * delta_t);
 				quaternion_rotate_about_axis(&orientation, rotationAxis, rotational_velocity * delta_t);
 				[self orientationChanged];
 			}
@@ -750,9 +750,22 @@ static const BaseFace kTexturedFaces[][3] =
 
 
 // TODO: some translucent stuff is drawn in the opaque pass, which is Naughty.
-- (void) drawEntity:(BOOL)immediate :(BOOL)translucent
+- (void) drawImmediate:(bool)immediate translucent:(bool)translucent
 {
-	if ([UNIVERSE breakPatternHide] || translucent || immediate)   return; // DON'T DRAW
+	if ([UNIVERSE breakPatternHide] || translucent || immediate)  return;  // DON'T DRAW
+	GLfloat radWithAtmosphere = [self radius] + ATMOSPHERE_DEPTH;
+	if (radWithAtmosphere * radWithAtmosphere * 1.1 < cam_zero_distance)
+	{
+		// for some reason this check doesn't work when extremely close to
+		// the planet and with the horizon near the side of the frame (FP
+		// inaccuracy?)
+		if (![UNIVERSE viewFrustumIntersectsSphereAt:position withRadius:radWithAtmosphere])
+		{
+			// Don't draw
+			return;
+		}
+	}
+
 	[self drawUnconditionally];
 }
 
@@ -764,7 +777,7 @@ static const BaseFace kTexturedFaces[][3] =
 	double  drawFactor = [[UNIVERSE gameView] viewSize].width / 100.0;
 	double  drawRatio2 = drawFactor * collision_radius / sqrt_zero_distance; // equivalent to size on screen in pixels
 	
-	if (zero_distance > 0.0)
+	if (cam_zero_distance > 0.0)
 	{
 		subdivideLevel = 2 + floor(drawRatio2);
 		if (subdivideLevel > 4)  subdivideLevel = 4;
@@ -777,10 +790,11 @@ static const BaseFace kTexturedFaces[][3] =
 		
 	lastSubdivideLevel = subdivideLevel;	// record
 	
+	OOSetOpenGLState(OPENGL_STATE_OPAQUE);
 	OOGL(glPushAttrib(GL_ENABLE_BIT));
 	
-	OOGL(glEnable(GL_LIGHTING));
-	OOGL(glEnable(GL_LIGHT1));
+//	OOGL(glEnable(GL_LIGHTING));
+//	OOGL(glEnable(GL_LIGHT1));
 	GLfloat specular[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
 	OOGL(glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular));
 	OOGL(glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 0));
@@ -797,7 +811,7 @@ static const BaseFace kTexturedFaces[][3] =
 	
 	BOOL ignoreDepthBuffer = (planet_type == STELLAR_TYPE_ATMOSPHERE);
 	
-	if (zero_distance > collision_radius * collision_radius * 25) // is 'far away'
+	if (cam_zero_distance > collision_radius * collision_radius * 25) // is 'far away'
 	{
 		ignoreDepthBuffer |= YES;
 	}
@@ -812,8 +826,10 @@ static const BaseFace kTexturedFaces[][3] =
 				subdivideLevel = root_planet->lastSubdivideLevel;	// copy it from the planet (stops jerky LOD and such)
 			}
 			GLMultOOMatrix(rotMatrix);	// rotate the clouds!
+			OOGL(glEnable(GL_BLEND));
+//			OOGL(glDisable(GL_LIGHTING));
 			// Fall through.
-			
+
 		case STELLAR_TYPE_MOON:
 		case STELLAR_TYPE_NORMAL_PLANET:
 		case STELLAR_TYPE_MINIATURE:
@@ -823,7 +839,7 @@ static const BaseFace kTexturedFaces[][3] =
 				// Drop the detail level a bit, it still looks OK in wireframe and does not penalize
 				// the system that much.
 				subdivideLevel = 2;
-				GLDebugWireframeModeOn();
+				OOGLWireframeModeOn();
 			}
 			
 			{
@@ -840,7 +856,7 @@ static const BaseFace kTexturedFaces[][3] =
 					}
 					else
 					{
-						OOGL(glEnable(GL_TEXTURE_2D));
+//						OOGL(glEnable(GL_TEXTURE_2D));
 					}
 					OOGL(glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, mat1));
 					[_texture apply];
@@ -862,12 +878,11 @@ static const BaseFace kTexturedFaces[][3] =
 				OOGL(glColor4fv(mat1));
 				OOGL(glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, mat1));
 				
-				OOGL(glDisableClientState(GL_EDGE_FLAG_ARRAY));
 				OOGL(glEnableClientState(GL_COLOR_ARRAY));
 				OOGL(glColorPointer(4, GL_FLOAT, 0, vertexdata.color_array));
-				OOGL(glEnableClientState(GL_VERTEX_ARRAY));
+//				OOGL(glEnableClientState(GL_VERTEX_ARRAY));
 				OOGL(glVertexPointer(3, GL_FLOAT, 0, vertexdata.vertex_array));
-				OOGL(glEnableClientState(GL_NORMAL_ARRAY));
+//				OOGL(glEnableClientState(GL_NORMAL_ARRAY));
 				OOGL(glNormalPointer(GL_FLOAT, 0, vertexdata.normal_array));
 				
 				if (_texture != nil)
@@ -908,45 +923,60 @@ static const BaseFace kTexturedFaces[][3] =
 						OOGL(glEndList());
 					}
 				}
-				
+
 #if OO_TEXTURE_CUBE_MAP
 				if ([_texture isCubeMap])
 				{
 					OOGL(glDisable(GL_TEXTURE_CUBE_MAP));
+					OOGL(glEnable(GL_TEXTURE_2D));
 				}
 #endif
-				
-				if (atmosphere)
-				{
-					OOGL(glDisable(GL_DEPTH_TEST));
 
-					OOGL(glPopMatrix());	// get old draw matrix back
-					OOGL(glPushMatrix());	// and store it again
-					OOGL(glTranslatef(position.x,position.y,position.z)); // centre on the planet
-					// rotate
-					GLMultOOMatrix([atmosphere rotationMatrix]);
-					// draw atmosphere entity
-					[atmosphere drawEntity:NO :NO];
-				}
+			OOGL(glDisableClientState(GL_COLOR_ARRAY));
+			OOGL(glDisableClientState(GL_TEXTURE_COORD_ARRAY));
+			OOGL(glEnable(GL_DEPTH_TEST));
+
 			}
 			
 			//if ((gDebugFlags & DEBUG_WIREFRAME_GRAPHICS)
 			if ([UNIVERSE wireframeGraphics])
 			{
-				GLDebugWireframeModeOff();
+				OOGLWireframeModeOff();
 			}
+//			OOGL(glDisableClientState(GL_VERTEX_ARRAY));
+//			OOGL(glDisableClientState(GL_NORMAL_ARRAY));
 			
-			OOGL(glDisableClientState(GL_VERTEX_ARRAY));
-			OOGL(glDisableClientState(GL_NORMAL_ARRAY));
-			OOGL(glDisableClientState(GL_COLOR_ARRAY));
-			OOGL(glDisableClientState(GL_TEXTURE_COORD_ARRAY));
 			break;
 			
 		case STELLAR_TYPE_SUN:
 			break;
 	}
+
+	if (ignoreDepthBuffer)
+	{
+		OOGL(glEnable(GL_DEPTH_TEST));
+	}
+	OOGL(glEnable(GL_TEXTURE_2D));
+//	OOGL(glEnable(GL_LIGHTING));
+	OOGL(glDisable(GL_BLEND));
+
+
 	OOGL(glPopAttrib());
-	CheckOpenGLErrors(@"PlanetEntity after drawing %@", self);
+	OOVerifyOpenGLState();
+	OOCheckOpenGLErrors(@"PlanetEntity after drawing %@", self);
+
+	if (atmosphere)
+	{
+		OOGL(glPopMatrix());	// get old draw matrix back
+		OOGL(glPushMatrix());	// and store it again
+		OOGL(glTranslatef(position.x,position.y,position.z)); // centre on the planet
+		// rotate
+//		GLMultOOMatrix([atmosphere rotationMatrix]);
+		// draw atmosphere entity
+		[atmosphere drawImmediate:false translucent:false];
+	}
+
+
 }
 
 
@@ -990,13 +1020,13 @@ static const BaseFace kTexturedFaces[][3] =
 	
 	land_polar_hsb.x = land_hsb.x;  land_polar_hsb.y = (land_hsb.y / 5.0);  land_polar_hsb.z = 1.0 - (land_hsb.z / 10.0);
 	
-	amb_sea[0] = amb_land[0] = [[OOColor colorWithCalibratedHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0] redComponent];
-	amb_sea[1] = amb_land[1] = [[OOColor colorWithCalibratedHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0] blueComponent];
-	amb_sea[2] = amb_land[2] = [[OOColor colorWithCalibratedHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0] greenComponent];
+	amb_sea[0] = amb_land[0] = [[OOColor colorWithHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0] redComponent];
+	amb_sea[1] = amb_land[1] = [[OOColor colorWithHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0] blueComponent];
+	amb_sea[2] = amb_land[2] = [[OOColor colorWithHue:land_hsb.x saturation:land_hsb.y brightness:land_hsb.z alpha:1.0] greenComponent];
 	amb_sea[3] = amb_land[3] = 1.0;
-	amb_polar_sea[0] =amb_polar_land[0] = [[OOColor colorWithCalibratedHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0] redComponent];
-	amb_polar_sea[1] =amb_polar_land[1] = [[OOColor colorWithCalibratedHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0] blueComponent];
-	amb_polar_sea[2] = amb_polar_land[2] = [[OOColor colorWithCalibratedHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0] greenComponent];
+	amb_polar_sea[0] =amb_polar_land[0] = [[OOColor colorWithHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0] redComponent];
+	amb_polar_sea[1] =amb_polar_land[1] = [[OOColor colorWithHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0] blueComponent];
+	amb_polar_sea[2] = amb_polar_land[2] = [[OOColor colorWithHue:land_polar_hsb.x saturation:land_polar_hsb.y brightness:land_polar_hsb.z alpha:1.0] greenComponent];
 	amb_polar_sea[3] =amb_polar_land[3] = 1.0;
 }
 
@@ -1008,7 +1038,7 @@ static const BaseFace kTexturedFaces[][3] =
 	[self loadTexture:OOTextureSpecFromObject(fileName, nil)];
 	[self deleteDisplayLists];
 	
-	OOUInteger i;
+	unsigned i;
 	[self setUseTexturedModel:YES];
 	
 	// recolour main planet according to "texture_hsb_color"
@@ -1024,7 +1054,13 @@ static const BaseFace kTexturedFaces[][3] =
 	
 	[self scaleVertices];
 	
-	NSDictionary *atmo_dictionary = [NSDictionary dictionaryWithObject:@"0" forKey:@"percent_cloud"];
+	GLfloat oldCloudAlpha = 0.0;
+	if (atmosphere)
+	{
+		oldCloudAlpha = [atmosphere amb_sea][3] / CLOUD_ALPHA;
+	}
+
+	NSDictionary *atmo_dictionary = [NSDictionary dictionaryWithObjectsAndKeys:@"0", @"percent_cloud", [NSNumber numberWithFloat:oldCloudAlpha], @"cloud_alpha", nil];
 	[atmosphere autorelease];
 	atmosphere = [self hasAtmosphere] ? [[PlanetEntity alloc] initAsAtmosphereForPlanet:self dictionary:atmo_dictionary] : nil;
 	
@@ -1082,7 +1118,7 @@ static const BaseFace kTexturedFaces[][3] =
 }
 
 
-- (void) setRadius:(double) rad
+- (void) setRadius:(GLfloat) rad
 {
 	collision_radius = rad;
 }
@@ -1281,7 +1317,7 @@ static unsigned baseVertexIndexForEdge(GLushort va, GLushort vb, BOOL textured)
 	if (num != 0)
 	{
 		// Overall cache hit rate is just over 83 %.
-		return num - 1;
+		return (unsigned)num - 1;
 	}
 	else
 	{
@@ -1385,7 +1421,7 @@ static unsigned baseVertexIndexForEdge(GLushort va, GLushort vb, BOOL textured)
 }
 
 
-- (void) paintVertex:(int) vi :(int) seed
+- (void) paintVertex:(unsigned) vi :(int) seed
 {
 	BOOL isTextured = _texture != nil;
 	
@@ -1434,7 +1470,7 @@ static unsigned baseVertexIndexForEdge(GLushort va, GLushort vb, BOOL textured)
 
 - (void) scaleVertices
 {
-	OOUInteger vi;
+	NSUInteger vi;
 	for (vi = 0; vi < next_free_vertex; vi++)
 	{
 		Vector	v = base_vertex_array[vi];
@@ -1534,6 +1570,7 @@ static unsigned baseVertexIndexForEdge(GLushort va, GLushort vb, BOOL textured)
 	unsigned char *data;
 	GLuint width, height;
 	
+	fillRanNoiseBuffer();
 	if (![TextureStore getCloudTextureNameFor:cloudColor
 											 :cloud_impress
 											 :cloud_bias
