@@ -3,7 +3,7 @@
 PlayerEntityStickMapper.h
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,6 +29,18 @@ MA 02110-1301, USA.
 #import "OOCollectionExtractors.h"
 
 
+@interface PlayerEntity (StickMapperInternal)
+
+- (void) removeFunction:(int)selFunctionIdx;
+- (NSArray *)stickFunctionList;
+- (void)displayFunctionList:(GuiDisplayGen *)gui
+					   skip:(NSUInteger) skip;
+- (NSString *)describeStickDict:(NSDictionary *)stickDict;
+- (NSString *)hwToString:(int)hwFlags;
+
+@end
+
+
 @implementation PlayerEntity (StickMapper)
 
 - (void) setGuiToStickMapperScreen:(unsigned)skip
@@ -57,7 +69,7 @@ MA 02110-1301, USA.
 			   forRow:i + GUI_ROW_STICKNAME];
 	}
 	
-	[self displayFunctionList: gui skip: skip];
+	[self displayFunctionList:gui skip:skip];
 	
 	[gui setArray:[NSArray arrayWithObject:@"Select a function and press Enter to modify or 'u' to unset."]
 		   forRow:GUI_ROW_INSTRUCT];
@@ -82,7 +94,7 @@ MA 02110-1301, USA.
 		{
 			[stickHandler clearCallback];
 			[gui setArray: [NSArray arrayWithObjects:
-							[NSString stringWithString: @"Function setting aborted."], nil]
+							@"Function setting aborted.", nil]
 				   forRow: GUI_ROW_INSTRUCT];
 			waitingForStickCallback=NO;
 		}
@@ -217,9 +229,9 @@ MA 02110-1301, USA.
 - (void) removeFunction:(int)idx
 {
 	OOJoystickManager	*stickHandler = [OOJoystickManager sharedStickHandler];
-	NSDictionary	*entry = [stickFunctions objectAtIndex:idx];
-	NSNumber		*butfunc = [entry objectForKey:KEY_BUTTONFN];
-	NSNumber		*axfunc = [entry objectForKey:KEY_AXISFN];
+	NSDictionary		*entry = [stickFunctions objectAtIndex:idx];
+	NSNumber			*butfunc = [entry objectForKey:KEY_BUTTONFN];
+	NSNumber			*axfunc = [entry objectForKey:KEY_AXISFN];
 	selFunctionIdx = idx;
 	
 	// Some things can have either axis or buttons - make sure we clear
@@ -244,11 +256,10 @@ MA 02110-1301, USA.
 
 
 - (void) displayFunctionList:(GuiDisplayGen *)gui
-						skip:(unsigned) skip
+						skip:(NSUInteger)skip
 {
 	OOJoystickManager	*stickHandler = [OOJoystickManager sharedStickHandler];
 	
-	unsigned i;
 	[gui setColor:[OOColor greenColor] forRow: GUI_ROW_HEADING];
 	[gui setArray:[NSArray arrayWithObjects:
 				   @"Function", @"Assigned to", @"Type", nil]
@@ -256,13 +267,13 @@ MA 02110-1301, USA.
 	
 	if(!stickFunctions)
 	{
-		stickFunctions = [self getStickFunctionList];
+		stickFunctions = [[self stickFunctionList] retain];
 	}
 	NSDictionary *assignedAxes = [stickHandler axisFunctions];
 	NSDictionary *assignedButs = [stickHandler buttonFunctions];
 	
-	unsigned n_functions = [stickFunctions count];
-	int n_rows, start_row, previous = 0;
+	NSUInteger i, n_functions = [stickFunctions count];
+	NSInteger n_rows, start_row, previous = 0;
 	
 	if (skip >= n_functions)
 		skip = n_functions - 1;
@@ -295,7 +306,7 @@ MA 02110-1301, USA.
 		{
 			[gui setColor:[OOColor greenColor] forRow:GUI_ROW_FUNCSTART];
 			[gui setArray:[NSArray arrayWithObjects:DESC(@"gui-back"), @" <-- ", nil] forRow:GUI_ROW_FUNCSTART];
-			[gui setKey:[NSString stringWithFormat:@"More:%d", previous] forRow:GUI_ROW_FUNCSTART];
+			[gui setKey:[NSString stringWithFormat:@"More:%ld", previous] forRow:GUI_ROW_FUNCSTART];
 		}
 		
 		for(i=0; i < (n_functions - skip) && (int)i < n_rows; i++)
@@ -309,17 +320,17 @@ MA 02110-1301, USA.
 			switch(allowable)
 			{
 				case HW_AXIS:
-					allowedThings=[NSString stringWithString: @"Axis"];
+					allowedThings=@"Axis";
 					assignment=[self describeStickDict:
 								[assignedAxes objectForKey: axFuncKey]];
 					break;
 				case HW_BUTTON:
-					allowedThings=[NSString stringWithString: @"Button"];
+					allowedThings=@"Button";
 					assignment=[self describeStickDict:
 								[assignedButs objectForKey: butFuncKey]];
 					break;
 				default:
-					allowedThings=[NSString stringWithString: @"Axis/Button"];
+					allowedThings=@"Axis/Button";
 					
 					// axis has priority
 					assignment=[self describeStickDict:
@@ -330,20 +341,22 @@ MA 02110-1301, USA.
 			}
 			
 			// Find out what's assigned for this function currently.
-			if(!assignment)
-				assignment=[NSString stringWithString: @"   -   "];
+			if (assignment == nil)
+			{
+				assignment = @"   -   ";
+			}
 			
 			[gui setArray: [NSArray arrayWithObjects: 
 							[entry objectForKey: KEY_GUIDESC], assignment, allowedThings, nil]
 				   forRow: i + start_row];
 			//[gui setKey: GUI_KEY_OK forRow: i + start_row];
-			[gui setKey: [NSString stringWithFormat: @"Index:%d", i + skip] forRow: i + start_row];
+			[gui setKey: [NSString stringWithFormat: @"Index:%ld", i + skip] forRow: i + start_row];
 		}
 		if (i < n_functions - skip)
 		{
 			[gui setColor: [OOColor greenColor] forRow: start_row + i];
 			[gui setArray: [NSArray arrayWithObjects: DESC(@"gui-more"), @" --> ", nil] forRow: start_row + i];
-			[gui setKey: [NSString stringWithFormat: @"More:%d", n_rows + skip] forRow: start_row + i];
+			[gui setKey: [NSString stringWithFormat: @"More:%ld", n_rows + skip] forRow: start_row + i];
 			i++;
 		}
 		
@@ -391,13 +404,13 @@ MA 02110-1301, USA.
 	switch(hwFlags)
 	{
 		case HW_AXIS:
-			hwString=[NSString stringWithString: @"axis"];
+			hwString = @"axis";
 			break;
 		case HW_BUTTON:
-			hwString=[NSString stringWithString: @"button"];
+			hwString = @"button";
 			break;
 		default:
-			hwString=[NSString stringWithString: @"axis/button"];
+			hwString = @"axis/button";
 	}
 	return hwString;   
 }
@@ -405,9 +418,9 @@ MA 02110-1301, USA.
 
 // TODO: This data could be put into a plist (i18n or just modifiable by
 // the user). It is otherwise an ugly method, but it'll do for testing.
-- (NSArray *)getStickFunctionList
+- (NSArray *)stickFunctionList
 {
-	NSMutableArray *funcList=[[NSMutableArray alloc] init];
+	NSMutableArray *funcList = [NSMutableArray array];
 	
 	[funcList addObject: 
 	 [self makeStickGuiDict:DESC(@"stickmapper-roll")
@@ -489,6 +502,16 @@ MA 02110-1301, USA.
 				  allowable:HW_BUTTON
 					 axisfn:STICK_NOFUNCTION
 					  butfn:BUTTON_ID]];
+	[funcList addObject:
+	 [self makeStickGuiDict:DESC(@"stickmapper-previous-target")
+				  allowable:HW_BUTTON
+					 axisfn:STICK_NOFUNCTION
+					  butfn:BUTTON_PREVTARGET]];
+	[funcList addObject:
+	 [self makeStickGuiDict:DESC(@"stickmapper-next-target")
+				  allowable:HW_BUTTON
+					 axisfn:STICK_NOFUNCTION
+					  butfn:BUTTON_NEXTTARGET]];
 	[funcList addObject:
 	 [self makeStickGuiDict:DESC(@"stickmapper-fuel-injection")
 				  allowable:HW_BUTTON
@@ -548,7 +571,8 @@ MA 02110-1301, USA.
 							axisfn:(int)axisfn
 							 butfn:(int)butfn
 {
-	NSMutableDictionary *guiDict=[[NSMutableDictionary alloc] init];
+	NSMutableDictionary *guiDict = [NSMutableDictionary dictionary];
+	
 	if ([what length] > 30)  what = [[what substringToIndex:28] stringByAppendingString:@"..."];
 	[guiDict setObject: what  forKey: KEY_GUIDESC];
 	[guiDict setObject: [NSNumber numberWithInt: allowable]  
