@@ -1,43 +1,36 @@
 /*
-
-ShipEntity.h
-
-Entity subclass representing a ship, or various other flying things like cargo
-pods and stations (a subclass).
-
-Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-MA 02110-1301, USA.
-
-*/
+ 
+ ShipEntity.h
+ 
+ Entity subclass representing a ship, or various other flying things like cargo
+ pods and stations (a subclass).
+ 
+ Oolite
+ Copyright (C) 2004-2013 Giles C Williams and contributors
+ 
+ This program is free software; you can redistribute it and/or
+ modify it under the terms of the GNU General Public License
+ as published by the Free Software Foundation; either version 2
+ of the License, or (at your option) any later version.
+ 
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU General Public License for more details.
+ 
+ You should have received a copy of the GNU General Public License
+ along with this program; if not, write to the Free Software
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ MA 02110-1301, USA.
+ 
+ */
 
 #import "OOEntityWithDrawable.h"
 #import "OOPlanetEntity.h"
 #import "OOJSPropID.h"
 
 @class	OOColor, StationEntity, WormholeEntity, AI, Octree, OOMesh, OOScript,
-		OOJSScript, OORoleSet, OOShipGroup, OOEquipmentType;
-
-#ifdef OO_BRAIN_AI
-@class OOBrain;
-#endif
-
-@protocol OOHUDBeaconIcon;
-
+OOJSScript, OORoleSet, OOShipGroup, OOEquipmentType, OOWeakSet;
 
 #define MAX_TARGETS						24
 #define RAIDER_MAX_CARGO				5
@@ -58,7 +51,9 @@ MA 02110-1301, USA.
 #define MILITARY_JAMMER_MIN_ENERGY		128
 
 #define COMBAT_IN_RANGE_FACTOR			0.035f
+#define COMBAT_BROADSIDE_IN_RANGE_FACTOR			0.020f
 #define COMBAT_OUT_RANGE_FACTOR			0.500f
+#define COMBAT_BROADSIDE_RANGE_FACTOR			0.900f
 #define COMBAT_WEAPON_RANGE_FACTOR		1.200f
 #define COMBAT_JINK_OFFSET				500.0f
 
@@ -100,22 +95,48 @@ MA 02110-1301, USA.
 #define INITIAL_SHOT_TIME				100.0
 
 #define	MIN_FUEL						0				// minimum fuel required for afterburner use
+#define MAX_JUMP_RANGE					7.0				// the 7 ly limit
 
 #define ENTITY_PERSONALITY_MAX			0x7FFFU
 #define ENTITY_PERSONALITY_INVALID		0xFFFFU
 
-#define WEAPON_FACING_NONE				0
-#define WEAPON_FACING_FORWARD			1
-#define WEAPON_FACING_AFT				2
-#define WEAPON_FACING_PORT				4
-#define WEAPON_FACING_STARBOARD			8
+
+#define WEAPON_COOLING_FACTOR			6.0f
+#define NPC_MAX_WEAPON_TEMP				256.0f
+#define WEAPON_COOLING_CUTOUT			0.85f
+
+#define COMBAT_AI_WEAPON_TEMP_READY		0.25f * NPC_MAX_WEAPON_TEMP
+#define COMBAT_AI_WEAPON_TEMP_USABLE	WEAPON_COOLING_CUTOUT * NPC_MAX_WEAPON_TEMP
+// factor determining how close to target AI has to be to be confident in aim
+// higher factor makes confident at longer ranges
+#define COMBAT_AI_CONFIDENCE_FACTOR		1250000.0f
+#define COMBAT_AI_ISNT_AWFUL			0.0f
+// removes BEHAVIOUR_ATTACK_FLY_TO_TARGET_SIX/TWELVE (unless thargoid)
+#define COMBAT_AI_IS_SMART				5.0f
+// adds BEHAVIOUR_(FLEE_)EVASIVE_ACTION
+#define COMBAT_AI_FLEES_BETTER			6.0f
+// adds BEHAVIOUR_ATTACK_BREAK_OFF_TARGET
+#define COMBAT_AI_DOGFIGHTER			6.5f
+// adds BEHAVIOUR_ATTACK_SLOW_DOGFIGHT
+#define COMBAT_AI_TRACKS_CLOSER			7.5f
+#define COMBAT_AI_USES_SNIPING			8.5f
+// adds BEHAVIOUR_ATTACK_SNIPER
+#define COMBAT_AI_FLEES_BETTER_2		9.0f
+
+
+
+#define MAX_LANDING_SPEED				50.0
+#define MAX_LANDING_SPEED2				(MAX_LANDING_SPEED * MAX_LANDING_SPEED)
+
+#define MAX_COS							0.995	// cos(5 degrees) is close enough in most cases for navigation
+#define MAX_COS2						(MAX_COS * MAX_COS)
 
 
 #define ENTRY(label, value) label = value,
 
 typedef enum OOBehaviour
 {
-	#include "OOBehaviour.tbl"
+#include "OOBehaviour.tbl"
 } OOBehaviour;
 
 #undef ENTRY
@@ -148,23 +169,14 @@ typedef enum
 typedef enum
 {
 #define DIFF_STRING_ENTRY(label, string) label,
-	#include "OOShipDamageType.tbl"
+#include "OOShipDamageType.tbl"
 #undef DIFF_STRING_ENTRY
 	
 	kOOShipDamageTypeDefault = kOODamageTypeEnergy
 } OOShipDamageType;
 
 
-// Methods that must be supported by subentities, regardless of type.
-@protocol OOSubEntity
-
-- (void) rescaleBy:(GLfloat)factor;
-
-@end
-
-
-
-@interface ShipEntity: OOEntityWithDrawable <OOSubEntity>
+@interface ShipEntity: OOEntityWithDrawable <OOSubEntity, OOBeaconEntity>
 {
 @public
 	// derived variables
@@ -173,11 +185,15 @@ typedef enum
 	// navigation
 	Vector					v_forward, v_up, v_right;	// unit vectors derived from the direction faced
 	
-	// variables which are controlled by instincts/AI
+	// variables which are controlled by AI
 	Vector					destination;				// for flying to/from a set point
-	OOUniversalID			primaryTarget;				// for combat or rendezvous
+
 	GLfloat					desired_range;				// range to which to journey/scan
 	GLfloat					desired_speed;				// speed at which to travel
+	// next three used to set desired attitude, flightRoll etc. gradually catch up to target
+	GLfloat					stick_roll;					// stick roll
+	GLfloat					stick_pitch;				// stick pitch
+	GLfloat					stick_yaw;					// stick yaw
 	OOBehaviour				behaviour;					// ship's behavioural state
 	
 	BoundingBox				totalBoundingBox;			// records ship configuration
@@ -194,8 +210,6 @@ typedef enum
 	//docking instructions
 	NSDictionary			*dockingInstructions;
 	
-	OOUniversalID			last_escort_target;			// last target an escort was deployed after
-	
 	OOColor					*laser_color;
 	OOColor					*scanner_display_color1;
 	OOColor					*scanner_display_color2;
@@ -212,13 +226,12 @@ typedef enum
 	GLfloat					thrust;						// acceleration
 	float					hyperspaceMotorSpinTime;	// duration of hyperspace countdown
 	
-	// TODO: stick all equipment in a list, and move list from playerEntity to shipEntity. -- Ahruman
 	unsigned				military_jammer_active: 1,	// military_jammer
 	
 							docking_match_rotation: 1,
 	
-	
 							pitching_over: 1,			// set to YES if executing a sharp loop
+							rolling_over: 1,			// set to YES if executing a sharp roll
 							reportAIMessages: 1,		// normally NO, suppressing AI message reporting
 	
 							being_mined: 1,				// normally NO, set to Yes when fired on by mining laser
@@ -227,7 +240,6 @@ typedef enum
 	
 							isHulk: 1,					// This is used to distinguish abandoned ships from cargo
 							trackCloseContacts: 1,
-	
 							isNearPlanetSurface: 1,		// check for landing on planet
 							isFrangible: 1,				// frangible => subEntities can be damaged individually
 							cloaking_device_active: 1,	// cloaking_device
@@ -239,31 +251,41 @@ typedef enum
 							isMissile: 1,				// Whether this was launched by fireMissile (used to track submunitions).
 							isUnpiloted: 1,				// Is meant to not have crew
 							hasScoopMessage: 1,			// suppress scoop messages when false.
-	
-	// scripting
+							
+							// scripting
 							scripted_misjump: 1,
 							haveExecutedSpawnAction: 1,
 							noRocks: 1,
 							_lightsActive: 1;
+
+	GLfloat    _scriptedMisjumpRange; 
 	
 	OOFuelQuantity			fuel;						// witch-space fuel
 	GLfloat					fuel_accumulator;
 	
-	OOCargoQuantity			likely_cargo;				// likely amount of cargo (for merchantmen, this is what is spilled as loot)
+	OOCargoQuantity			likely_cargo;				// likely amount of cargo (for pirates, this is what is spilled as loot)
 	OOCargoQuantity			max_cargo;					// capacity of cargo hold
 	OOCargoQuantity			extra_cargo;				// capacity of cargo hold extension (if any)
+	OOCargoQuantity			equipment_weight;			// amount of equipment using cargo space (excluding passenger_berth & extra_cargo_bay)
 	OOCargoType				cargo_type;					// if this is scooped, this is indicates contents
 	OOCargoFlag				cargo_flag;					// indicates contents for merchantmen
 	OOCreditsQuantity		bounty;						// bounty (if any)
 	
 	GLfloat					energy_recharge_rate;		// recharge rate for energy banks
 	
+	OOWeaponFacingSet		weapon_facings;				// weapon mounts available (bitmask)
 	OOWeaponType			forward_weapon_type;		// type of forward weapon (allows lasers, plasma cannon, others)
 	OOWeaponType			aft_weapon_type;			// type of aft weapon (allows lasers, plasma cannon, others)
+	OOWeaponType			port_weapon_type;			// type of port weapon
+	OOWeaponType			starboard_weapon_type;			// type of starboard weapon
 	GLfloat					weapon_damage;				// energy damage dealt by weapon
 	GLfloat					weapon_damage_override;		// custom energy damage dealt by front laser, if applicable
 	GLfloat					weaponRange;				// range of the weapon (in meters)
+	OOWeaponFacing			currentWeaponFacing;		// not necessarily the same as view for the player
 	
+	GLfloat					weapon_temp, weapon_shot_temperature; // active weapon temp, delta-temp
+	GLfloat					forward_weapon_temp, aft_weapon_temp, port_weapon_temp, starboard_weapon_temp; // current weapon temperatures
+
 	GLfloat					scannerRange;				// typically 25600
 	
 	unsigned				missiles;					// number of on-board missiles
@@ -272,9 +294,6 @@ typedef enum
 	OOTimeDelta				missile_load_time;			// minimum time interval between missile launches
 	OOTimeAbsolute			missile_launch_time;		// time of last missile launch
 	
-#ifdef OO_BRAIN_AI
-	OOBrain					*brain;						// brain controlling ship, could be a character brain or the autopilot
-#endif
 	AI						*shipAI;					// ship's AI system
 	
 	NSString				*name;						// descriptive name
@@ -286,20 +305,18 @@ typedef enum
 	Vector					jink;						// x and y set factors for offsetting a pursuing ship's position
 	Vector					coordinates;				// for flying to/from a set point
 	Vector					reference;					// a direction vector of magnitude 1 (* turrets *)
-	OOUniversalID			primaryAggressor;			// recorded after an attack
-	OOUniversalID			targetStation;				// for docking
-	OOUniversalID			found_target;				// from scans
-	OOUInteger				_subIdx;					// serialisation index - used only if this ship is a subentity
-	OOUInteger				_maxShipSubIdx;				// serialisation index - the number of ship subentities inside the shipdata
+	
+	NSUInteger				_subIdx;					// serialisation index - used only if this ship is a subentity
+	NSUInteger				_maxShipSubIdx;				// serialisation index - the number of ship subentities inside the shipdata
 	double					launch_time;				// time at which launched
 	double					launch_delay;				// delay for thinking after launch
+	OOUniversalID			planetForLanding;			// for landing
 	
 	GLfloat					frustration,				// degree of dissatisfaction with the current behavioural state, factor used to test this
-							success_factor;
+	success_factor;
 	
 	int						patrol_counter;				// keeps track of where the ship is along a patrol route
 	
-	OOUniversalID			proximity_alert;			// id of a ShipEntity within 2x collision_radius
 	NSMutableDictionary		*previousCondition;			// restored after collision avoidance
 	
 	// derived variables
@@ -308,10 +325,10 @@ typedef enum
 	double					cargo_dump_time;			// time cargo was last dumped
 	
 	NSMutableArray			*cargo;						// cargo containers go in here
-
-	int						commodity_type;				// type of commodity in a container
-	int						commodity_amount;			// 1 if unit is TONNES (0), possibly more if precious metals KILOGRAMS (1)
-														// or gem stones GRAMS (2)
+	
+	OOCommodityType			commodity_type;				// type of commodity in a container
+	OOCargoQuantity			commodity_amount;			// 1 if unit is TONNES (0), possibly more if precious metals KILOGRAMS (1)
+	// or gem stones GRAMS (2)
 	
 	// navigation
 	GLfloat					flightSpeed;				// current speed
@@ -319,18 +336,18 @@ typedef enum
 	GLfloat					flightPitch;				// current pitch rate
 	GLfloat					flightYaw;					// current yaw rate
 	
-	float					accuracy;
-	float					pitch_tolerance;
+	GLfloat					accuracy;
+	GLfloat					pitch_tolerance;
+	GLfloat					aim_tolerance;
+	int					_missed_shots;
 	
 	OOAegisStatus			aegis_status;				// set to YES when within the station's protective zone
-
+	
 	
 	double					messageTime;				// counts down the seconds a radio message is active for
 	
 	double					next_spark_time;			// time of next spark when throwing sparks
 	
-	OOUniversalID			thanked_ship_id;			// last ship thanked
-	
 	Vector					collision_vector;			// direction of colliding thing.
 	
 	//position of gun ports
@@ -354,7 +371,7 @@ typedef enum
 	float					ship_temperature;
 	
 	// for advanced scanning etc.
-	ShipEntity*				scanned_ships[MAX_SCAN_NUMBER + 1];
+	ShipEntity				*scanned_ships[MAX_SCAN_NUMBER + 1];
 	GLfloat					distance2_scanned_ships[MAX_SCAN_NUMBER + 1];
 	unsigned				n_scanned_ships;
 	
@@ -376,10 +393,23 @@ typedef enum
 	
 	NSMutableArray			*subEntities;
 	OOEquipmentType			*missile_list[SHIPENTITY_MAX_MISSILES];
+
+	// various types of target
+	OOWeakReference			*_primaryTarget;			// for combat or rendezvous
+	OOWeakReference			*_primaryAggressor;			// recorded after attack
+	OOWeakReference			*_targetStation;			// for docking
+	OOWeakReference			*_foundTarget;				// from scans
+	OOWeakReference			*_lastEscortTarget;			// last target an escort was deployed after
+	OOWeakReference			*_thankedShip;				// last ship thanked
+	OOWeakReference			*_rememberedShip;			// ship being remembered
+	OOWeakReference			*_proximityAlert;			// a ShipEntity within 2x collision_radius
+	
+	
+
 	
 @private
 	OOWeakReference			*_subEntityTakingDamage;	//	frangible => subEntities can be damaged individually
-	
+
 	NSString				*_shipKey;
 	
 	NSMutableSet			*_equipment;
@@ -395,20 +425,26 @@ typedef enum
 	Vector					_escortPositions[MAX_ESCORTS];
 	BOOL					_escortPositionsValid;
 	
+	OOWeakSet				*_defenseTargets;			 // defense targets
+	
 	GLfloat					_profileRadius;
 	
 	OOWeakReference			*_shipHitByLaser;			// entity hit by the last laser shot
 	
 	// beacons
 	NSString				*_beaconCode;
+	OOWeakReference			*_prevBeacon;
 	OOWeakReference			*_nextBeacon;
 	id <OOHUDBeaconIcon>	_beaconDrawable;
+
+	double			_nextAegisCheck;
 }
 
 // ship brains
 - (void) setStateMachine:(NSString *)ai_desc;
 - (void) setAI:(AI *)ai;
 - (AI *) getAI;
+- (BOOL) hasAutoAI;
 - (void) setShipScript:(NSString *)script_name;
 - (void) removeScript;
 - (OOScript *) shipScript;
@@ -417,10 +453,8 @@ typedef enum
 
 - (void) interpretAIMessage:(NSString *)message;
 
-#ifdef OO_BRAIN_AI
-- (OOBrain *)brain;
-- (void)setBrain:(OOBrain*) aBrain;
-#endif
+- (GLfloat)accuracy;
+- (void)setAccuracy:(GLfloat) new_accuracy;
 
 - (OOMesh *)mesh;
 - (void)setMesh:(OOMesh *)mesh;
@@ -432,7 +466,7 @@ typedef enum
 - (Vector) rightVector;
 
 - (NSArray *)subEntities;
-- (unsigned) subEntityCount;
+- (NSUInteger) subEntityCount;
 - (BOOL) hasSubEntity:(Entity<OOSubEntity> *)sub;
 
 - (NSEnumerator *)subEntityEnumerator;
@@ -448,29 +482,31 @@ typedef enum
 // subentities management
 - (NSString *) serializeShipSubEntities;
 - (void) deserializeShipSubEntitiesFrom:(NSString *)string;
-- (OOUInteger) maxShipSubEntities;
-- (void) setSubIdx:(OOUInteger)value;
-- (OOUInteger) subIdx;
+- (NSUInteger) maxShipSubEntities;
+- (void) setSubIdx:(NSUInteger)value;
+- (NSUInteger) subIdx;
 
 - (Octree *) octree;
 - (float) volume;
 
 // octree collision hunting
-- (GLfloat)doesHitLine:(Vector) v0: (Vector) v1;
-- (GLfloat)doesHitLine:(Vector) v0: (Vector) v1 :(ShipEntity**) hitEntity;
-- (GLfloat)doesHitLine:(Vector) v0: (Vector) v1 withPosition:(Vector) o andIJK:(Vector) i :(Vector) j :(Vector) k;	// for subentities
+- (GLfloat)doesHitLine:(Vector)v0 :(Vector)v1;
+- (GLfloat)doesHitLine:(Vector)v0 :(Vector)v1 :(ShipEntity**)hitEntity;
+- (GLfloat)doesHitLine:(Vector)v0 :(Vector)v1 withPosition:(Vector)o andIJK:(Vector)i :(Vector)j :(Vector)k;	// for subentities
 
-- (BoundingBox) findBoundingBoxRelativeToPosition:(Vector)opv InVectors:(Vector) _i :(Vector) _j :(Vector) _k;
+- (BoundingBox) findBoundingBoxRelativeToPosition:(Vector)opv InVectors:(Vector)i :(Vector)j :(Vector)k;
 
 - (Vector)absoluteTractorPosition;
 
-// beacons
-- (NSString *) beaconCode;
+// beacons // definitions now in <OOBeaconEntity> protocol
+/*- (NSString *) beaconCode;
 - (void) setBeaconCode:(NSString *)bcode;
 - (BOOL) isBeacon;
 - (id <OOHUDBeaconIcon>) beaconDrawable;
+- (ShipEntity *) prevBeacon;
 - (ShipEntity *) nextBeacon;
-- (void) setNextBeacon:(ShipEntity*) beaconShip;
+- (void) setPrevBeacon:(ShipEntity*) beaconShip;
+- (void) setNextBeacon:(ShipEntity*) beaconShip; */
 
 - (void) setIsBoulder:(BOOL)flag;
 - (BOOL) isBoulder;
@@ -484,13 +520,20 @@ typedef enum
 - (BOOL)setUpFromDictionary:(NSDictionary *) shipDict;
 - (BOOL)setUpShipFromDictionary:(NSDictionary *) shipDict;
 - (BOOL)setUpSubEntities;
+- (BOOL) setUpOneStandardSubentity:(NSDictionary *) subentDict asTurret:(BOOL)asTurret;
+- (GLfloat)frustumRadius;
 
 - (NSString *) shipDataKey;
+- (NSString *) shipDataKeyAutoRole;
 - (void)setShipDataKey:(NSString *)key;
 
 - (NSDictionary *)shipInfoDictionary;
 
 - (void) setDefaultWeaponOffsets;
+- (Vector) aftWeaponOffset;
+- (Vector) forwardWeaponOffset;
+- (Vector) portWeaponOffset;
+- (Vector) starboardWeaponOffset;
 
 - (BOOL) isFrangible;
 - (BOOL) suppressFlightNotifications;
@@ -498,23 +541,27 @@ typedef enum
 - (void) respondToAttackFrom:(Entity *)from becauseOf:(Entity *)other;
 
 // Equipment
+- (OOWeaponFacingSet) weaponFacings;
 - (BOOL) hasEquipmentItem:(id)equipmentKeys includeWeapons:(BOOL)includeWeapons whileLoading:(BOOL)loading;	// This can take a string or an set or array of strings. If a collection, returns YES if ship has _any_ of the specified equipment. If includeWeapons is NO, missiles and primary weapons are not checked.
 - (BOOL) hasEquipmentItem:(id)equipmentKeys;			// Short for hasEquipmentItem:foo includeWeapons:NO whileLoading:NO
 - (BOOL) hasAllEquipment:(id)equipmentKeys includeWeapons:(BOOL)includeWeapons whileLoading:(BOOL)loading;		// Like hasEquipmentItem:includeWeapons:, but requires _all_ elements in collection.
 - (BOOL) hasAllEquipment:(id)equipmentKeys;				// Short for hasAllEquipment:foo includeWeapons:NO
-- (BOOL) canAddEquipment:(NSString *)equipmentKey;		// Test ability to add equipment, taking equipment-specific constriants into account. 
-- (BOOL) equipmentValidToAdd:(NSString *)equipmentKey;	// Actual test if equipment satisfies validation criteria.
-- (BOOL) equipmentValidToAdd:(NSString *)equipmentKey whileLoading:(BOOL)loading;
-- (BOOL) addEquipmentItem:(NSString *)equipmentKey;
-- (BOOL) addEquipmentItem:(NSString *)equipmentKey withValidation:(BOOL)validateAddition;
+- (BOOL) setWeaponMount:(OOWeaponFacing)facing toWeapon:(NSString *)eqKey;
+- (BOOL) canAddEquipment:(NSString *)equipmentKey inContext:(NSString *)context;		// Test ability to add equipment, taking equipment-specific constriants into account. 
+- (BOOL) equipmentValidToAdd:(NSString *)equipmentKey inContext:(NSString *)context;	// Actual test if equipment satisfies validation criteria.
+- (BOOL) equipmentValidToAdd:(NSString *)equipmentKey whileLoading:(BOOL)loading inContext:(NSString *)context;
+- (BOOL) addEquipmentItem:(NSString *)equipmentKey inContext:(NSString *)context;
+- (BOOL) addEquipmentItem:(NSString *)equipmentKey withValidation:(BOOL)validateAddition inContext:(NSString *)context;
 - (BOOL) hasHyperspaceMotor;
+- (float) hyperspaceSpinTime;
+
 
 - (NSEnumerator *) equipmentEnumerator;
-- (unsigned) equipmentCount;
+- (NSUInteger) equipmentCount;
 - (void) removeEquipmentItem:(NSString *)equipmentKey;
 - (void) removeAllEquipment;
 - (OOEquipmentType *) selectMissile;
-- (int) removeMissiles;
+- (OOCreditsQuantity) removeMissiles;
 
 // Internal, subject to change. Use the methods above instead.
 - (BOOL) hasOneEquipmentItem:(NSString *)itemKey includeWeapons:(BOOL)includeMissiles whileLoading:(BOOL)loading;
@@ -522,14 +569,15 @@ typedef enum
 - (BOOL) hasPrimaryWeapon:(OOWeaponType)weaponType;
 - (BOOL) removeExternalStore:(OOEquipmentType *)eqType;
 
-// Passengers - not supported for NPCs, but interface is here for genericity.
-- (unsigned) passengerCount;
-- (unsigned) passengerCapacity;
+// Passengers and parcels - not supported for NPCs, but interface is here for genericity.
+- (NSUInteger) parcelCount;
+- (NSUInteger) passengerCount;
+- (NSUInteger) passengerCapacity;
 
-- (unsigned) missileCount;
-- (unsigned) missileCapacity;
+- (NSUInteger) missileCount;
+- (NSUInteger) missileCapacity;
 
-- (unsigned) extraCargo;
+- (NSUInteger) extraCargo;
 
 // Tests for the various special-cased equipment items
 - (BOOL) hasScoop;
@@ -558,6 +606,7 @@ typedef enum
 - (float) maxThrust;
 - (float) thrust;
 
+- (void) processBehaviour:(OOTimeDelta)delta_t;
 // Behaviours
 - (void) behaviour_stop_still:(double) delta_t;
 - (void) behaviour_idle:(double) delta_t;
@@ -566,20 +615,32 @@ typedef enum
 - (void) behaviour_track_target:(double) delta_t;
 - (void) behaviour_intercept_target:(double) delta_t;
 - (void) behaviour_attack_target:(double) delta_t;
+- (void) behaviour_attack_slow_dogfight:(double) delta_t;
+- (void) behaviour_evasive_action:(double) delta_t;
+- (void) behaviour_attack_break_off_target:(double) delta_t;
 - (void) behaviour_fly_to_target_six:(double) delta_t;
 - (void) behaviour_attack_mining_target:(double) delta_t;
 - (void) behaviour_attack_fly_to_target:(double) delta_t;
 - (void) behaviour_attack_fly_from_target:(double) delta_t;
 - (void) behaviour_running_defense:(double) delta_t;
 - (void) behaviour_flee_target:(double) delta_t;
+- (void) behaviour_attack_broadside:(double) delta_t;
+- (void) behaviour_attack_broadside_left:(double) delta_t;
+- (void) behaviour_attack_broadside_right:(double) delta_t;
+- (void) behaviour_close_to_broadside_range:(double) delta_t;
+- (void) behaviour_close_with_target:(double) delta_t;
+- (void) behaviour_attack_broadside_target:(double) delta_t leftside:(BOOL)leftside;
+- (void) behaviour_attack_sniper:(double) delta_t;
 - (void) behaviour_fly_range_from_destination:(double) delta_t;
 - (void) behaviour_face_destination:(double) delta_t;
+- (void) behaviour_land_on_planet:(double) delta_t;
 - (void) behaviour_formation_form_up:(double) delta_t;
 - (void) behaviour_fly_to_destination:(double) delta_t;
 - (void) behaviour_fly_from_destination:(double) delta_t;
 - (void) behaviour_avoid_collision:(double) delta_t;
 - (void) behaviour_track_as_turret:(double) delta_t;
 - (void) behaviour_fly_thru_navpoints:(double) delta_t;
+- (void) behaviour_scripted_ai:(double) delta_t;
 
 - (GLfloat *) scannerDisplayColorForShip:(ShipEntity*)otherShip :(BOOL)isHostile :(BOOL)flash :(OOColor *)scannerDisplayColor1 :(OOColor *)scannerDisplayColor2;
 - (void)setScannerDisplayColor1:(OOColor *)color1;
@@ -592,9 +653,8 @@ typedef enum
 - (BOOL)hasAutoCloak;
 - (void)setAutoCloak:(BOOL)automatic;
 
-- (BOOL) isJammingScanning;
-
 - (void) applyThrust:(double) delta_t;
+- (void) applyAttitudeChanges:(double) delta_t;
 
 - (void) avoidCollision;
 - (void) resumePostProximityAlert;
@@ -620,8 +680,9 @@ typedef enum
 - (uint8_t) pendingEscortCount;
 - (void) setPendingEscortCount:(uint8_t)count;
 
-- (ShipEntity *) proximity_alert;
-- (void) setProximity_alert:(ShipEntity*) other;
+// allow adjustment of escort numbers from shipdata.plist levels
+- (uint8_t) maxEscortCount;
+- (void) setMaxEscortCount:(uint8_t)newCount;
 
 - (NSString *) name;
 - (NSString *) displayName;
@@ -655,42 +716,59 @@ typedef enum
 - (BOOL) hasHostileTarget;
 - (BOOL) isHostileTo:(Entity *)entity;
 
+// defense target handling
+- (NSUInteger) defenseTargetCount;
+- (NSArray *) allDefenseTargets;
+- (NSEnumerator *) defenseTargetEnumerator;
+- (BOOL) addDefenseTarget:(Entity *)target;
+- (BOOL) isDefenseTarget:(Entity *)target;
+- (void) removeDefenseTarget:(Entity *)target;
+- (void) removeAllDefenseTargets;
+
+
 - (GLfloat) weaponRange;
 - (void) setWeaponRange:(GLfloat) value;
 - (void) setWeaponDataFromType:(OOWeaponType)weapon_type;
 - (float) weaponRechargeRate;
 - (void) setWeaponRechargeRate:(float)value;
 - (void) setWeaponEnergy:(float)value;
+- (OOWeaponFacing) currentWeaponFacing;
 
 - (GLfloat) scannerRange;
-- (void) setScannerRange: (GLfloat) value;
+- (void) setScannerRange:(GLfloat)value;
 
 - (Vector) reference;
-- (void) setReference:(Vector) v;
+- (void) setReference:(Vector)v;
 
 - (BOOL) reportAIMessages;
-- (void) setReportAIMessages:(BOOL) yn;
+- (void) setReportAIMessages:(BOOL)yn;
 
 - (void) transitionToAegisNone;
 - (OOPlanetEntity *) findNearestPlanet;
 - (Entity<OOStellarBody> *) findNearestStellarBody;		// NOTE: includes sun.
 - (OOPlanetEntity *) findNearestPlanetExcludingMoons;
 - (OOAegisStatus) checkForAegis;
+- (void) forceAegisCheck;
 - (BOOL) withinStationAegis;
+- (void) setLastAegisLock:(Entity<OOStellarBody> *)lastAegisLock;
 
 - (NSArray*) crew;
-- (void) setCrew: (NSArray*) crewArray;
+- (void) setCrew:(NSArray *)crewArray;
 
 // Fuel and capacity in tenths of light-years.
 - (OOFuelQuantity) fuel;
-- (void) setFuel:(OOFuelQuantity) amount;
+- (void) setFuel:(OOFuelQuantity)amount;
 - (OOFuelQuantity) fuelCapacity;
 
 - (GLfloat) fuelChargeRate;
 
-- (void) setRoll:(double) amount;
-- (void) setPitch:(double) amount;
-- (void) setThrust:(double) amount;
+- (void) setRoll:(double)amount;
+- (void) setRawRoll:(double)amount; // does not multiply by PI/2
+- (void) setPitch:(double)amount;
+- (void) setYaw:(double)amount;
+- (void) setThrust:(double)amount;
+- (void) applySticks:(double)delta_t;
+
 
 - (void)setThrustForDemo:(float)factor;
 
@@ -698,56 +776,65 @@ typedef enum
  Sets the bounty on this ship to amount.  
  Does not check to see if the ship is allowed to have a bounty, for example if it is police.
  */
-- (void) setBounty:(OOCreditsQuantity) amount;
+- (void) setBounty:(OOCreditsQuantity)amount;
+- (void) setBounty:(OOCreditsQuantity)amount withReason:(OOLegalStatusReason)reason;
+- (void) setBounty:(OOCreditsQuantity)amount withReasonAsString:(NSString *)reason;
 - (OOCreditsQuantity) bounty;
 
 - (int) legalStatus;
 
-- (void) setCommodity:(OOCargoType)co_type andAmount:(OOCargoQuantity)co_amount;
-- (void) setCommodityForPod:(OOCargoType)co_type andAmount:(OOCargoQuantity)co_amount;
-- (OOCargoType) commodityType;
+- (void) setUpCargoType:(NSString *)cargoString;
+- (void) setCommodity:(OOCommodityType)co_type andAmount:(OOCargoQuantity)co_amount;
+- (void) setCommodityForPod:(OOCommodityType)co_type andAmount:(OOCargoQuantity)co_amount;
+- (OOCommodityType) commodityType;
 - (OOCargoQuantity) commodityAmount;
 
-- (OOCargoQuantity) maxCargo;
+- (OOCargoQuantity) maxAvailableCargoSpace;
 - (OOCargoQuantity) availableCargoSpace;
 - (OOCargoQuantity) cargoQuantityOnBoard;
 - (OOCargoType) cargoType;
 - (NSMutableArray *) cargo;
-- (void) setCargo:(NSArray *) some_cargo;
+- (void) setCargo:(NSArray *)some_cargo;
 - (BOOL) showScoopMessage;
 
 - (NSArray *) passengerListForScripting;
+- (NSArray *) parcelListForScripting;
 - (NSArray *) contractListForScripting;
 - (NSArray *) equipmentListForScripting;
-- (OOEquipmentType *) weaponTypeForFacing:(int) facing;
+- (OOEquipmentType *) weaponTypeForFacing:(OOWeaponFacing)facing;
 - (NSArray *) missilesList;
 
 - (OOCargoFlag) cargoFlag;
-- (void) setCargoFlag:(OOCargoFlag) flag;
+- (void) setCargoFlag:(OOCargoFlag)flag;
 
-- (void) setSpeed:(double) amount;
+- (void) setSpeed:(double)amount;
 - (double) desiredSpeed;
-- (void) setDesiredSpeed:(double) amount;
+- (void) setDesiredSpeed:(double)amount;
+- (double) desiredRange;
+- (void) setDesiredRange:(double)amount;
 
 - (double) cruiseSpeed;
 
 - (Vector) thrustVector;
 - (void) setTotalVelocity:(Vector)vel;	// Set velocity to vel - thrustVector, effectively setting the instanteneous velocity to vel.
 
-- (void) increase_flight_speed:(double) delta;
-- (void) decrease_flight_speed:(double) delta;
-- (void) increase_flight_roll:(double) delta;
-- (void) decrease_flight_roll:(double) delta;
-- (void) increase_flight_pitch:(double) delta;
-- (void) decrease_flight_pitch:(double) delta;
-- (void) increase_flight_yaw:(double) delta;
-- (void) decrease_flight_yaw:(double) delta;
+- (void) increase_flight_speed:(double)delta;
+- (void) decrease_flight_speed:(double)delta;
+- (void) increase_flight_roll:(double)delta;
+- (void) decrease_flight_roll:(double)delta;
+- (void) increase_flight_pitch:(double)delta;
+- (void) decrease_flight_pitch:(double)delta;
+- (void) increase_flight_yaw:(double)delta;
+- (void) decrease_flight_yaw:(double)delta;
 
 - (GLfloat) flightRoll;
 - (GLfloat) flightPitch;
 - (GLfloat) flightYaw;
 - (GLfloat) flightSpeed;
+- (GLfloat) maxFlightPitch;
 - (GLfloat) maxFlightSpeed;
+- (GLfloat) maxFlightRoll;
+- (GLfloat) maxFlightYaw;
 - (GLfloat) speedFactor;
 
 - (GLfloat) temperature;
@@ -761,6 +848,7 @@ typedef enum
 // the percentage of damage taken (100 is destroyed, 0 is fine)
 - (int) damage;
 
+- (void) dealEnergyDamage:(GLfloat) baseDamage atRange:(GLfloat) range withBias:(GLfloat) velocityBias;
 - (void) dealEnergyDamageWithinDesiredRange;
 - (void) dealMomentumWithinDesiredRange:(double)amount;
 
@@ -773,6 +861,7 @@ typedef enum
 - (void) becomeExplosion;
 - (void) becomeLargeExplosion:(double) factor;
 - (void) becomeEnergyBlast;
+- (void) broadcastEnergyBlastImminent;
 
 - (Vector) positionOffsetForAlignment:(NSString*) align;
 Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q, NSString* align);
@@ -783,9 +872,12 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 
 - (Triangle) absoluteIJKForSubentity;
 
-- (NSComparisonResult) compareBeaconCodeWith:(ShipEntity *)other;
-
+- (GLfloat)weaponRecoveryTime;
 - (GLfloat)laserHeatLevel;
+- (GLfloat)laserHeatLevelAft;
+- (GLfloat)laserHeatLevelForward;
+- (GLfloat)laserHeatLevelPort;
+- (GLfloat)laserHeatLevelStarboard;
 - (GLfloat)hullHeatLevel;
 - (GLfloat)entityPersonality;
 - (GLint)entityPersonalityInt;
@@ -796,22 +888,34 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 - (void) resetExhaustPlumes;
 
 /*-----------------------------------------
-
-	AI piloting methods
-
------------------------------------------*/
+ 
+ AI piloting methods
+ 
+ -----------------------------------------*/
 
 - (void) checkScanner;
 - (ShipEntity**) scannedShips;
 - (int) numberOfScannedShips;
 
-- (void) setFound_target:(Entity *) targetEntity;
+- (Entity *)foundTarget;
+- (Entity *)primaryAggressor;
+- (Entity *)lastEscortTarget;
+- (Entity *)thankedShip;
+- (Entity *)rememberedShip;
+- (Entity *)proximityAlert;
+- (void) setFoundTarget:(Entity *) targetEntity;
 - (void) setPrimaryAggressor:(Entity *) targetEntity;
+- (void) setLastEscortTarget:(Entity *) targetEntity;
+- (void) setThankedShip:(Entity *) targetEntity;
+- (void) setRememberedShip:(Entity *) targetEntity;
+- (void) setProximityAlert:(ShipEntity *) targetEntity;
 - (void) setTargetStation:(Entity *) targetEntity;
+- (BOOL) isValidTarget:(Entity *) target;
 - (void) addTarget:(Entity *) targetEntity;
 - (void) removeTarget:(Entity *) targetEntity;
 - (id) primaryTarget;
-- (OOUniversalID) primaryTargetID;
+- (id) primaryTargetWithoutValidityCheck;
+- (StationEntity *) targetStation;
 
 - (BOOL) isFriendlyTo:(ShipEntity *)otherShip;
 
@@ -826,53 +930,66 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 
 - (void) trackOntoTarget:(double) delta_t withDForward: (GLfloat) dp;
 
-- (double) ballTrackTarget:(double) delta_t;
-- (double) ballTrackLeadingTarget:(double) delta_t;
+- (double) ballTrackLeadingTarget:(double) delta_t atTarget:(Entity *)target;
 
 - (GLfloat) rollToMatchUp:(Vector) up_vec rotating:(GLfloat) match_roll;
 
 - (GLfloat) rangeToDestination;
 - (double) trackDestination:(double) delta_t :(BOOL) retreat;
-//- (double) trackPosition:(Vector) track_pos :(double) delta_t :(BOOL) retreat;
 
 - (void) setCoordinate:(Vector)coord;
 - (Vector) coordinates;
 - (Vector) destination;
 - (Vector) distance_six: (GLfloat) dist;
-- (Vector) distance_twelve: (GLfloat) dist;
+- (Vector) distance_twelve: (GLfloat) dist withOffset:(GLfloat)offset;
 
+- (void) setEvasiveJink:(GLfloat) z;
+- (void) evasiveAction:(double) delta_t;
 - (double) trackPrimaryTarget:(double) delta_t :(BOOL) retreat;
+- (double) trackSideTarget:(double) delta_t :(BOOL) leftside;
 - (double) missileTrackPrimaryTarget:(double) delta_t;
 
 //return 0.0 if there is no primary target
 - (double) rangeToPrimaryTarget;
-- (BOOL) onTarget:(BOOL) fwd_weapon;
+- (double) approachAspectToPrimaryTarget;
+- (double) rangeToSecondaryTarget:(Entity *)target;
+- (BOOL) hasProximityAlertIgnoringTarget:(BOOL)ignore_target;
+- (GLfloat) currentAimTolerance;
+- (BOOL) onTarget:(OOWeaponFacing)direction withWeapon:(OOWeaponType)weapon;
 
 - (OOTimeDelta) shotTime;
 - (void) resetShotTime;
 
 - (BOOL) fireMainWeapon:(double)range;
 - (BOOL) fireAftWeapon:(double)range;
+- (BOOL) firePortWeapon:(double)range;
+- (BOOL) fireStarboardWeapon:(double)range;
 - (BOOL) fireTurretCannon:(double)range;
 - (void) setLaserColor:(OOColor *)color;
 - (OOColor *)laserColor;
 - (BOOL) fireSubentityLaserShot:(double)range;
-- (BOOL) fireDirectLaserShot;
-- (BOOL) fireLaserShotInDirection:(OOViewID)direction;
+- (BOOL) fireDirectLaserShot:(double)range;
+- (BOOL) fireDirectLaserDefensiveShot;
+- (BOOL) fireDirectLaserShotAt:(Entity *)my_target;
+- (BOOL) fireLaserShotInDirection:(OOWeaponFacing)direction;
+- (void) adjustMissedShots:(int)delta;
+- (int) missedShots;
 - (BOOL) firePlasmaShotAtOffset:(double)offset speed:(double)speed color:(OOColor *)color;
+- (void) considerFiringMissile:(double)delta_t;
 - (ShipEntity *) fireMissile;
 - (ShipEntity *) fireMissileWithIdentifier:(NSString *) identifier andTarget:(Entity *) target;
 - (BOOL) isMissileFlagSet;
 - (void) setIsMissileFlag:(BOOL)newValue;
 - (OOTimeDelta) missileLoadTime;
 - (void) setMissileLoadTime:(OOTimeDelta)newMissileLoadTime;
+- (void) noticeECM;
 - (BOOL) fireECM;
 - (BOOL) cascadeIfAppropriateWithDamageAmount:(double)amount cascadeOwner:(Entity *)owner;
 - (BOOL) activateCloakingDevice;
 - (void) deactivateCloakingDevice;
 - (BOOL) launchCascadeMine;
-- (OOUniversalID) launchEscapeCapsule;
-- (OOCargoType) dumpCargo;
+- (ShipEntity *) launchEscapeCapsule;
+- (OOCommodityType) dumpCargo;
 - (ShipEntity *) dumpCargoItem;
 - (OOCargoType) dumpItem: (ShipEntity*) jetto;
 
@@ -900,10 +1017,11 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 - (BOOL) witchspaceLeavingEffects;
 
 /* 
-   Mark this ship as an offender, this is different to setBounty as some ships such as police 
-   are not markable.  The final bounty may not be equal to existing bounty plus offence_value.
+ Mark this ship as an offender, this is different to setBounty as some ships such as police 
+ are not markable.  The final bounty may not be equal to existing bounty plus offence_value.
  */
 - (void) markAsOffender:(int)offence_value;
+- (void) markAsOffender:(int)offence_value withReason:(OOLegalStatusReason)reason;
 
 - (void) switchLightsOn;
 - (void) switchLightsOff;
@@ -961,11 +1079,15 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 - (void) pilotArrived;
 #endif
 
-- (OOJSScript *)script;
-- (NSDictionary *)scriptInfo;
+- (OOJSScript *) script;
+- (NSDictionary *) scriptInfo;
+- (void) overrideScriptInfo:(NSDictionary *)override;	// Add items from override (if not nil) to scriptInfo, replacing in case of duplicates. Used for subentities.
 
 - (BOOL) scriptedMisjump;
 - (void) setScriptedMisjump:(BOOL)newValue;
+- (GLfloat) scriptedMisjumpRange;
+- (void) setScriptedMisjumpRange:(GLfloat)newValue;
+
 
 - (Entity *)entityForShaderProperties;
 
@@ -984,15 +1106,15 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 	ShipScriptEventNoCx(ship, "doSomething", INT_TO_JSVAL(42));
 */
 #define ShipScriptEvent(context, ship, event, ...) do { \
-	jsval argv[] = { __VA_ARGS__ }; \
-	uintN argc = sizeof argv / sizeof *argv; \
-	[ship doScriptEvent:OOJSID(event) inContext:context withArguments:argv count:argc]; \
+jsval argv[] = { __VA_ARGS__ }; \
+uintN argc = sizeof argv / sizeof *argv; \
+[ship doScriptEvent:OOJSID(event) inContext:context withArguments:argv count:argc]; \
 } while (0)
 
 #define ShipScriptEventNoCx(ship, event, ...) do { \
-	jsval argv[] = { __VA_ARGS__ }; \
-	uintN argc = sizeof argv / sizeof *argv; \
-	[ship doScriptEvent:OOJSID(event) withArguments:argv count:argc]; \
+jsval argv[] = { __VA_ARGS__ }; \
+uintN argc = sizeof argv / sizeof *argv; \
+[ship doScriptEvent:OOJSID(event) withArguments:argv count:argc]; \
 } while (0)
 
 - (void) reactToAIMessage:(NSString *)message context:(NSString *)debugContext;	// Immediate message
@@ -1012,9 +1134,11 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 #endif
 
 
-// For the common case of testing whether foo is a ship, bar is a ship, bar is a subentity of foo and this relationship is represented sanely.
 @interface Entity (SubEntityRelationship)
 
+/*	For the common case of testing whether foo is a ship, bar is a ship, bar
+	is a subentity of foo and this relationship is represented sanely.
+*/
 - (BOOL) isShipWithSubEntityShip:(Entity *)other;
 
 @end
@@ -1022,6 +1146,7 @@ Vector positionOffsetForShipInRotationToAlignment(ShipEntity* ship, Quaternion q
 
 NSDictionary *OODefaultShipShaderMacros(void);
 
+GLfloat getWeaponRangeFromType(OOWeaponType weapon_type);
 
 // Stuff implemented in OOConstToString.m
 enum
