@@ -3,7 +3,7 @@
 WormholeEntity.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -65,6 +65,8 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 		scan_info = WH_SCANINFO_NONE;
 		scan_time = 0;
 		hasExitPosition = NO;
+		containsPlayer = NO;
+		exit_speed = 50.0;
 	}
 	return self;
 }
@@ -100,6 +102,12 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 		// saving/restoring wormholes from dictionaries should know this!
 		expiry_time = [dict oo_doubleForKey:@"expiry_time"];
 		arrival_time = [dict oo_doubleForKey:@"arrival_time"];
+		// just in case an old save game has one with crossed times
+		if (expiry_time > arrival_time)
+		{
+			expiry_time = arrival_time - 1.0; 
+		}
+
 		// Since this is new for 1.75.1, we must give it a default values as we could be loading an old savegame
 		estimated_arrival_time = [dict oo_doubleForKey:@"estimated_arrival_time" defaultValue:arrival_time];
 		position = [dict oo_vectorForKey:@"position"];
@@ -170,6 +178,16 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 		travel_time = (distance * distance * 3600); // Taken from PlayerEntity.h
 		arrival_time = now + travel_time;
 		estimated_arrival_time = arrival_time;
+
+		/* There are a number of bugs where the arrival time is < than the
+		 * expiry time (i.e. both ends open at once).
+		 * Rather than try to flatten all of them, having been unsuccessful twice
+		 * it seems easier to declare as a matter of wormhole physics that it 
+		 * _can't_ be open at both ends at once. - CIM: 13/12/12 */
+		if (expiry_time > arrival_time)
+		{
+			expiry_time = arrival_time - 1.0; 
+		}
 		position = [ship position];
 		zero_distance = distance2([PLAYER position], position);
 	}	
@@ -194,21 +212,67 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 }
 
 
+- (void) setMisjumpWithRange:(GLfloat)range
+{
+	if (range <= 0.0 || range >= 1.0)
+	{
+		range = 0.5; // for safety, though nothing should be setting this
+	}
+	_misjumpRange = range;
+	// Test for misjump first - it's entirely possibly that the wormhole
+	// has already been marked for misjumping when another ship enters it.
+	if (!_misjump)
+	{
+		double distance = distanceBetweenPlanetPositions(originCoords.x, originCoords.y, destinationCoords.x, destinationCoords.y);
+		double time_adjust = (distance * (1-_misjumpRange))*(distance * (1-_misjumpRange))*3600.0;
+		// time adjustment ensures that misjumps not faster than normal jumps
+		// formulae for time and distance by mwerle at http://developer.berlios.de/pm/task.php?func=detailtask&project_task_id=4703&group_id=3577&group_project_id=1753
+		arrival_time -= time_adjust;
+		travel_time -= time_adjust;
+	
+		destinationCoords.x = (originCoords.x * (1-_misjumpRange)) + (destinationCoords.x * _misjumpRange);
+		destinationCoords.y = (originCoords.y * (1-_misjumpRange)) + (destinationCoords.y * _misjumpRange);
+		_misjump = YES;
+	}
+}
+
+
 - (BOOL) withMisjump
 {
 	return _misjump;
 }
 
 
+- (GLfloat) misjumpRange
+{
+	return _misjumpRange;
+}
+
+
 - (BOOL) suckInShip:(ShipEntity *) ship
 {
-	if (!ship)
+	if (!ship || [ship status] == STATUS_ENTERING_WITCHSPACE)
+	{
 		return NO;
+	}
+	if (!equal_seeds(origin,[UNIVERSE systemSeed]))
+	{
+		// if we're no longer in the origin system, can't suck in
+		return NO;
+	}
+	if ([PLAYER galaxy_coordinates].x != originCoords.x || [PLAYER galaxy_coordinates].y != originCoords.y)
+	{
+		// if we're no longer at the origin coordinates, can't suck in (handles interstellar space case)
+		return NO;
+	}
 
 	double now = [PLAYER clockTimeAdjusted];
 
-	if (now > arrival_time)
-		return NO;	// far end of the wormhole!
+/* CIM: removed test. Not valid for wormholes which last longer than their travel time. Most likely for short distances e.g. zero-distance doubles. equal_seeds test above should cover it, with expiry_time test for safety.  */
+/*	if (now > arrival_time)
+		return NO;	// far end of the wormhole! */
+	if( now > expiry_time )
+		return NO;
 	
 	// MKW 2010.11.18 - calculate time it takes for ship to reach wormhole
 	// This is for AI ships which get told to enter the wormhole even though they
@@ -241,6 +305,12 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 						nil]];
 	witch_mass += [ship mass];
 	expiry_time = now + (witch_mass / WORMHOLE_SHRINK_RATE / shrink_factor);
+	// and, again, cap to be earlier than arrival time
+	if (expiry_time > arrival_time)
+	{
+		expiry_time = arrival_time - 1.0; 
+	}
+
 	collision_radius = 0.5 * M_PI * pow(witch_mass, 1.0/3.0);
 	
 	[UNIVERSE addWitchspaceJumpEffectForShip:ship];
@@ -249,10 +319,21 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	[ship setStatus:STATUS_ENTERING_WITCHSPACE];
 	[ship doScriptEvent:OOJSID("shipWillEnterWormhole")];
 	[[ship getAI] message:@"ENTERED_WITCHSPACE"];
-	
+
 	[UNIVERSE removeEntity:ship];
 	[[ship getAI] clearStack];	// get rid of any preserved states
-	
+
+	if ([ship isStation])
+	{
+		if ([PLAYER dockedStation] == (StationEntity*)ship)
+		{
+			// the carrier has jumped while the player is docked
+			[ship retain];
+			[UNIVERSE carryPlayerOn:(StationEntity*)ship inWormhole:self];
+			[ship release];
+		}
+	}		
+
 	return YES;
 }
 
@@ -260,23 +341,23 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 - (void) disgorgeShips
 {
 	double now = [PLAYER clockTimeAdjusted];
-	int n_ships = [shipsInTransit count];
-	NSMutableArray * shipsStillInTransit = [[NSMutableArray alloc] initWithCapacity:n_ships];
+	NSMutableArray* shipsStillInTransit = [[NSMutableArray alloc] initWithCapacity:[shipsInTransit count]];
 	BOOL hasShiftedExitPosition = NO;
+	BOOL useExitXYScatter = NO;
 	
-	int i;
-	for (i = 0; i < n_ships; i++)
+	NSDictionary *shipInfo = nil;
+	foreach (shipInfo, shipsInTransit)
 	{
-		ShipEntity* ship = (ShipEntity*)[(NSDictionary*)[shipsInTransit objectAtIndex:i] objectForKey:@"ship"];
-		NSString *shipBeacon = [(NSDictionary *)[shipsInTransit objectAtIndex:i] objectForKey:@"shipBeacon"];
-		double	ship_arrival_time = arrival_time + [(NSNumber*)[(NSDictionary*)[shipsInTransit objectAtIndex:i] objectForKey:@"time"] doubleValue];
+		ShipEntity *ship = [shipInfo objectForKey:@"ship"];
+		NSString *shipBeacon = [shipInfo objectForKey:@"shipBeacon"];
+		double	ship_arrival_time = arrival_time + [shipInfo oo_doubleForKey:@"time"];
 		double	time_passed = now - ship_arrival_time;
-
+		
 		if ([ship status] == STATUS_DEAD) continue; // skip dead ships.
 		
 		if (ship_arrival_time > now)
 		{
-			[shipsStillInTransit addObject:[shipsInTransit objectAtIndex:i]];
+			[shipsStillInTransit addObject:shipInfo];
 		}
 		else
 		{
@@ -284,18 +365,45 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 			if (!hasExitPosition)
 			{
 				position = [UNIVERSE getWitchspaceExitPosition];	// no need to reset PRNG.
+				GLfloat min_d1 = [UNIVERSE safeWitchspaceExitDistance];
 				Quaternion	q1;
 				quaternion_set_random(&q1);
 				double		d1 = SCANNER_MAX_RANGE*((ranrot_rand() % 256)/256.0 - 0.5);
-				if (abs(d1) < 750.0)	// no closer than 750m
-					d1 += ((d1 > 0.0)? 750.0: -750.0);
+				if (abs(d1) < min_d1)	// no closer than 750m to edge of buoy
+					d1 += ((d1 > 0.0)? min_d1: -min_d1);
 				Vector		v1 = vector_forward_from_quaternion(q1);
 				position.x += v1.x * d1; // randomise exit position
 				position.y += v1.y * d1;
 				position.z += v1.z * d1;
 			}
-			[ship setPosition:position];
 			
+			if (hasExitPosition && (!containsPlayer || useExitXYScatter))
+			{
+				Vector shippos;
+				Vector exit_vector_x = vector_right_from_quaternion([UNIVERSE getWitchspaceExitRotation]);			
+				Vector exit_vector_y = vector_up_from_quaternion([UNIVERSE getWitchspaceExitRotation]);
+// entry wormhole has a radius of around 100m (or perhaps more)
+// so randomise exit positions slightly too for second and subsequent ships
+// helps avoid collisions when two ships enter wormhole at same time
+				double offset_x = randf()*150.0-75.0;
+				double offset_y = randf()*150.0-75.0;
+				shippos.x = position.x + (offset_x*exit_vector_x.x)+(offset_y*exit_vector_y.x);
+				shippos.y = position.y + (offset_x*exit_vector_x.y)+(offset_y*exit_vector_y.y);
+				shippos.z = position.z + (offset_x*exit_vector_x.z)+(offset_y*exit_vector_y.z);
+				[ship setPosition:shippos];
+			}
+			else
+			{
+				// this is the first ship out of the wormhole
+				[self setExitSpeed:[ship maxFlightSpeed]*WORMHOLE_LEADER_SPEED_FACTOR];
+				if (containsPlayer)
+				{ // reset the player's speed to the new speed
+					[PLAYER setSpeed:exit_speed];
+				}
+				useExitXYScatter = YES;
+				[ship setPosition:position];
+			}
+
 			if (shipBeacon != nil)
 			{
 				[ship setBeaconCode:shipBeacon];
@@ -303,7 +411,7 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 			
 			// Don't reduce bounty on misjump. Fixes #17992
 			// - MKW 2011.03.10	
-			if (!_misjump)  [ship setBounty:[ship bounty]/2];	// adjust legal status for new system
+			if (!_misjump)  [ship setBounty:[ship bounty]/2 withReason:kOOLegalStatusReasonNewSystem];	// adjust legal status for new system
 			
 			if ([ship cargoFlag] == CARGO_FLAG_FULL_PLENTIFUL)
 			{
@@ -320,10 +428,11 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 				[ship setOrientation: [UNIVERSE getWitchspaceExitRotation]];
 				[ship setPitch: 0.0];
 				[ship setRoll: 0.0];
-				[ship setSpeed: [ship maxFlightSpeed] * 0.25];
 				[ship setVelocity: kZeroVector];
 				[UNIVERSE addEntity:ship];	// AI and status get initialised here
 			}
+			[ship setSpeed:[self exitSpeed]]; // all ships from this wormhole have same velocity
+
 			
 			// Should probably pass the wormhole, but they have no JS representation
 			[ship doScriptEvent:OOJSID("shipExitedWormhole") andReactToAIMessage:@"EXITED WITCHSPACE"];
@@ -354,8 +463,25 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	}
 	[shipsInTransit release];
 	shipsInTransit = shipsStillInTransit;
+
+	if (containsPlayer)
+	{
+		// ships exiting the wormhole after now are following the player
+		// so appear behind them
+		position = vector_add([PLAYER position], vector_multiply_scalar([PLAYER forwardVector], -500.0f));
+		containsPlayer = NO;
+	}
+// else, the wormhole doesn't now (or never) contained the player, so
+// no need to move it
 }
 
+
+- (void) setContainsPlayer:(BOOL)val
+{
+	containsPlayer = val;
+}
+
+
 - (void) setExitPosition:(Vector)pos
 {
 	[self setPosition: pos];
@@ -382,6 +508,18 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	return destinationCoords;
 }
 
+- (double) exitSpeed
+{
+	return exit_speed;
+}
+
+
+- (void) setExitSpeed:(double) speed
+{
+	exit_speed = speed;
+}
+
+
 - (double) expiryTime
 {
 	return expiry_time;
@@ -484,10 +622,18 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 
 - (BOOL) canCollide
 {
-	if ([PLAYER clockTime] > arrival_time)
+	/* Correct test for far end of wormhole */
+	if (!equal_seeds(origin,[UNIVERSE systemSeed]))
 	{
-		return NO;	// far end of the wormhole!
+		// if we're no longer in the origin system, can't suck in
+		return NO;
 	}
+	if ([PLAYER galaxy_coordinates].x != originCoords.x || [PLAYER galaxy_coordinates].y != originCoords.y)
+	{
+		// if we're no longer at the origin coordinates, can't suck in (handles interstellar space case)
+		return NO;
+	}
+
 	return (witch_mass > 0.0);
 }
 
@@ -504,7 +650,7 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	
 	PlayerEntity	*player = PLAYER;
 	assert(player != nil);
-	rotMatrix = OOMatrixForBillboard(position, [player position]);
+	rotMatrix = OOMatrixForBillboard(position, [player viewpointPosition]);
 	double now = [player clockTimeAdjusted];
 	
 	if (witch_mass > 0.0)
@@ -520,6 +666,8 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	
 	if (now > expiry_time)
 	{
+		scanClass = CLASS_NO_DRAW; // witch_mass not certain to be limiting factor on extremely short jumps, so make sure now
+
 		// If we're a saved wormhole waiting to disgorge more ships, it's safe
 		// to remove self from UNIVERSE, but we need the current position!
 		[UNIVERSE removeEntity: self];
@@ -527,12 +675,12 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 }
 
 
-- (void) drawEntity:(BOOL) immediate :(BOOL) translucent
+- (void) drawImmediate:(bool)immediate translucent:(bool)translucent
 {	
 	if ([UNIVERSE breakPatternHide])
 		return;		// DON'T DRAW DURING BREAK PATTERN
 	
-	if (zero_distance > no_draw_distance)
+	if (cam_zero_distance > no_draw_distance)
 		return;	// TOO FAR AWAY TO SEE
 		
 	if (witch_mass <= 0.0)
@@ -547,12 +695,13 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	if (translucent)
 	{
 		// for now, a simple copy of the energy bomb draw routine
-		float srzd = sqrtf(zero_distance);
+		float srzd = sqrt(cam_zero_distance);
 		
-		GLfloat	color_fv[4] = { 0.0, 0.0, 1.0, 0.25};
+		GLfloat	color_fv[4] = { 0.0, 0.0, 1.0, 0.25 };
 		
-		OOGL(glDisable(GL_CULL_FACE));			// face culling
-		OOGL(glDisable(GL_TEXTURE_2D));
+		OOSetOpenGLState(OPENGL_STATE_TRANSLUCENT_PASS);
+		OOGL(glDisable(GL_CULL_FACE));
+		OOGL(glEnable(GL_BLEND));
 		
 		OOGL(glColor4fv(color_fv));
 		OOGLBEGIN(GL_TRIANGLE_FAN);
@@ -561,9 +710,12 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 				
 		DrawWormholeCorona(0.67 * collision_radius, collision_radius, 4, srzd, color_fv);
 					
-		OOGL(glEnable(GL_CULL_FACE));			// face culling
+		OOGL(glEnable(GL_CULL_FACE));
+		OOGL(glDisable(GL_BLEND));
 	}
-	CheckOpenGLErrors(@"WormholeEntity after drawing %@", self);
+	
+	OOVerifyOpenGLState();
+	OOCheckOpenGLErrors(@"WormholeEntity after drawing %@", self);
 }
 
 
@@ -599,13 +751,13 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 			
 			q = activity.location + rv0 * activity.length;
 			
-			s0 = r0 * sinf(theta);
-			c0 = r0 * cosf(theta);
+			s0 = r0 * sin(theta);
+			c0 = r0 * cos(theta);
 			glColor4f(col4v1[0] * q, col4v1[1] * q, col4v1[2] * q, col4v1[3] * rv0);
 			glVertex3f(s0, c0, 0.0);
 
-			s1 = r1 * sinf(theta - halfStep) * 0.5 * (1.0 + rv1);
-			c1 = r1 * cosf(theta - halfStep) * 0.5 * (1.0 + rv1);
+			s1 = r1 * sin(theta - halfStep) * 0.5 * (1.0 + rv1);
+			c1 = r1 * cos(theta - halfStep) * 0.5 * (1.0 + rv1);
 			glColor4f(col4v1[0], col4v1[1], col4v1[2], 0.0);
 			glVertex3f(s1, c1, 0.0);
 			
@@ -616,13 +768,13 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 			
 		q = activity.location + rv0 * activity.length;
 		
-		s0 = 0.0f;	// r0 * sinf(0);
-		c0 = r0;	// r0 * cosf(0);
+		s0 = 0.0f;	// r0 * sin(0);
+		c0 = r0;	// r0 * cos(0);
 		glColor4f(col4v1[0] * q, col4v1[1] * q, col4v1[2] * q, col4v1[3] * rv0);
 		glVertex3f(s0, c0, 0.0);
 
-		s1 = r1 * sinf(halfStep) * 0.5 * (1.0 + rv1);
-		c1 = r1 * cosf(halfStep) * 0.5 * (1.0 + rv1);
+		s1 = r1 * sin(halfStep) * 0.5 * (1.0 + rv1);
+		c1 = r1 * cos(halfStep) * 0.5 * (1.0 + rv1);
 		glColor4f(col4v1[0], col4v1[1], col4v1[2], 0.0);
 		glVertex3f(s1, c1, 0.0);
 	OOGLEND();
@@ -687,7 +839,7 @@ static void DrawWormholeCorona(GLfloat inner_radius, GLfloat outer_radius, int s
 	OOLog(@"dumpState.wormholeEntity", @"Scanned State          : %@", [self scanInfoString]);
 
 	OOLog(@"dumpState.wormholeEntity", @"Mass                   : %.2lf", witch_mass);
-	OOLog(@"dumpState.wormholeEntity", @"Ships                  : %d", [shipsInTransit count]);
+	OOLog(@"dumpState.wormholeEntity", @"Ships                  : %ld", [shipsInTransit count]);
 	unsigned i;
 	for (i = 0; i < [shipsInTransit count]; ++i)
 	{
