@@ -3,7 +3,7 @@
 GuiDisplayGen.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,11 +24,13 @@ MA 02110-1301, USA.
 
 #import "GuiDisplayGen.h"
 #import "Universe.h"
+#import "GameController.h"
 #import "PlayerEntity.h"
 #import "PlayerEntityControls.h"
 #import "OOTextureSprite.h"
 #import "ResourceManager.h"
 #import "OOSound.h"
+#import "OOStringExpander.h"
 #import "OOStringParsing.h"
 #import "HeadUpDisplay.h"
 #import "OOCollectionExtractors.h"
@@ -49,7 +51,10 @@ OOINLINE BOOL RowInRange(OOGUIRow row, NSRange range)
 - (void) drawCrossHairsWithSize:(GLfloat) size x:(GLfloat)x y:(GLfloat)y z:(GLfloat)z;
 - (void) drawStarChart:(GLfloat)x :(GLfloat)y :(GLfloat)z :(GLfloat) alpha;
 - (void) drawGalaxyChart:(GLfloat)x :(GLfloat)y :(GLfloat)z :(GLfloat) alpha;
-- (void) drawEqptList: (NSArray *)eqptList z:(GLfloat)z;
+- (void) drawSystemMarkers:(NSArray *)marker atX:(GLfloat)x andY:(GLfloat)y andZ:(GLfloat)z withAlpha:(GLfloat)alpha andScale:(GLfloat)scale;
+- (void) drawSystemMarker:(NSDictionary *)marker atX:(GLfloat)x andY:(GLfloat)y andZ:(GLfloat)z withAlpha:(GLfloat)alpha andScale:(GLfloat)scale;
+
+- (void) drawEquipmentList:(NSArray *)eqptList z:(GLfloat)z;
 - (void) drawAdvancedNavArrayAtX:(float)x y:(float)y z:(float)z alpha:(float)alpha usingRoute:(NSDictionary *) route optimizedBy:(OORouteType) optimizeBy;
 
 @end
@@ -61,50 +66,52 @@ static BOOL _refreshStarChart = NO;
 
 - (id) init
 {
-	self = [super init];
+	if ((self = [super init]))
+	{
+		size_in_pixels  = NSMakeSize(MAIN_GUI_PIXEL_WIDTH, MAIN_GUI_PIXEL_HEIGHT);
+		n_columns		= GUI_DEFAULT_COLUMNS;
+		n_rows			= GUI_DEFAULT_ROWS;
+		pixel_row_center = size_in_pixels.width / 2;
+		pixel_row_height = MAIN_GUI_ROW_HEIGHT;
+		pixel_row_start	= MAIN_GUI_PIXEL_ROW_START;		// first position down the page...
+		max_alpha = 1.0;
 		
-	size_in_pixels  = NSMakeSize(MAIN_GUI_PIXEL_WIDTH, MAIN_GUI_PIXEL_HEIGHT);
-	n_columns		= GUI_DEFAULT_COLUMNS;
-	n_rows			= GUI_DEFAULT_ROWS;
-	pixel_row_center = size_in_pixels.width / 2;
-	pixel_row_height = MAIN_GUI_ROW_HEIGHT;
-	pixel_row_start	= MAIN_GUI_PIXEL_ROW_START;		// first position down the page...
-	max_alpha = 1.0;
-
-	pixel_text_size = NSMakeSize(0.9f * pixel_row_height, pixel_row_height);	// main gui has 18x20 characters
-	
-	pixel_title_size = NSMakeSize(pixel_row_height * 1.75f, pixel_row_height * 1.5f);
-	
-	int stops[6] = {0, 192, 256, 320, 384, 448};
-	unsigned i;
-	
-	rowRange = NSMakeRange(0,n_rows);
-
-	rowText =   [[NSMutableArray alloc] initWithCapacity:n_rows];   // alloc retains
-	rowKey =	[[NSMutableArray alloc] initWithCapacity:n_rows];   // alloc retains
-	rowColor =	[[NSMutableArray alloc] initWithCapacity:n_rows];   // alloc retains
-	
-	for (i = 0; i < n_rows; i++)
-	{
-		[rowText addObject:@"."];
-		[rowKey addObject:[NSString stringWithFormat:@"%d",i]];
-		[rowColor addObject:[OOColor yellowColor]];
-		rowPosition[i].x = 0.0f;
-		rowPosition[i].y = size_in_pixels.height - (pixel_row_start + i * pixel_row_height);
-		rowAlignment[i] = GUI_ALIGN_LEFT;
+		pixel_text_size = NSMakeSize(0.9f * pixel_row_height, pixel_row_height);	// main gui has 18x20 characters
+		
+		pixel_title_size = NSMakeSize(pixel_row_height * 1.75f, pixel_row_height * 1.5f);
+		
+		int stops[6] = {0, 192, 256, 320, 384, 448};
+		unsigned i;
+		
+		rowRange = NSMakeRange(0,n_rows);
+		
+		rowText =   [[NSMutableArray alloc] initWithCapacity:n_rows];   // alloc retains
+		rowKey =	[[NSMutableArray alloc] initWithCapacity:n_rows];   // alloc retains
+		rowColor =	[[NSMutableArray alloc] initWithCapacity:n_rows];   // alloc retains
+		
+		for (i = 0; i < n_rows; i++)
+		{
+			[rowText addObject:@"."];
+			[rowKey addObject:[NSString stringWithFormat:@"%d",i]];
+			[rowColor addObject:[OOColor yellowColor]];
+			rowPosition[i].x = 0.0f;
+			rowPosition[i].y = size_in_pixels.height - (pixel_row_start + i * pixel_row_height);
+			rowAlignment[i] = GUI_ALIGN_LEFT;
+		}
+		
+		for (i = 0; i < n_columns; i++)
+		{
+			tabStops[i] = stops[i];
+		}
+		
+		title = @"";
+		
+		textColor = [[OOColor yellowColor] retain];
+		
+		drawPosition = make_vector(0.0f, 0.0f, 640.0f);
+		
+		backgroundSpecial = GUI_BACKGROUND_SPECIAL_NONE;
 	}
-	
-	for (i = 0; i < n_columns; i++)
-	{
-		tabStops[i] = stops[i];
-	}
-	
-	title = @"";
-	
-	textColor = [[OOColor yellowColor] retain];
-	
-	drawPosition = make_vector(0.0f, 0.0f, 640.0f);
-
 	return self;
 }
 
@@ -382,6 +389,19 @@ static BOOL _refreshStarChart = NO;
 }
 
 
+- (OOGUIRow) rowForKey:(NSString*)key
+{
+	for (unsigned i=0;i<[rowKey count];i++)
+	{
+		if ([key isEqualToString:[rowKey objectAtIndex:i]])
+		{
+			return (OOGUIRow)i;
+		}
+	}
+	return -1;
+}
+
+
 - (NSString*) keyForRow:(OOGUIRow)row
 {
 	if (RowInRange(row, rowRange))
@@ -391,7 +411,7 @@ static BOOL _refreshStarChart = NO;
 }
 
 
-- (int) selectedRow
+- (OOGUIRow) selectedRow
 {
 	if (RowInRange(selectedRow, selectableRange))
 		return selectedRow;
@@ -418,7 +438,7 @@ static BOOL _refreshStarChart = NO;
 
 - (BOOL) setNextRow:(int) direction
 {
-	int row = selectedRow + direction;
+	OOGUIRow row = selectedRow + direction;
 	while (RowInRange(row, selectableRange))
 	{
 		if (![[rowKey objectAtIndex:row] isEqual:GUI_KEY_SKIP])
@@ -434,7 +454,7 @@ static BOOL _refreshStarChart = NO;
 
 - (BOOL) setFirstSelectableRow
 {
-	int row = selectableRange.location;
+	NSUInteger row = selectableRange.location;
 	while (RowInRange(row, selectableRange))
 	{
 		if (![[rowKey objectAtIndex:row] isEqual:GUI_KEY_SKIP])
@@ -451,7 +471,7 @@ static BOOL _refreshStarChart = NO;
 
 - (BOOL) setLastSelectableRow
 {
-	int row = selectableRange.location + selectableRange.length - 1;
+	NSUInteger row = selectableRange.location + selectableRange.length - 1;
 	while (RowInRange(row, selectableRange))
 	{
 		if (![[rowKey objectAtIndex:row] isEqual:GUI_KEY_SKIP])
@@ -580,9 +600,9 @@ static BOOL _refreshStarChart = NO;
 }
 
 
-- (int) addLongText:(NSString *)str
-	  startingAtRow:(OOGUIRow)row
-			  align:(OOGUIAlignment)alignment
+- (OOGUIRow) addLongText:(NSString *)str
+		   startingAtRow:(OOGUIRow)row
+				   align:(OOGUIAlignment)alignment
 {
 
 	if ([str rangeOfString:@"\n"].location != NSNotFound)
@@ -640,7 +660,7 @@ static BOOL _refreshStarChart = NO;
 }
 
 
-- (NSArray *) getLastLines;	// text, colour, fade time - text, colour, fade time
+- (NSArray *) getLastLines	// text, colour, fade time - text, colour, fade time
 {
 	if (n_rows <1) return nil;
 	
@@ -771,7 +791,7 @@ static BOOL _refreshStarChart = NO;
 	if([items count] == 0)
 		return;
 	
-	unsigned n_items = [items count];
+	NSUInteger n_items = [items count];
 	if ((item_keys)&&([item_keys count] != n_items))
 	{
 		// throw exception
@@ -877,14 +897,14 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		if (haveWidth)
 		{
 			// Width specified, but not height; preserve aspect ratio.
-			OOCGFloat ratio = originalDimensions.height / originalDimensions.width;
+			CGFloat ratio = originalDimensions.height / originalDimensions.width;
 			size.width = specifiedWidth;
 			size.height = ratio * size.width;
 		}
 		else if (haveHeight)
 		{
 			// Height specified, but not width; preserve aspect ratio.
-			OOCGFloat ratio = originalDimensions.width / originalDimensions.height;
+			CGFloat ratio = originalDimensions.width / originalDimensions.height;
 			size.height = specifiedHeight;
 			size.width = ratio * size.height;
 		}
@@ -899,9 +919,57 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 }
 
 
+- (void) setBackgroundTextureSpecial:(OOGUIBackgroundSpecial)spec withBackground:(BOOL)withBackground
+{
+	if (withBackground) 
+	{
+		NSDictionary *bgDescriptor = nil;
+		OOGalaxyID galaxy_number = [PLAYER galaxyNumber];
+
+		switch (spec) 
+		{
+		case GUI_BACKGROUND_SPECIAL_SHORT:
+			bgDescriptor = [UNIVERSE screenTextureDescriptorForKey:@"short_range_chart_mission"];
+			if (bgDescriptor == nil) 
+			{
+				bgDescriptor = [UNIVERSE screenTextureDescriptorForKey:@"short_range_chart"];
+			}
+			break;
+		case GUI_BACKGROUND_SPECIAL_LONG:
+		case GUI_BACKGROUND_SPECIAL_LONG_ANA_SHORTEST:
+		case GUI_BACKGROUND_SPECIAL_LONG_ANA_QUICKEST:
+			bgDescriptor = [UNIVERSE screenTextureDescriptorForKey:[NSString stringWithFormat:@"long_range_chart%d_mission", galaxy_number+1]];
+			if (bgDescriptor == nil) 
+			{
+				bgDescriptor = [UNIVERSE screenTextureDescriptorForKey:@"long_range_chart_mission"];
+				if (bgDescriptor == nil) 
+				{
+					bgDescriptor = [UNIVERSE screenTextureDescriptorForKey:[NSString stringWithFormat:@"long_range_chart%d", galaxy_number+1]];
+					if (bgDescriptor == nil) 
+					{
+						bgDescriptor = [UNIVERSE screenTextureDescriptorForKey:@"long_range_chart"];
+						
+					}
+				}
+			}
+			break;
+		case GUI_BACKGROUND_SPECIAL_NONE:
+			break;
+		}
+		if (bgDescriptor != nil)
+		{
+			[self setBackgroundTextureDescriptor:bgDescriptor];
+		}
+	}
+	backgroundSpecial = spec;
+	[self refreshStarChart];
+}
+
+
 - (BOOL) setBackgroundTextureDescriptor:(NSDictionary *)descriptor
 {
 	[backgroundSprite autorelease];
+	backgroundSpecial = GUI_BACKGROUND_SPECIAL_NONE; // reset
 	backgroundSprite = NewTextureSpriteWithDescriptor(descriptor);
 	return backgroundSprite != nil;
 }
@@ -957,6 +1025,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	if (result == nil)
 	{
 		NSString *name = OOStringFromJSValue(context, value);
+
 		if (name != nil)
 		{
 			result = [NSDictionary dictionaryWithObject:name forKey:@"name"];
@@ -977,7 +1046,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 }
 
 
-- (void) setStatusPage:(int)pageNum
+- (void) setStatusPage:(NSUInteger)pageNum
 {
 	if (pageNum==0) 
 		statusPage=1;
@@ -986,17 +1055,24 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 }
 
 
-- (void) drawEqptList:(NSArray *)eqptList z:(GLfloat)z 
+- (NSUInteger) statusPage
+{
+	return statusPage;
+}
+
+
+- (void) drawEquipmentList:(NSArray *)eqptList z:(GLfloat)z
 {
 	if ([eqptList count] == 0) return;
 	
-	int				first_row = STATUS_EQUIPMENT_FIRST_ROW;
-	int				items_per_column = STATUS_EQUIPMENT_MAX_ROWS;
+	OOGUIRow		firstRow = STATUS_EQUIPMENT_FIRST_ROW;
+	NSUInteger		itemsPerColumn = STATUS_EQUIPMENT_MAX_ROWS;
 
-	int				first_y = 40;	// first_row =10 :-> 40  - first_row=11 -> 24 etc...
-	int				items_count = [eqptList count];
-	int				page_count=1;
-	int				i, start;
+	NSInteger		firstY = 40;	// firstRow =10 :-> 40  - firstRow=11 -> 24 etc...
+	NSUInteger		eqptCount = [eqptList count];
+	NSUInteger		pageCount = 1;
+	NSUInteger		i;
+	NSInteger		start;
 	NSArray			*info = nil;
 	NSString		*name = nil;
 	BOOL			damaged;
@@ -1004,16 +1080,20 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	// Paging calculations. Assuming 10 lines we get - one page:20 items per page (ipp)
 	// two pages: 18 ipp - three+ pages:  1st & last 18pp,  middle pages 16ipp
 	
-	i = items_per_column * 2 + 2;
-	if (items_count > i) // don't fit in one page?
+	i = itemsPerColumn * 2 + 2;
+	if (eqptCount > i) // don't fit in one page?
 	{
-		[UNIVERSE setDisplayCursor: YES];
-		i = items_per_column * 4; // total items in the first and last pages
-		items_per_column--; // for all the middle pages.
-		if (items_count <= i) // two pages
+		[[UNIVERSE gameController] setMouseInteractionModeForUIWithMouseInteraction:YES];
+		 
+		i = itemsPerColumn * 4; // total items in the first and last pages
+		itemsPerColumn--; // for all the middle pages.
+		if (eqptCount <= i) // two pages
 		{
-			page_count++;
-			if (statusPage == 1) start=0;
+			pageCount++;
+			if (statusPage == 1)
+			{
+				start = 0;
+			}
 			else
 			{
 				statusPage = 2;
@@ -1022,64 +1102,66 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		}
 		else // three or more
 		{
-			page_count = ceil((float)(items_count-i)/(items_per_column*2)) + 2;
-			statusPage = OOClampInteger(statusPage, 1, page_count);
-			start = (statusPage == 1) ? 0 : (statusPage-1) * items_per_column * 2 + 2;
+			pageCount = ceil((float)(eqptCount-i)/(itemsPerColumn*2)) + 2;
+			statusPage = (NSInteger)OOClampInteger(statusPage, 1, pageCount);
+			start = (statusPage == 1) ? 0 : (statusPage-1) * itemsPerColumn * 2 + 2;
 		}
 	}
 	else
 	{
-		statusPage=page_count; // one page
-		start=0;
+		statusPage = pageCount; // one page
+		start = 0;
 	}
 	
 	if (statusPage > 1)
 	{
-		[self setColor:[OOColor greenColor] forRow:first_row];
-		[self setArray:[NSArray arrayWithObjects:DESC(@"gui-back"),  @"", @" <-- ",nil] forRow:first_row];
-		[self setKey:GUI_KEY_OK forRow:first_row];
-		first_y -= 16; // start 1 row down!
-		if (statusPage == page_count)
+		[self setColor:[OOColor greenColor] forRow:firstRow];
+		[self setArray:[NSArray arrayWithObjects:DESC(@"gui-back"),  @"", @" <-- ",nil] forRow:firstRow];
+		[self setKey:GUI_KEY_OK forRow:firstRow];
+		firstY -= 16; // start 1 row down!
+		if (statusPage == pageCount)
 		{
-			[self setSelectableRange:NSMakeRange(first_row, 1)];
-			[self setSelectedRow:first_row];
+			[self setSelectableRange:NSMakeRange(firstRow, 1)];
+			[self setSelectedRow:firstRow];
 		}
 	}
-	if (statusPage < page_count)
+	if (statusPage < pageCount)
 	{
-		[self setColor:[OOColor greenColor] forRow:first_row + STATUS_EQUIPMENT_MAX_ROWS];
-		[self setArray:[NSArray arrayWithObjects:DESC(@"gui-more"),  @"", @" --> ",nil] forRow:first_row + STATUS_EQUIPMENT_MAX_ROWS];
-		[self setKey:GUI_KEY_OK forRow:first_row + STATUS_EQUIPMENT_MAX_ROWS];
+		[self setColor:[OOColor greenColor] forRow:firstRow + STATUS_EQUIPMENT_MAX_ROWS];
+		[self setArray:[NSArray arrayWithObjects:DESC(@"gui-more"),  @"", @" --> ",nil] forRow:firstRow + STATUS_EQUIPMENT_MAX_ROWS];
+		[self setKey:GUI_KEY_OK forRow:firstRow + STATUS_EQUIPMENT_MAX_ROWS];
 		if (statusPage == 1)
 		{
-			[self setSelectableRange:NSMakeRange(first_row + STATUS_EQUIPMENT_MAX_ROWS, 1)];
-			[self setSelectedRow:first_row + STATUS_EQUIPMENT_MAX_ROWS];
+			[self setSelectableRange:NSMakeRange(firstRow + STATUS_EQUIPMENT_MAX_ROWS, 1)];
+			[self setSelectedRow:firstRow + STATUS_EQUIPMENT_MAX_ROWS];
 		}
 	}
-	if (statusPage > 1 && statusPage < page_count)
+	if (statusPage > 1 && statusPage < pageCount)
 	{
-		[self setSelectableRange:NSMakeRange(first_row, first_row + STATUS_EQUIPMENT_MAX_ROWS)];
+		[self setSelectableRange:NSMakeRange(firstRow, firstRow + STATUS_EQUIPMENT_MAX_ROWS)];
 		// default selected row to 'More -->' if we are looking at one of the middle pages
-		if ([self selectedRow] == -1)  [self setSelectedRow:first_row + STATUS_EQUIPMENT_MAX_ROWS];
+		if ([self selectedRow] == -1)  [self setSelectedRow:firstRow + STATUS_EQUIPMENT_MAX_ROWS];
 	}
 
-	if (statusPage == 1 || statusPage == page_count) items_per_column++;
-	items_count = OOClampInteger(items_count, 1, start + items_per_column * 2);
-	for (i=start; i < items_count; i++)
+	if (statusPage == 1 || statusPage == pageCount) itemsPerColumn++;
+	eqptCount = (NSInteger)OOClampInteger(eqptCount, 1, start + itemsPerColumn * 2);
+	for (i = start; i < eqptCount; i++)
 	{
-		info = [eqptList objectAtIndex:i];
+		info = [eqptList oo_arrayAtIndex:i];
 		name = [info oo_stringAtIndex:0];
+		if([name length] > 42)  name = [[name substringToIndex:40] stringByAppendingString:@"..."];
+		
 		damaged = ![info oo_boolAtIndex:1];
-		if (damaged)  glColor4f (1.0f, 0.5f, 0.0f, 1.0f); // Damaged items  show up  orange.
-		else glColor4f (1.0f, 1.0f, 0.0f, 1.0f);	// Normal items in yellow.
-		if([name length] > 42)  name =[[name substringToIndex:40] stringByAppendingString:@"..."];
-		if (i - start < items_per_column)
+		if (damaged)  glColor4f (1.0f, 0.5f, 0.0f, 1.0f); // Damaged items show up orange.
+		else  glColor4f (1.0f, 1.0f, 0.0f, 1.0f);	// Normal items in yellow.
+		
+		if (i - start < itemsPerColumn)
 		{
-			OODrawString (name, -220, first_y - 16 * (i - start), z, NSMakeSize(15, 15));
+			OODrawString(name, -220, firstY - 16 * (NSInteger)(i - start), z, NSMakeSize(15, 15));
 		}
 		else
 		{
-			OODrawString (name, 50, first_y - 16 * (i - items_per_column - start), z, NSMakeSize(15, 15));
+			OODrawString(name, 50, firstY - 16 * (NSInteger)(i - itemsPerColumn - start), z, NSMakeSize(15, 15));
 		}
 	}
 }
@@ -1095,6 +1177,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	{
 		[backgroundSprite blitBackgroundCentredToX:x Y:y Z:z alpha:1.0f];
 	}
+	
 }
 
 
@@ -1118,15 +1201,20 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		
 		if (self == [UNIVERSE gui])
 		{
-			if ([player guiScreen] == GUI_SCREEN_SHORT_RANGE_CHART)
+			if ([player guiScreen] == GUI_SCREEN_SHORT_RANGE_CHART || backgroundSpecial == GUI_BACKGROUND_SPECIAL_SHORT)
+			{
 				[self drawStarChart:x - 0.5f * size_in_pixels.width :y - 0.5f * size_in_pixels.height :z :alpha];
-			if ([player guiScreen] == GUI_SCREEN_LONG_RANGE_CHART)
+			}
+			if ([player guiScreen] == GUI_SCREEN_LONG_RANGE_CHART || 
+					backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG || 
+					backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG_ANA_QUICKEST ||
+					backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG_ANA_SHORTEST)
 			{
 				[self drawGalaxyChart:x - 0.5f * size_in_pixels.width :y - 0.5f * size_in_pixels.height :z :alpha];
 			}
 			if ([player guiScreen] == GUI_SCREEN_STATUS)
 			{
-				[self drawEqptList:[player equipmentList] z:z ];
+				[self drawEquipmentList:[player equipmentList] z:z];
 			}
 		}
 		
@@ -1162,8 +1250,8 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		
 		GLfloat h1 = 3.0f;
 		GLfloat h3 = 9.0f;
-		OOGL(glColor4f(0.2f, 0.2f, 1.0f, 0.5f));
-		OOGL(glLineWidth(2.0f));
+		OOGL(glColor4f(0.6f, 0.6f, 1.0f, 0.5f)); // original value of (0.2f, 0.2f, 1.0f, 0.5f) too dark - Nikos 20130616
+		OOGL(GLScaledLineWidth(2.0f));
 		
 		cursor_x += x;
 		cursor_y += y;
@@ -1175,7 +1263,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 			glVertex3f((float)cursor_x, (float)cursor_y - h1, z);	glVertex3f((float)cursor_x, (float)cursor_y - h3, z);
 			glVertex3f((float)cursor_x, (float)cursor_y + h1, z);	glVertex3f((float)cursor_x, (float)cursor_y + h3, z);
 		OOGLEND();
-		OOGL(glLineWidth(1.0f));
+		OOGL(GLScaledLineWidth(1.0f));
 		
 	}
 	
@@ -1276,7 +1364,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 				}
 				if (i == (unsigned)selectedRow)
 				{
-					NSRect block = OORectFromString(text, x + rowPosition[i].x + 2, y + rowPosition[i].y + 2, characterSize);
+					NSRect		block = OORectFromString(text, x + rowPosition[i].x + 2, y + rowPosition[i].y + 2, characterSize);
 					OOGL(glColor4f(1.0f, 0.0f, 0.0f, row_alpha));	// red
 					OOGLBEGIN(GL_QUADS);
 						glVertex3f(block.origin.x,						block.origin.y,						z);
@@ -1309,27 +1397,58 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		}
 		if ([[rowText objectAtIndex:i] isKindOfClass:[NSArray class]])
 		{
-			unsigned j;
-			NSArray*	array = (NSArray *)[rowText objectAtIndex:i];
-			unsigned max_columns=[array count] < n_columns ? [array count] : n_columns;
-			BOOL isLeftAligned;
+			NSArray		*array = [rowText oo_arrayAtIndex:i];
+			NSUInteger	j, max_columns = MIN([array count], n_columns);
+			BOOL		isLeftAligned;
 			
-			for (j = 0; j < max_columns ; j++)
+			for (j = 0; j < max_columns; j++)
 			{
 				NSString*   text = [array oo_stringAtIndex:j];
 				if ([text length] != 0)
 				{
-					isLeftAligned=tabStops[j]>=0;
+					isLeftAligned = tabStops[j] >= 0;
 					rowPosition[i].x = abs(tabStops[j]);
-					NSRect block = OORectFromString(text, x + rowPosition[i].x + 2, y + rowPosition[i].y + 2, characterSize);
+					
+					// we don't want to highlight leading space(s)
+					NSString	*hilitedText = [text stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@" "]];
+					NSRange		txtRange = [text rangeOfString:hilitedText];
+					unsigned	leadingSpaces = 0;
+					
+					if (EXPECT_NOT(txtRange.location == NSNotFound))
+					{
+						// This never happens!
+						hilitedText = text;
+					}
+					else if (txtRange.location > 0)
+					{
+						// padded string!
+						NSRect charBlock = OORectFromString(@" ", 0, 0, characterSize);
+						leadingSpaces = charBlock.size.width * txtRange.location;
+						
+						// if we're displaying commodity-quantity-none, let's try and be pixel perfect!
+						NSString *qtyNone = DESC(@"commodity-quantity-none");
+						txtRange = [hilitedText rangeOfString:qtyNone];
+						
+						if (txtRange.location == 0) // bingo!
+						{
+							rowPosition[i].x += OORectFromString(@"0", 0, 0, characterSize).size.width - OORectFromString(qtyNone, 0, 0, characterSize).size.width;
+						}
+					}
+					
+					// baseline text rect, needed for correct highlight positioning.
+					NSRect		block = OORectFromString(text, x + rowPosition[i].x + 2, y + rowPosition[i].y + 2, characterSize);
+					
 					if(!isLeftAligned)
 					{
-						rowPosition[i].x -=block.size.width+6;
-						block = OORectFromString(text, x + rowPosition[i].x, y + rowPosition[i].y + 2, characterSize);
-						block.size.width+=2;
-					}					
+						rowPosition[i].x -= block.size.width + 3;
+					}
+					block = OORectFromString(hilitedText, x + rowPosition[i].x + 1 + leadingSpaces, y + rowPosition[i].y + 2, characterSize);
+					block.size.width += 3;
+						
+					
 					if (i == (unsigned)selectedRow)
 					{
+					
 						OOGL(glColor4f(1.0f, 0.0f, 0.0f, row_alpha));	// red
 						OOGLBEGIN(GL_QUADS);
 							glVertex3f(block.origin.x,						block.origin.y,						z);
@@ -1391,7 +1510,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		cursor_coordinates = galaxy_coordinates;	// home
 	
 	// get a list of systems marked as contract destinations
-	NSArray* markedDestinations = [player markedDestinations];
+	NSDictionary* markedDestinations = [player markedDestinations];
 	
 	// get present location
 	cu = NSMakePoint((float)(hscale*galaxy_coordinates.x+hoffset),(float)(vscale*galaxy_coordinates.y+voffset));
@@ -1400,13 +1519,13 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	{
 		// draw fuel range circle
 		OOGL(glColor4f(0.0f, 1.0f, 0.0f, alpha));	//	green
-		OOGL(glLineWidth(2.0f));
+		OOGL(GLScaledLineWidth(2.0f));
 		GLDrawOval(x + cu.x, y + cu.y, z, NSMakeSize((float)(fuel*hscale), 2*(float)(fuel*vscale)), 5);
 	}
 		
 	// draw marks and stars
 	//
-	OOGL(glLineWidth(1.5f));
+	OOGL(GLScaledLineWidth(1.5f));
 	OOGL(glColor4f(1.0f, 1.0f, 0.75f, alpha));	// pale yellow
 
 	for (i = 0; i < 256; i++)
@@ -1424,16 +1543,12 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		
 		if ((dx < 20)&&(dy < 38))
 		{
-			if ([markedDestinations oo_boolAtIndex:i])	// is marked
+			NSArray *markers = [markedDestinations objectForKey:[NSNumber numberWithInt:i]];
+			if (markers != nil)	// is marked
 			{
-				GLfloat mark_size = 0.5f * blob_size + 2.5f;
-				OOGL(glColor4f(1.0f, 0.0f, 0.0f, alpha));	// red
-				OOGLBEGIN(GL_LINES);
-					glVertex3f(x + star.x - mark_size,	y + star.y - mark_size,	z);
-					glVertex3f(x + star.x + mark_size,	y + star.y + mark_size,	z);
-					glVertex3f(x + star.x - mark_size,	y + star.y + mark_size,	z);
-					glVertex3f(x + star.x + mark_size,	y + star.y - mark_size,	z);
-				OOGLEND();
+				GLfloat base_size = 0.5f * blob_size + 2.5f;
+				[self drawSystemMarkers:markers atX:x+star.x andY:y+star.y andZ:z withAlpha:alpha andScale:base_size];
+
 				OOGL(glColor4f(1.0f, 1.0f, 0.75f, alpha));	// pale yellow
 			}
 			GLDrawFilledOval(x + star.x, y + star.y, z, NSMakeSize(blob_size,blob_size), 15);
@@ -1504,6 +1619,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	
 	int targetIdx = -1;
 	struct saved_system *sys;
+	NSSize chSize = NSMakeSize(pixel_row_height,pixel_row_height);
 	
 	for (i = 0; i < num_nearby_systems; i++)
 	{
@@ -1511,17 +1627,19 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		
 		star.x = (float)(sys->seed_d * hscale + hoffset);
 		star.y = (float)(sys->seed_b * vscale + voffset);
-		if (sys->seed_d == target.d && sys->seed_b == target.b)
-			if ([sys->p_name isEqualToString:targetName]) targetIdx = i; // Distinguish between overlapping systems! (e.g. Divees & Tezabi in galaxy 5)
-		
-		if (![player showInfoFlag])
+		if (sys->seed_d == target.d && sys->seed_b == target.b	// same place as target system?
+			&& [sys->p_name isEqualToString:targetName])		// not overlapping twin? (example: Divees & Tezabi in galaxy 5)
 		{
-			OODrawString(sys->p_name, x + star.x + 2.0, y + star.y, z, NSMakeSize(pixel_row_height,pixel_row_height));
+			 targetIdx = i;		// we have a winner!
 		}
-		else
+		
+		if (![player showInfoFlag])	// System's name
 		{
-			if ( sys->gov >= 0 )	// Not a nova? Show the info.
-				OODrawPlanetInfo(sys->gov, sys->eco, sys->tec, x + star.x + 2.0, y + star.y + 2.0, z, NSMakeSize(pixel_row_height,pixel_row_height));
+			OODrawString(sys->p_name, x + star.x + 2.0, y + star.y, z, chSize);
+		}
+		else if (EXPECT(sys->gov >= 0))	// Not a nova? Show the info.
+		{
+			OODrawPlanetInfo(sys->gov, sys->eco, sys->tec, x + star.x + 2.0, y + star.y + 2.0, z, chSize);
 		}
 	}
 	
@@ -1535,22 +1653,20 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		
 		if (![player showInfoFlag])
 		{
-			OODrawHilightedString(sys->p_name, x + star.x + 2.0, y + star.y, z, NSMakeSize(pixel_row_height,pixel_row_height));
+			OODrawHilightedString(sys->p_name, x + star.x + 2.0, y + star.y, z, chSize);
 		}
-		else
+		else if (sys->gov >= 0)	// Not a nova? Show the info.
 		{
-			if ( sys->gov >= 0 )	// Not a nova? Show the info.
-				OODrawHilightedPlanetInfo(sys->gov, sys->eco, sys->tec, x + star.x + 2.0, y + star.y + 2.0, z,
-										  NSMakeSize(pixel_row_height,pixel_row_height));
+			OODrawHilightedPlanetInfo(sys->gov, sys->eco, sys->tec, x + star.x + 2.0, y + star.y + 2.0, z, chSize);
 		}
 	}
 	
-	// draw cross-hairs over current location
+	// draw crosshairs over current location
 	//
 	OOGL(glColor4f(0.0f, 1.0f, 0.0f, alpha));	//	green
 	[self drawCrossHairsWithSize:14 x:x + cu.x y:y + cu.y z:z];
 	
-	// draw cross hairs over cursor
+	// draw crosshairs over cursor
 	//
 	OOGL(glColor4f(1.0f, 0.0f, 0.0f, alpha));	//	red
 	cu = NSMakePoint((float)(hscale*cursor_coordinates.x+hoffset),(float)(vscale*cursor_coordinates.y+voffset));
@@ -1558,6 +1674,79 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 }
 
 
+- (void) drawSystemMarkers:(NSArray *)markers atX:(GLfloat)x andY:(GLfloat)y andZ:(GLfloat)z withAlpha:(GLfloat)alpha andScale:(GLfloat)scale
+{
+	NSEnumerator *mEnum; 
+	NSDictionary *marker;
+	for (mEnum = [markers objectEnumerator]; (marker = [mEnum nextObject]); )
+	{
+		[self drawSystemMarker:marker atX:x andY:y andZ:z withAlpha:alpha andScale:scale];
+	}
+}
+
+
+- (void) drawSystemMarker:(NSDictionary *)marker atX:(GLfloat)x andY:(GLfloat)y andZ:(GLfloat)z withAlpha:(GLfloat)alpha andScale:(GLfloat)scale
+{
+	NSString *colorDesc = [marker oo_stringForKey:@"markerColor" defaultValue:@"redColor"];
+	OORGBAComponents color = [[OOColor colorWithDescription:colorDesc] rgbaComponents];
+	
+	OOGL(glColor4f(color.r, color.g, color.b, alpha));	// red
+	GLfloat mark_size = [marker oo_floatForKey:@"markerScale" defaultValue:1.0];
+	if (mark_size > 2.0)
+	{
+		mark_size = 2.0;
+	}
+	else if (mark_size < 0.5)
+	{
+		mark_size = 0.5;
+	}
+	mark_size *= scale;
+
+	NSString *shape = [marker oo_stringForKey:@"markerShape" defaultValue:@"MARKER_X"];
+
+	OOGLBEGIN(GL_LINES);
+	if ([shape isEqualToString:@"MARKER_X"])
+	{
+		glVertex3f(x - mark_size,	y - mark_size,	z);
+		glVertex3f(x + mark_size,	y + mark_size,	z);
+		glVertex3f(x - mark_size,	y + mark_size,	z);
+		glVertex3f(x + mark_size,	y - mark_size,	z);
+	}
+	else if ([shape isEqualToString:@"MARKER_PLUS"])
+	{
+		mark_size *= 1.4; // match volumes
+		glVertex3f(x,	y - mark_size,	z);
+		glVertex3f(x,	y + mark_size,	z);
+		glVertex3f(x - mark_size,	y,	z);
+		glVertex3f(x + mark_size,	y,	z);
+	}
+	else if ([shape isEqualToString:@"MARKER_SQUARE"])
+	{
+		glVertex3f(x - mark_size,	y - mark_size,	z);
+		glVertex3f(x - mark_size,	y + mark_size,	z);
+		glVertex3f(x - mark_size,	y + mark_size,	z);
+		glVertex3f(x + mark_size,	y + mark_size,	z);
+		glVertex3f(x + mark_size,	y + mark_size,	z);
+		glVertex3f(x + mark_size,	y - mark_size,	z);
+		glVertex3f(x + mark_size,	y - mark_size,	z);
+		glVertex3f(x - mark_size,	y - mark_size,	z);
+	}
+	else if ([shape isEqualToString:@"MARKER_DIAMOND"])
+	{
+		mark_size *= 1.4; // match volumes
+		glVertex3f(x,	y - mark_size,	z);
+		glVertex3f(x - mark_size,	y,	z);
+		glVertex3f(x - mark_size,	y,	z);
+		glVertex3f(x,	y + mark_size,	z);
+		glVertex3f(x,	y + mark_size,	z);
+		glVertex3f(x + mark_size,	y,	z);
+		glVertex3f(x + mark_size,	y,	z);
+		glVertex3f(x,	y - mark_size,	z);
+	}
+	OOGLEND();
+}
+
+
 - (Random_Seed) targetNextFoundSystem:(int)direction // +1 , 0 , -1
 {
 	Random_Seed sys = [PLAYER target_system_seed];
@@ -1607,7 +1796,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 
 - (void) drawGalaxyChart:(GLfloat)x :(GLfloat)y :(GLfloat)z :(GLfloat) alpha
 {
-	PlayerEntity*	player = PLAYER;
+	PlayerEntity	*player = PLAYER;
 	NSPoint			galaxy_coordinates = [player galaxy_coordinates];
 	NSPoint			cursor_coordinates = [player cursor_coordinates];
 	Random_Seed		galaxy_seed = [player galaxy_seed];
@@ -1615,7 +1804,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	double fuel = 35.0 * [player dialFuel];
 	
 	// get a list of systems marked as contract destinations
-	NSArray		*markedDestinations = [player markedDestinations];
+	NSDictionary		*markedDestinations = [player markedDestinations];
 	
 	BOOL		*systemsFound = [UNIVERSE systemsFound];
 	
@@ -1630,15 +1819,26 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	BOOL		routeExists = YES;
 	
 	int			i;
-	double		distance, time;
+	double		distance = 0.0, time = 0.0;
 	
-	if (showAdvancedNavArray) advancedNavArrayMode = [[UNIVERSE gameView] isCtrlDown] ? OPTIMIZED_BY_TIME : OPTIMIZED_BY_JUMPS;
+	if (showAdvancedNavArray)
+	{
+		advancedNavArrayMode = [[UNIVERSE gameView] isCtrlDown] ? OPTIMIZED_BY_TIME : OPTIMIZED_BY_JUMPS;
+	}
+	else if (backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG_ANA_SHORTEST)
+	{
+		advancedNavArrayMode = OPTIMIZED_BY_JUMPS;
+	}
+	else if (backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG_ANA_QUICKEST)
+	{
+		advancedNavArrayMode = OPTIMIZED_BY_TIME;
+	}
 	
 	if (advancedNavArrayMode != OPTIMIZED_BY_NONE && ![UNIVERSE strict] && [player hasEquipmentItem:@"EQ_ADVANCED_NAVIGATIONAL_ARRAY"])
 	{
-		int planetNumber = [UNIVERSE findSystemNumberAtCoords:galaxy_coordinates withGalaxySeed:galaxy_seed];
-		int destNumber = [UNIVERSE findSystemNumberAtCoords:cursor_coordinates withGalaxySeed:galaxy_seed];
-		NSDictionary* routeInfo = [UNIVERSE routeFromSystem:planetNumber toSystem:destNumber optimizedBy:advancedNavArrayMode];
+		OOSystemID planetNumber = [UNIVERSE findSystemNumberAtCoords:galaxy_coordinates withGalaxySeed:galaxy_seed];
+		OOSystemID destNumber = [UNIVERSE findSystemNumberAtCoords:cursor_coordinates withGalaxySeed:galaxy_seed];
+		NSDictionary *routeInfo = [UNIVERSE routeFromSystem:planetNumber toSystem:destNumber optimizedBy:advancedNavArrayMode];
 		
 		// if the ANA has been activated and we are in string input mode (i.e. planet search),
 		// get out of it so that distance and time data can be displayed
@@ -1660,24 +1860,34 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		time = distance * distance;
 	}
 	
+	OOGUITabSettings tab_stops;
+	tab_stops[0] = 0;
+	tab_stops[1] = 96;
+	tab_stops[2] = 288;
+	[self setTabStops:tab_stops];
+	NSString *targetSystemName = [[UNIVERSE getSystemName:[PLAYER target_system_seed]] retain];
+
 	if (routeExists)
 	{
 		// distance-f & est-travel-time-f are identical between short & long range charts in standard Oolite, however can be alterered separately via OXPs
-		[self setText:[NSString stringWithFormat:ExpandDescriptionForCurrentSystem(@"[long-range-chart-distance-f]"), distance] forRow:18];
+		NSString *travelDistLine = [NSString stringWithFormat:OOExpandKey(@"long-range-chart-distance-f"), distance];
 		NSString *travelTimeLine = @"";
 		if (advancedNavArrayMode != OPTIMIZED_BY_NONE && distance > 0)
 		{
-			travelTimeLine = [NSString stringWithFormat:ExpandDescriptionForCurrentSystem(@"[long-range-chart-est-travel-time-f]"), time];
+			travelTimeLine = [NSString stringWithFormat:OOExpandKey(@"long-range-chart-est-travel-time-f"), time];
 		}
-		[self setText:travelTimeLine forRow:19];
+			
+		[self setArray:[NSArray arrayWithObjects:targetSystemName, travelDistLine,travelTimeLine,nil] forRow:16];
 	}
 	else
 	{
-		[self setText:DESC(@"long-range-chart-system-unreachable")  forRow:18];
+		[self setArray:[NSArray arrayWithObjects:targetSystemName, DESC(@"long-range-chart-system-unreachable"), nil] forRow:16];
 	}
+	[targetSystemName release];
 	
+
 	OOGL(glColor4f(0.0f, 1.0f, 0.0f, alpha));	//	green
-	OOGL(glLineWidth(2.0f));
+	OOGL(GLScaledLineWidth(2.0f));
 	cu = NSMakePoint((float)(hscale*galaxy_coordinates.x+hoffset),(float)(vscale*galaxy_coordinates.y+voffset));
 	
 	if ([player hasHyperspaceMotor])
@@ -1698,22 +1908,17 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	
 	// draw marks
 	//
-	OOGL(glLineWidth(1.5f));
-	OOGL(glColor4f(1.0f, 0.0f, 0.0f, alpha));
+	OOGL(GLScaledLineWidth(1.5f));
 	for (i = 0; i < 256; i++)
 	{
 		g_seed = [UNIVERSE systemSeedForSystemNumber:i];
-		BOOL mark = [markedDestinations oo_boolAtIndex:i];
-		if (mark)
+		NSArray *markers = [markedDestinations objectForKey:[NSNumber numberWithInt:i]];
+		if (markers != nil)
 		{
 			star.x = (float)(g_seed.d * hscale + hoffset);
 			star.y = (float)(g_seed.b * vscale + voffset);
-			OOGLBEGIN(GL_LINES);
-				glVertex3f(x + star.x - 2.5f,	y + star.y - 2.5f,	z);
-				glVertex3f(x + star.x + 2.5f,	y + star.y + 2.5f,	z);
-				glVertex3f(x + star.x - 2.5f,	y + star.y + 2.5f,	z);
-				glVertex3f(x + star.x + 2.5f,	y + star.y - 2.5f,	z);
-			OOGLEND();
+			
+			[self drawSystemMarkers:markers atX:x+star.x andY:y+star.y andZ:z withAlpha:alpha andScale:2.5f];
 		}
 	}
 	
@@ -1739,7 +1944,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		
 	// draw found stars and captions
 	//
-	OOGL(glLineWidth(1.5f));
+	OOGL(GLScaledLineWidth(1.5f));
 	OOGL(glColor4f(0.0f, 1.0f, 0.0f, alpha));
 	int n_matches = 0, foundIndex = -1;
 	
@@ -1753,6 +1958,10 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	{
 		foundSystem = 0;
 	}
+	else if (backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG_ANA_SHORTEST || backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG_ANA_QUICKEST || backgroundSpecial == GUI_BACKGROUND_SPECIAL_LONG)
+	{
+		// do nothing at this stage
+	}
 	else
 	{
 		BOOL drawNames = n_matches < 4;
@@ -1798,9 +2007,8 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 // Advanced Navigation Array -- galactic chart route mapping - contributed by Nikos Barkas (another_commander).
 - (void) drawAdvancedNavArrayAtX:(float)x y:(float)y z:(float)z alpha:(float)alpha usingRoute:(NSDictionary *) routeInfo optimizedBy:(OORouteType) optimizeBy
 {
-	PlayerEntity	*player = PLAYER;
 	Random_Seed		g_seed, g_seed2;
-	int				i, j;
+	NSUInteger		i, j;
 	double			hscale = size_in_pixels.width / 256.0;
 	double			vscale = -1.0 * size_in_pixels.height / 512.0;
 	double			hoffset = 0.0f;
@@ -1821,7 +2029,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		star2.y = (float)(g_seed2.b * vscale + voffset);
 		double d = distanceBetweenPlanetPositions(g_seed.d, g_seed.b, g_seed2.d, g_seed2.b);
 		
-		if (d <= ([player fuelCapacity] / 10.0f))	// another_commander - Default to 7.0 LY.
+		if (d <= MAX_JUMP_RANGE)	// another_commander - Default to 7.0 LY.
 		{
 			glVertex3f(x+star.x, y+star.y, z);
 			glVertex3f(x+star2.x, y+star2.y, z);
@@ -1831,7 +2039,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 	
 	if (routeInfo)
 	{
-		int route_hops = [(NSArray *)[routeInfo objectForKey:@"route"] count] -1;
+		NSUInteger route_hops = [[routeInfo oo_arrayForKey:@"route"] count] - 1;
 		
 		if (optimizeBy == OPTIMIZED_BY_JUMPS)
 		{
@@ -1841,7 +2049,7 @@ static OOTextureSprite *NewTextureSpriteWithDescriptor(NSDictionary *descriptor)
 		{
 			OOGL(glColor4f(0.0f, 1.0f, 1.0f, alpha)); // Cyan for plotting routes optimized for time.
 		}
-		int loc;
+		OOSystemID loc;
 		for (i = 0; i < route_hops; i++)
 		{
 			loc = [[routeInfo objectForKey:@"route"] oo_intAtIndex:i];
