@@ -6,7 +6,7 @@ Singleton subclass of OOTexture representing the empty texture. Applying
 OONullTexture is equivalent to [OOTexture applyNone].
 
 
-Copyright (C) 2008-2011 Jens Ayton
+Copyright (C) 2008-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,6 @@ SOFTWARE.
 
 @interface OONullTexture: OOTexture
 
-+ (id) sharedNullTexture;
++ (OONullTexture *) sharedNullTexture;
 
 @end
