@@ -2,7 +2,7 @@
 	
 	OOTexture.m
 	
-	Copyright (C) 2007-2011 Jens Ayton and contributors
+	Copyright (C) 2007-2013 Jens Ayton and contributors
 	
 	Permission is hereby granted, free of charge, to any person obtaining a copy
 	of this software and associated documentation files (the "Software"), to deal
@@ -42,6 +42,27 @@
 #import "OOPixMap.h"
 
 
+NSString * const kOOTextureSpecifierNameKey					= @"name";
+NSString * const kOOTextureSpecifierSwizzleKey				= @"extract_channel";
+NSString * const kOOTextureSpecifierMinFilterKey			= @"min_filter";
+NSString * const kOOTextureSpecifierMagFilterKey			= @"mag_filter";
+NSString * const kOOTextureSpecifierNoShrinkKey				= @"no_shrink";
+NSString * const kOOTextureSpecifierRepeatSKey				= @"repeat_s";
+NSString * const kOOTextureSpecifierRepeatTKey				= @"repeat_t";
+NSString * const kOOTextureSpecifierCubeMapKey				= @"cube_map";
+NSString * const kOOTextureSpecifierAnisotropyKey			= @"anisotropy";
+NSString * const kOOTextureSpecifierLODBiasKey				= @"texture_LOD_bias";
+
+NSString * const kOOTextureSpecifierModulateColorKey		= @"color";
+NSString * const kOOTextureSpecifierIlluminationModeKey		= @"illumination_mode";
+NSString * const kOOTextureSpecifierSelfColorKey			= @"self_color";
+NSString * const kOOTextureSpecifierScaleFactorKey			= @"scale_factor";
+NSString * const kOOTextureSpecifierBindingKey				= @"binding";
+
+// Used only by "internal" specifiers from OOMakeTextureSpecifier.
+static NSString * const kOOTextureSpecifierFlagValueInternalKey = @"_oo_internal_flags";
+
+
 /*	Texture caching:
 	two and a half parallel caching mechanisms are used. sLiveTextureCache
 	tracks all live texture objects with cache keys, without retaining them
@@ -111,7 +132,7 @@ static NSString *sGlobalTraceContext = nil;
 
 + (id)textureWithName:(NSString *)name
 			 inFolder:(NSString*)directory
-			  options:(uint32_t)options
+			  options:(OOTextureFlags)options
 		   anisotropy:(GLfloat)anisotropy
 			  lodBias:(GLfloat)lodBias
 {
@@ -133,7 +154,7 @@ static NSString *sGlobalTraceContext = nil;
 	}
 	
 	noFNF = (options & kOOTextureNoFNFMessage) != 0;
-	options = OOApplyTetureOptionDefaults(options & ~kOOTextureNoFNFMessage);
+	options = OOApplyTextureOptionDefaults(options & ~kOOTextureNoFNFMessage);
 	
 	// Look for existing texture
 	key = [NSString stringWithFormat:@"%@%@%@:0x%.4X/%g/%g", directory ? directory : (NSString *)@"", directory ? @"/" : @"", name, options, anisotropy, lodBias];
@@ -173,14 +194,14 @@ static NSString *sGlobalTraceContext = nil;
 }
 
 
-+ (id) textureWithConfiguration:(id)configuration extraOptions:(uint32_t)extraOptions
++ (id) textureWithConfiguration:(id)configuration extraOptions:(OOTextureFlags)extraOptions
 {
 	NSString				*name = nil;
-	uint32_t				options = 0;
+	OOTextureFlags			options = 0;
 	GLfloat					anisotropy = 0.0f;
 	GLfloat					lodBias = 0.0f;
 	
-	if (!OOInterpretTextureSpecifier(configuration, &name, &options, &anisotropy, &lodBias))  return nil;
+	if (!OOInterpretTextureSpecifier(configuration, &name, &options, &anisotropy, &lodBias, NO))  return nil;
 	
 	return [self textureWithName:name inFolder:@"Textures" options:options | extraOptions anisotropy:anisotropy lodBias:lodBias];
 }
@@ -210,7 +231,7 @@ static NSString *sGlobalTraceContext = nil;
 	
 	OOTexture *result = [[[OOConcreteTexture alloc] initWithLoader:generator
 															   key:[generator cacheKey]
-														   options:OOApplyTetureOptionDefaults([generator textureOptions])
+														   options:OOApplyTextureOptionDefaults([generator textureOptions])
 														anisotropy:[generator anisotropy]
 														   lodBias:[generator lodBias]] autorelease];
 	
@@ -362,7 +383,7 @@ static NSString *sGlobalTraceContext = nil;
 {
 	if (trace && !_trace)
 	{
-		OOLog(@"texture.allocTrace.begin", @"Started tracing texture %p with retain count %u.", self, [self retainCount]);
+		OOLog(@"texture.allocTrace.begin", @"Started tracing texture %p with retain count %lu.", self, [self retainCount]);
 	}
 	_trace = trace;
 }
@@ -449,7 +470,7 @@ static NSString *sGlobalTraceContext = nil;
 	if (cacheKey == nil)  return;
 	
 	[sLiveTextureCache removeObjectForKey:cacheKey];
-	NSAssert([sRecentTextures objectForKey:cacheKey] != self, @"Texture retain count error."); //miscount in autorelease
+	NSAssert2([sRecentTextures objectForKey:cacheKey] != self, @"Texture retain count error for %@; cacheKey is %@.", self, cacheKey); //miscount in autorelease
 	// The following line is needed in order to avoid crashes when there's a 'texture retain count error'. Please do not delete. -- Kaks 20091221
 	[sRecentTextures removeObjectForKey:cacheKey]; // make sure there's no reference left inside sRecentTexture ( was a show stopper for 1.73)
 #endif
@@ -530,8 +551,8 @@ static NSString *sGlobalTraceContext = nil;
 {
 	if (_trace)
 	{
-		if (context)  OOLog(@"texture.allocTrace.retain", @"Texture %p retained (retain count -> %u) - %@.", self, [self retainCount] + 1, context);
-		else  OOLog(@"texture.allocTrace.retain", @"Texture %p retained.", self, [self retainCount] + 1);
+		if (context)  OOLog(@"texture.allocTrace.retain", @"Texture %p retained (retain count -> %lu) - %@.", self, [self retainCount] + 1, context);
+		else  OOLog(@"texture.allocTrace.retain", @"Texture %p retained  (retain count -> %lu).", self, [self retainCount] + 1);
 	}
 	
 	return [super retain];
@@ -542,8 +563,8 @@ static NSString *sGlobalTraceContext = nil;
 {
 	if (_trace)
 	{
-		if (context)  OOLog(@"texture.allocTrace.release", @"Texture %p released (retain count -> %u) - %@.", self, [self retainCount] - 1, context);
-		else  OOLog(@"texture.allocTrace.release", @"Texture %p released (retain count -> %u).", self, [self retainCount] - 1);
+		if (context)  OOLog(@"texture.allocTrace.release", @"Texture %p released (retain count -> %lu) - %@.", self, [self retainCount] - 1, context);
+		else  OOLog(@"texture.allocTrace.release", @"Texture %p released (retain count -> %lu).", self, [self retainCount] - 1);
 	}
 	
 	[super release];
@@ -568,7 +589,7 @@ static NSString *sGlobalTraceContext = nil;
 }
 
 
-- (void) release
+- (oneway void) release
 {
 	[self releaseInContext:sGlobalTraceContext];
 }
@@ -648,10 +669,10 @@ BOOL OOCubeMapsAvailable(void)
 }
 
 
-BOOL OOInterpretTextureSpecifier(id specifier, NSString **outName, uint32_t *outOptions, float *outAnisotropy, float *outLODBias)
+BOOL OOInterpretTextureSpecifier(id specifier, NSString **outName, OOTextureFlags *outOptions, float *outAnisotropy, float *outLODBias, BOOL ignoreExtract)
 {
 	NSString			*name = nil;
-	uint32_t			options = kOOTextureDefaultOptions;
+	OOTextureFlags		options = kOOTextureDefaultOptions;
 	float				anisotropy = kOOTextureDefaultAnisotropy;
 	float				lodBias = kOOTextureDefaultLODBias;
 	
@@ -661,42 +682,53 @@ BOOL OOInterpretTextureSpecifier(id specifier, NSString **outName, uint32_t *out
 	}
 	else if ([specifier isKindOfClass:[NSDictionary class]])
 	{
-		name = [specifier oo_stringForKey:@"name"];
+		name = [specifier oo_stringForKey:kOOTextureSpecifierNameKey];
 		if (name == nil)
 		{
 			OOLog(@"texture.load.noName", @"Invalid texture configuration dictionary (must specify name):\n%@", specifier);
 			return NO;
 		}
 		
-		NSString *filterString = [specifier oo_stringForKey:@"min_filter" defaultValue:@"default"];
-		if ([filterString isEqualToString:@"nearest"])  options |= kOOTextureMinFilterNearest;
-		else if ([filterString isEqualToString:@"linear"])  options |= kOOTextureMinFilterLinear;
-		else if ([filterString isEqualToString:@"mipmap"])  options |= kOOTextureMinFilterMipMap;
-		else  options |= kOOTextureMinFilterDefault;	// Covers "default"
-		
-		filterString = [specifier oo_stringForKey:@"mag_filter" defaultValue:@"default"];
-		if ([filterString isEqualToString:@"nearest"])  options |= kOOTextureMagFilterNearest;
-		else  options |= kOOTextureMagFilterLinear;	// Covers "default" and "linear"
-		
-		if ([specifier oo_boolForKey:@"no_shrink" defaultValue:NO])  options |= kOOTextureNoShrink;
-		if ([specifier oo_boolForKey:@"repeat_s" defaultValue:NO])  options |= kOOTextureRepeatS;
-		if ([specifier oo_boolForKey:@"repeat_t" defaultValue:NO])  options |= kOOTextureRepeatT;
-		if ([specifier oo_boolForKey:@"cube_map" defaultValue:NO])  options |= kOOTextureAllowCubeMap;
-		anisotropy = [specifier oo_floatForKey:@"anisotropy" defaultValue:kOOTextureDefaultAnisotropy];
-		lodBias = [specifier oo_floatForKey:@"texture_LOD_bias" defaultValue:kOOTextureDefaultLODBias];
-		
-		NSString *extractChannel = [specifier oo_stringForKey:@"extract_channel"];
-		if (extractChannel != nil)
+		int quickFlags = [specifier oo_intForKey:kOOTextureSpecifierFlagValueInternalKey defaultValue:-1];
+		if (quickFlags != -1)
 		{
-			if ([extractChannel isEqualToString:@"r"])  options |= kOOTextureExtractChannelR;
-			else if ([extractChannel isEqualToString:@"g"])  options |= kOOTextureExtractChannelG;
-			else if ([extractChannel isEqualToString:@"b"])  options |= kOOTextureExtractChannelB;
-			else if ([extractChannel isEqualToString:@"a"])  options |= kOOTextureExtractChannelA;
-			else
+			options = quickFlags;
+		}
+		else
+		{
+			NSString *filterString = [specifier oo_stringForKey:kOOTextureSpecifierMinFilterKey defaultValue:@"default"];
+			if ([filterString isEqualToString:@"nearest"])  options |= kOOTextureMinFilterNearest;
+			else if ([filterString isEqualToString:@"linear"])  options |= kOOTextureMinFilterLinear;
+			else if ([filterString isEqualToString:@"mipmap"])  options |= kOOTextureMinFilterMipMap;
+			else  options |= kOOTextureMinFilterDefault;	// Covers "default"
+			
+			filterString = [specifier oo_stringForKey:kOOTextureSpecifierMagFilterKey defaultValue:@"default"];
+			if ([filterString isEqualToString:@"nearest"])  options |= kOOTextureMagFilterNearest;
+			else  options |= kOOTextureMagFilterLinear;	// Covers "default" and "linear"
+			
+			if ([specifier oo_boolForKey:kOOTextureSpecifierNoShrinkKey defaultValue:NO])  options |= kOOTextureNoShrink;
+			if ([specifier oo_boolForKey:kOOTextureSpecifierRepeatSKey defaultValue:NO])  options |= kOOTextureRepeatS;
+			if ([specifier oo_boolForKey:kOOTextureSpecifierRepeatTKey defaultValue:NO])  options |= kOOTextureRepeatT;
+			if ([specifier oo_boolForKey:kOOTextureSpecifierCubeMapKey defaultValue:NO])  options |= kOOTextureAllowCubeMap;
+			
+			if (!ignoreExtract)
 			{
-				OOLogWARN(@"texture.load.extractChannel.invalid", @"Unknown value \"%@\" for extract_channel (should be \"r\", \"g\", \"b\" or \"a\").", extractChannel);
+				NSString *extractChannel = [specifier oo_stringForKey:@"extract_channel"];
+				if (extractChannel != nil)
+				{
+					if ([extractChannel isEqualToString:@"r"])  options |= kOOTextureExtractChannelR;
+					else if ([extractChannel isEqualToString:@"g"])  options |= kOOTextureExtractChannelG;
+					else if ([extractChannel isEqualToString:@"b"])  options |= kOOTextureExtractChannelB;
+					else if ([extractChannel isEqualToString:@"a"])  options |= kOOTextureExtractChannelA;
+					else
+					{
+						OOLogWARN(@"texture.load.extractChannel.invalid", @"Unknown value \"%@\" for extract_channel in specifier \"%@\" (should be \"r\", \"g\", \"b\" or \"a\").", extractChannel,specifier);
+					}
+				}
 			}
 		}
+		anisotropy = [specifier oo_floatForKey:@"anisotropy" defaultValue:kOOTextureDefaultAnisotropy];
+		lodBias = [specifier oo_floatForKey:@"texture_LOD_bias" defaultValue:kOOTextureDefaultLODBias];
 	}
 	else
 	{
@@ -716,7 +748,88 @@ BOOL OOInterpretTextureSpecifier(id specifier, NSString **outName, uint32_t *out
 }
 
 
-uint32_t OOApplyTetureOptionDefaults(uint32_t options)
+NSDictionary *OOMakeTextureSpecifier(NSString *name, OOTextureFlags options, float anisotropy, float lodBias, BOOL internal)
+{
+	NSMutableDictionary *result = [NSMutableDictionary dictionary];
+	
+	[result setObject:name forKey:kOOTextureSpecifierNameKey];
+	
+	if (anisotropy != kOOTextureDefaultAnisotropy)  [result oo_setFloat:anisotropy forKey:kOOTextureSpecifierAnisotropyKey];
+	if (lodBias != kOOTextureDefaultLODBias)  [result oo_setFloat:lodBias forKey:kOOTextureSpecifierLODBiasKey];
+	
+	if (internal)
+	{
+		[result oo_setUnsignedInteger:options forKey:kOOTextureSpecifierFlagValueInternalKey];
+	}
+	else
+	{
+		NSString *value = nil;
+		switch (options & kOOTextureMinFilterMask)
+		{
+			case kOOTextureMinFilterDefault:
+				break;
+				
+			case kOOTextureMinFilterNearest:
+				value = @"nearest";
+				break;
+				
+			case kOOTextureMinFilterLinear:
+				value = @"linear";
+				break;
+				
+			case kOOTextureMinFilterMipMap:
+				value = @"mipmap";
+				break;
+		}
+		if (value != nil)  [result setObject:value forKey:kOOTextureSpecifierNoShrinkKey];
+		
+		value = nil;
+		switch (options & kOOTextureMagFilterMask)
+		{
+			case kOOTextureMagFilterNearest:
+				value = @"nearest";
+				break;
+				
+			case kOOTextureMagFilterLinear:
+				break;
+		}
+		if (value != nil)  [result setObject:value forKey:kOOTextureSpecifierMagFilterKey];
+		
+		value = nil;
+		switch (options & kOOTextureExtractChannelMask)
+		{
+			case kOOTextureExtractChannelNone:
+				break;
+				
+			case kOOTextureExtractChannelR:
+				value = @"r";
+				break;
+				
+			case kOOTextureExtractChannelG:
+				value = @"g";
+				break;
+				
+			case kOOTextureExtractChannelB:
+				value = @"b";
+				break;
+				
+			case kOOTextureExtractChannelA:
+				value = @"a";
+				break;
+		}
+		if (value != nil)  [result setObject:value forKey:kOOTextureSpecifierSwizzleKey];
+		
+		if (options & kOOTextureNoShrink)  [result oo_setBool:YES forKey:kOOTextureSpecifierNoShrinkKey];
+		if (options & kOOTextureRepeatS)  [result oo_setBool:YES forKey:kOOTextureSpecifierRepeatSKey];
+		if (options & kOOTextureRepeatT)  [result oo_setBool:YES forKey:kOOTextureSpecifierRepeatTKey];
+		if (options & kOOTextureAllowCubeMap)  [result oo_setBool:YES forKey:kOOTextureSpecifierCubeMapKey];
+	}
+	
+	return result;
+}
+
+
+OOTextureFlags OOApplyTextureOptionDefaults(OOTextureFlags options)
 {
 	// Set default flags if needed
 	if ((options & kOOTextureMinFilterMask) == kOOTextureMinFilterDefault)
