@@ -3,7 +3,7 @@
 NSObjectOOExtensions.m
 
 
-Copyright (C) 2010-2011 Jens Ayton
+Copyright (C) 2010-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -29,11 +29,7 @@ SOFTWARE.
 
 
 #if OOLITE_MAC_OS_X
-#if OOLITE_LEOPARD
 #import <objc/runtime.h>
-#else
-#import <objc/objc-class.h>
-#endif
 #endif
 
 
@@ -43,11 +39,7 @@ SOFTWARE.
 
 + (size_t) oo_instanceSize
 {
-#if OOLITE_LEOPARD || OOLITE_GNUSTEP_1_20
 	return class_getInstanceSize(self);
-#else
-	return ((Class)self)->instance_size;
-#endif
 }
 
 
