@@ -4,8 +4,12 @@ NSScannerOOExtensions.h
 
 Additions to NSScanner to work around bugs.
 
+FIXME: does this work around bugs that actually exist in any system we're
+targetting? It's a conundrum.
+
+
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,7 +28,8 @@ MA 02110-1301, USA.
 
 */
 
-#import <Foundation/NSScanner.h>
+#import "OOCocoa.h"
+
 
 @interface NSScanner (OOExtensions)
 
