@@ -3,7 +3,7 @@
 OOCacheManager.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@ MA 02110-1301, USA.
 #import "OODeepCopy.h"
 #import "OOCollectionExtractors.h"
 #import "OOJavaScriptEngine.h"
+#import "NSFileManagerOOExtensions.h"
 
 
 #define WRITE_ASYNC				1
@@ -73,7 +74,7 @@ static NSString * const kCacheKeyCaches						= @"caches";
 enum
 {
 	kEndianTagValue			= 0x0123456789ABCDEFULL,
-	kFormatVersionValue		= 200
+	kFormatVersionValue		= 214
 };
 
 
@@ -147,7 +148,7 @@ static OOCacheManager *sSingleton = nil;
 }
 
 
-+ (id)sharedCache
++ (OOCacheManager *) sharedCache
 {
 	// NOTE: assumes single-threaded access.
 	if (sSingleton == nil)
@@ -181,7 +182,7 @@ static OOCacheManager *sSingleton = nil;
 	}
 	else
 	{
-		OODebugLog(kOOLogDataCacheRetrieveFailed, @"Failed to retreive\"%@\" cache object %@ -- no such cache.", inCacheKey, inKey);
+		OODebugLog(kOOLogDataCacheRetrieveFailed, @"Failed to retreive \"%@\" cache object %@ -- no such cache.", inCacheKey, inKey);
 	}
 	
 	return result;
@@ -476,18 +477,21 @@ static OOCacheManager *sSingleton = nil;
 	path = [self cachePathCreatingIfNecessary:NO];
 	if (path == nil) return nil;
 	
-	NS_DURING
+	@try
+	{
 		data = [NSData dataWithContentsOfFile:path];
-		if (data == nil)  NS_VALUERETURN(nil, NSDictionary *);
+		if (data == nil)  return nil;
 		
 		contents = [NSPropertyListSerialization propertyListFromData:data
 													mutabilityOption:NSPropertyListImmutable
 															  format:NULL
 													errorDescription:&errorString];
-	NS_HANDLER
-		errorString = [localException reason];
+	}
+	@catch (NSException *exception)
+	{
+		errorString = [exception reason];
 		contents = nil;
-	NS_ENDHANDLER
+	}
 	
 	if (errorString != nil)
 	{
@@ -574,7 +578,7 @@ static OOCacheManager *sSingleton = nil;
 
 - (NSDictionary *)dictionaryOfCaches
 {
-	return OODeepCopy(_caches);
+	return [OODeepCopy(_caches) autorelease];
 }
 
 
@@ -593,7 +597,7 @@ static OOCacheManager *sSingleton = nil;
 	if (!exists)
 	{
 		if (!inCreate) return NO;
-		if (![fmgr createDirectoryAtPath:inPath attributes:nil])
+		if (![fmgr oo_createDirectoryAtPath:inPath attributes:nil])
 		{
 			OOLog(kOOLogDataCacheBuildPathError, @"Could not create folder %@.", inPath);
 			return NO;
@@ -690,7 +694,7 @@ static OOCacheManager *sSingleton = nil;
 }
 
 
-- (OOUInteger)retainCount
+- (NSUInteger)retainCount
 {
 	return UINT_MAX;
 }
