@@ -35,7 +35,7 @@ SOFTWARE.
 @interface OOCrosshairs: NSObject
 {
 @private
-	unsigned					_count;
+	NSUInteger					_count;
 	GLfloat						*_data;
 }
 
