@@ -1,4 +1,5 @@
 #import "OOFunctionAttributes.h"
+#include <tgmath.h>
 
 
 // Utilities for working with NSPoints as 2D vectors.
@@ -12,17 +13,17 @@ OOINLINE NSPoint PtSub(NSPoint a, NSPoint b)
 	return NSMakePoint(a.x - b.x, a.y - b.y);
 }
 
-OOINLINE NSPoint PtScale(NSPoint p, OOCGFloat scale)
+OOINLINE NSPoint PtScale(NSPoint p, CGFloat scale)
 {
 	return NSMakePoint(p.x * scale, p.y * scale);
 }
 
-OOINLINE OOCGFloat PtDot(NSPoint a, NSPoint b)
+OOINLINE CGFloat PtDot(NSPoint a, NSPoint b)
 {
 	return a.x * b.x + a.y * b.y;
 }
 
-OOINLINE OOCGFloat PtCross(NSPoint a, NSPoint b)
+OOINLINE CGFloat PtCross(NSPoint a, NSPoint b)
 {
 	return a.x * b.y - b.x * a.y;
 }
@@ -39,7 +40,7 @@ OOINLINE NSPoint PtRotACW(NSPoint p)
 	return NSMakePoint(-p.y, p.x);
 }
 
-OOINLINE NSPoint PtFastNormal(NSPoint p)
+OOINLINE NSPoint PtNormal(NSPoint p)
 {
-	return PtScale(p, OOFastInvSqrtf(PtDot(p, p)));
+	return PtScale(p, 1.0f / sqrt(PtDot(p, p)));
 }
