@@ -3,7 +3,7 @@
 OOProbabilitySet.m
 
 
-Copyright (C) 2008-2011 Jens Ayton
+Copyright (C) 2008-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -57,7 +57,7 @@ static NSString * const	kWeightsKey = @"weights";
 
 @protocol OOProbabilitySetEnumerable <NSObject>
 
-- (id) privObjectAtIndex:(OOUInteger)index;
+- (id) privObjectAtIndex:(NSUInteger)index;
 
 @end
 
@@ -79,6 +79,7 @@ static NSString * const	kWeightsKey = @"weights";
 
 @interface OOSingleObjectProbabilitySet: OOProbabilitySet
 {
+@private
 	id					_object;
 	float				_weight;
 }
@@ -90,7 +91,8 @@ static NSString * const	kWeightsKey = @"weights";
 
 @interface OOConcreteProbabilitySet: OOProbabilitySet <OOProbabilitySetEnumerable>
 {
-	OOUInteger			_count;
+@private
+	NSUInteger			_count;
 	id					*_objects;
 	float				*_cumulativeWeights;	// Each cumulative weight is weight of object at this index + weight of all objects to left.
 	float				_sumOfWeights;
@@ -100,6 +102,7 @@ static NSString * const	kWeightsKey = @"weights";
 
 @interface OOConcreteMutableProbabilitySet: OOMutableProbabilitySet
 {
+@private
 	NSMutableArray		*_objects;
 	NSMutableArray		*_weights;
 	float				_sumOfWeights;
@@ -112,8 +115,9 @@ static NSString * const	kWeightsKey = @"weights";
 
 @interface OOProbabilitySetEnumerator: NSEnumerator
 {
+@private
 	id					_enumerable;
-	OOUInteger			_index;
+	NSUInteger			_index;
 }
 
 - (id) initWithEnumerable:(id<OOProbabilitySetEnumerable>)enumerable;
@@ -134,7 +138,7 @@ static void ThrowAbstractionViolationException(id obj)  GCC_ATTR((noreturn));
 }
 
 
-+ (id) probabilitySetWithObjects:(id *)objects weights:(float *)weights count:(OOUInteger)count
++ (id) probabilitySetWithObjects:(id *)objects weights:(float *)weights count:(NSUInteger)count
 {
 	return [[[self alloc] initWithObjects:objects weights:weights count:count] autorelease];
 }
@@ -153,7 +157,7 @@ static void ThrowAbstractionViolationException(id obj)  GCC_ATTR((noreturn));
 }
 
 
-- (id) initWithObjects:(id *)objects weights:(float *)weights count:(OOUInteger)count
+- (id) initWithObjects:(id *)objects weights:(float *)weights count:(NSUInteger)count
 {
 	NSZone *zone = [self zone];
 	DESTROY(self);
@@ -179,7 +183,7 @@ static void ThrowAbstractionViolationException(id obj)  GCC_ATTR((noreturn));
 {
 	NSArray					*objects = nil;
 	NSArray					*weights = nil;
-	OOUInteger				i = 0, count = 0;
+	NSUInteger				i = 0, count = 0;
 	id						*rawObjects = NULL;
 	float					*rawWeights = NULL;
 	
@@ -203,7 +207,7 @@ static void ThrowAbstractionViolationException(id obj)  GCC_ATTR((noreturn));
 		// Extract and convert weights.
 		for (i = 0; i < count; ++i)
 		{
-			rawWeights[i] = fmaxf([weights oo_floatAtIndex:i], 0.0f);
+			rawWeights[i] = fmax([weights oo_floatAtIndex:i], 0.0f);
 		}
 		
 		self = [self initWithObjects:rawObjects weights:rawWeights count:count];
@@ -256,7 +260,7 @@ static void ThrowAbstractionViolationException(id obj)  GCC_ATTR((noreturn));
 }
 
 
-- (OOUInteger) count
+- (NSUInteger) count
 {
 	ThrowAbstractionViolationException(self);
 }
@@ -354,7 +358,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (OOUInteger) count
+- (NSUInteger) count
 {
 	return 0;
 }
@@ -407,7 +411,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (OOUInteger) retainCount
+- (NSUInteger) retainCount
 {
 	return UINT_MAX;
 }
@@ -438,7 +442,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 	if ((self = [super initPriv]))
 	{
 		_object = [object retain];
-		_weight = fmaxf(weight, 0.0f);
+		_weight = fmax(weight, 0.0f);
 	}
 	
 	return self;
@@ -481,7 +485,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (OOUInteger) count
+- (NSUInteger) count
 {
 	return 1;
 }
@@ -503,9 +507,9 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 
 @implementation OOConcreteProbabilitySet
 
-- (id) initWithObjects:(id *)objects weights:(float *)weights count:(OOUInteger)count
+- (id) initWithObjects:(id *)objects weights:(float *)weights count:(NSUInteger)count
 {
-	OOUInteger				i = 0;
+	NSUInteger				i = 0;
 	float					cuWeight = 0.0f;
 	
 	assert(count > 1 && objects != NULL && weights != NULL);
@@ -538,7 +542,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 
 - (void) dealloc
 {
-	OOUInteger				i = 0;
+	NSUInteger				i = 0;
 	
 	if (_objects != NULL)
 	{
@@ -565,7 +569,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 	NSArray					*objects = nil;
 	NSMutableArray			*weights = nil;
 	float					cuWeight = 0.0f, sum = 0.0f;
-	OOUInteger				i = 0;
+	NSUInteger				i = 0;
 	
 	objects = [NSArray arrayWithObjects:_objects count:_count];
 	weights = [NSMutableArray arrayWithCapacity:_count];
@@ -582,7 +586,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 			nil];
 }
 
-- (OOUInteger) count
+- (NSUInteger) count
 {
 	return _count;
 }
@@ -596,7 +600,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 		leftmost, i.e. the one where the delta is non-zero.
 	*/
 	
-	OOUInteger					low = 0, high = _count - 1, idx = 0;
+	NSUInteger					low = 0, high = _count - 1, idx = 0;
 	float						weight = 0.0f;
 	
 	while (low < high)
@@ -636,7 +640,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 
 - (float) weightForObject:(id)object
 {
-	OOUInteger					i;
+	NSUInteger					i;
 	
 	// Can't have nil in collection.
 	if (object == nil)  return -1.0f;
@@ -674,7 +678,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (id) privObjectAtIndex:(OOUInteger)index
+- (id) privObjectAtIndex:(NSUInteger)index
 {
 	return (index < _count) ? _objects[index] : nil;
 }
@@ -684,7 +688,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 {
 	id						result = nil;
 	float					*weights = NULL;
-	OOUInteger				i = 0;
+	NSUInteger				i = 0;
 	float					weight = 0.0f, sum = 0.0f;
 	
 	// Convert cumulative weights to "plain" weights.
@@ -723,7 +727,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (id) initWithObjects:(id *)objects weights:(float *)weights count:(OOUInteger)count
+- (id) initWithObjects:(id *)objects weights:(float *)weights count:(NSUInteger)count
 {
 	NSZone *zone = [self zone];
 	[self release];
@@ -789,9 +793,9 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (id) initWithObjects:(id *)objects weights:(float *)weights count:(OOUInteger)count
+- (id) initWithObjects:(id *)objects weights:(float *)weights count:(NSUInteger)count
 {
-	OOUInteger				i = 0;
+	NSUInteger				i = 0;
 	
 	// Validate parameters.
 	if (count != 0 && (objects == NULL || weights == NULL))
@@ -805,7 +809,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 	{
 		for (i = 0; i != count; ++i)
 		{
-			[self setWeight:fmaxf(weights[i], 0.0f) forObject:objects[i]];
+			[self setWeight:fmax(weights[i], 0.0f) forObject:objects[i]];
 		}
 	}
 	
@@ -818,7 +822,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 	BOOL					OK = YES;
 	NSArray					*objects = nil;
 	NSArray					*weights = nil;
-	OOUInteger				i = 0, count = 0;
+	NSUInteger				i = 0, count = 0;
 	
 	if (!(self = [super initPriv]))  OK = NO;
 	
@@ -869,7 +873,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 }
 
 
-- (OOUInteger) count
+- (NSUInteger) count
 {
 	return [_objects count];
 }
@@ -878,7 +882,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 - (id) randomObject
 {
 	float					target = 0.0f, sum = 0.0f, sumOfWeights;
-	OOUInteger				i = 0, count = 0;
+	NSUInteger				i = 0, count = 0;
 	
 	sumOfWeights = [self sumOfWeights];
 	target = randf() * sumOfWeights;
@@ -891,7 +895,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 		if (sum >= target)  return [_objects objectAtIndex:i];
 	}
 	
-	OOLog(@"probabilitySet.broken", @"%s fell off end, returning first object. Nominal sum = %f, target = %f, actual sum = %f, count = %u. %@", __PRETTY_FUNCTION__, sumOfWeights, target, sum, count,@"This is an internal error, please report it.");
+	OOLog(@"probabilitySet.broken", @"%s fell off end, returning first object. Nominal sum = %f, target = %f, actual sum = %f, count = %lu. %@", __PRETTY_FUNCTION__, sumOfWeights, target, sum, count,@"This is an internal error, please report it.");
 	return [_objects objectAtIndex:0];
 }
 
@@ -902,7 +906,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 	
 	if (object != nil)
 	{
-		OOUInteger index = [_objects indexOfObject:object];
+		NSUInteger index = [_objects indexOfObject:object];
 		if (index != NSNotFound)
 		{
 			result = [_weights oo_floatAtIndex:index];
@@ -917,7 +921,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 {
 	if (_sumOfWeights < 0.0f)
 	{
-		OOUInteger			i, count;
+		NSUInteger			i, count;
 		count = [self count];
 		
 		_sumOfWeights = 0.0f;
@@ -946,13 +950,17 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 {
 	if (object == nil)  return;
 	
-	weight = fmaxf(weight, 0.0f);
-	OOUInteger index = [_objects indexOfObject:object];
+	weight = fmax(weight, 0.0f);
+	NSUInteger index = [_objects indexOfObject:object];
 	if (index == NSNotFound)
 	{
 		[_objects addObject:object];
 		[_weights oo_addFloat:weight];
-		_sumOfWeights += weight;
+		if (_sumOfWeights >= 0)
+		{
+			_sumOfWeights += weight;
+		}
+		// Else, _sumOfWeights is invalid and will need to be recalculated on demand.
 	}
 	else
 	{
@@ -966,7 +974,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 {
 	if (object == nil)  return;
 	
-	OOUInteger index = [_objects indexOfObject:object];
+	NSUInteger index = [_objects indexOfObject:object];
 	if (index != NSNotFound)
 	{
 		[_objects removeObjectAtIndex:index];
@@ -981,7 +989,7 @@ static OOEmptyProbabilitySet *sOOEmptyProbabilitySetSingleton = nil;
 	id						result = nil;
 	id						*objects = NULL;
 	float					*weights = NULL;
-	OOUInteger				i = 0, count = 0;
+	NSUInteger				i = 0, count = 0;
 	
 	count = [_objects count];
 	if (EXPECT_NOT(count == 0))  return [OOEmptyProbabilitySet singleton];
