@@ -3,7 +3,7 @@
 OOSkyDrawable.m
 
 
-Copyright (C) 2007-2011 Jens Ayton
+Copyright (C) 2007-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -76,6 +76,7 @@ enum
 */
 @interface OOSkyQuadSet: NSObject
 {
+@private
 	OOTexture				*_texture;
 	unsigned				_count;
 	GLfloat					*_positions;	// 3 entries per vertex, 12 per quad
@@ -192,20 +193,14 @@ static OOColor *SaturatedColorInRange(OOColor *color1, OOColor *color2);
 	// since it'll be behind everything else anyway.
 	
 	OO_ENTER_OPENGL();
+	OOSetOpenGLState(OPENGL_STATE_ADDITIVE_BLENDING);
 	
-	OOGL(glPushAttrib(GL_ENABLE_BIT | GL_DEPTH_BUFFER_BIT));
+	OOGL(glDisable(GL_DEPTH_TEST));  // don't read the depth buffer
+	OOGL(glEnable(GL_TEXTURE_2D));
 	
-	OOGL(glDisable(GL_LIGHTING));
-	OOGL(glDisable(GL_DEPTH_TEST));		// don't read the depth buffer
-	OOGL(glDepthMask(GL_FALSE));		// don't write to depth buffer
-	OOGL(glDisable(GL_CULL_FACE));
-	OOGL(glDisable(GL_FOG));
-	
-	OOGL(glDisableClientState(GL_NORMAL_ARRAY));
-	OOGL(glDisableClientState(GL_EDGE_FLAG_ARRAY));
-	OOGL(glEnableClientState(GL_VERTEX_ARRAY));
-	OOGL(glEnableClientState(GL_TEXTURE_COORD_ARRAY));
-	OOGL(glEnableClientState(GL_COLOR_ARRAY));
+	// Make stars dim in atmosphere. Note: works OK on night side because sky is dark blue, not black.
+	GLfloat fogColor[4] = {0.02, 0.02, 0.02, 1.0};
+	OOGL(glFogfv(GL_FOG_COLOR, fogColor));
 	
 	if (_displayListName != 0)
 	{
@@ -216,21 +211,28 @@ static OOColor *SaturatedColorInRange(OOColor *color1, OOColor *color2);
 		// Set up display list
 		[self ensureTexturesLoaded];
 		_displayListName = glGenLists(1);
+		
 		OOGL(glNewList(_displayListName, GL_COMPILE));
 		
-		OOGL(glEnable(GL_TEXTURE_2D));
-		OOGL(glBlendFunc(GL_ONE, GL_ONE));	// Pure additive blending, ignoring alpha
+		OOGL(glEnableClientState(GL_TEXTURE_COORD_ARRAY));
+		OOGL(glEnableClientState(GL_COLOR_ARRAY));
 		
 		[_quadSets makeObjectsPerformSelector:@selector(render)];
 		
-		OOGL(glDisable(GL_TEXTURE_2D));
-		OOGL(glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));	// Basic alpha blending
+		OOGL(glDisableClientState(GL_TEXTURE_COORD_ARRAY));
+		OOGL(glDisableClientState(GL_COLOR_ARRAY));
 		
 		OOGL(glEndList());
 	}
 	
 	// Restore state
-	OOGL(glPopAttrib());
+	OOGL(glEnable(GL_DEPTH_TEST));
+	OOGL(glDisable(GL_TEXTURE_2D));
+	
+	// Resetting fog is draw loop's responsibility.
+	
+	OOVerifyOpenGLState();
+	OOCheckOpenGLErrors(@"OOSkyDrawable after rendering");
 }
 
 
@@ -448,7 +450,7 @@ static OOColor *DebugColor(Vector orientation)
 	{
 		sStarTextures = [[OOProbabilisticTextureManager alloc]
 							initWithPListName:@"startextures.plist"
-									  options:kOOTextureMinFilterMipMap | kOOTextureMagFilterLinear
+									  options:kOOTextureMinFilterMipMap | kOOTextureMagFilterLinear | kOOTextureAlphaMask
 								   anisotropy:0.0f
 									  lodBias:-0.0f];
 		if (sStarTextures == nil)
@@ -468,7 +470,7 @@ static OOColor *DebugColor(Vector orientation)
 	{
 		sNebulaTextures = [[OOProbabilisticTextureManager alloc]
 							initWithPListName:@"nebulatextures.plist"
-									  options:kOOTextureDefaultOptions
+									  options:kOOTextureDefaultOptions | kOOTextureAlphaMask
 								   anisotropy:0.0f
 									  lodBias:0.0f];
 		if (sNebulaTextures == nil)
@@ -687,7 +689,7 @@ static OOColor *DebugColor(Vector orientation)
 static OOColor *SaturatedColorInRange(OOColor *color1, OOColor *color2)
 {
 	OOColor				*color = nil;
-	OOCGFloat			hue, saturation, brightness, alpha;
+	float				hue, saturation, brightness, alpha;
 	
 	color = [color1 blendedColorWithFraction:randf() ofColor:color2];
 	[color getHue:&hue saturation:&saturation brightness:&brightness alpha:&alpha];
@@ -698,5 +700,5 @@ static OOColor *SaturatedColorInRange(OOColor *color1, OOColor *color2)
 		in [0, 360], but colorWithCalibratedHue:... takes hue values in
 		[0, 1].
 	*/
-	return [OOColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha];
+	return [OOColor colorWithHue:hue saturation:saturation brightness:brightness alpha:alpha];
 }
