@@ -3,7 +3,7 @@
 OOTrumble.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -582,16 +582,15 @@ static void PlayTrumbleSqueal(void);
 	if (hunger > 0.50)
 	{
 		// consult menu...
-		ShipEntity* selectedCargopod = nil;
+		ShipEntity *selectedCargopod = nil;
 		float mostYummy = 0.0;
-		int i;
-		NSMutableArray* cargopods = [player cargo];	// the cargo pods
-		int n_pods = [cargopods count];
+		NSMutableArray *cargopods = [player cargo];	// the cargo pods
+		NSUInteger i, n_pods = [cargopods count];
 		float foodfactor[17] = { 1.00, 0.25, 0.75, 0.01, 0.95, 1.25, 1.05, 0.00, 0.00, 0.00, 0.00, 0.15, 0.00, 0.00, 0.00, 0.00, 0.00};
 		for (i = 0 ; i < n_pods; i++)
 		{
-			ShipEntity* cargopod = [cargopods objectAtIndex:i];
-			int cargo_type = [cargopod commodityType];
+			ShipEntity *cargopod = [cargopods objectAtIndex:i];
+			OOCommodityType cargo_type = [cargopod commodityType];
 			float yumminess = (1.0 + randf()) * foodfactor[cargo_type];
 			if (yumminess > mostYummy)
 			{
