@@ -3,7 +3,7 @@
 OOOXPVerifier.m
 
 
-Copyright (C) 2007-2011 Jens Ayton and contributors
+Copyright (C) 2007-2013 Jens Ayton and contributors
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -506,7 +506,8 @@ static void OpenLogFile(NSString *name);
 		
 		stageName = nil;
 		OOLogPushIndent();
-		NS_DURING
+		@try
+		{
 			stageName = [stageToRun name];
 			if ([stageToRun shouldRun])
 			{
@@ -520,10 +521,12 @@ static void OpenLogFile(NSString *name);
 				OOLog(@"verifyOXP.verbose.skipStage", @"- Skipping stage: %@ (nothing to do).", stageName);
 				[stageToRun noteSkipped];
 			}
-		NS_HANDLER
+		}
+		@catch (NSException *exception)
+		{
 			if (stageName == nil)  stageName = [[stageToRun class] description];
-			OOLog(@"verifyOXP.exception", @"***** Exception occurred when running OXP verifier stage \"%@\": %@: %@", stageName, [localException name], [localException reason]);
-		NS_ENDHANDLER
+			OOLog(@"verifyOXP.exception", @"***** Exception occurred when running OXP verifier stage \"%@\": %@: %@", stageName, [exception name], [exception reason]);
+		}
 		OOLogPopIndent();
 		
 		[_waitingStages removeObject:stageToRun];
@@ -680,7 +683,7 @@ static void OpenLogFile(NSString *name);
 	[graphViz appendString:[graphVizTemplate oo_stringForKey:@"postamble"]];
 	
 	// Write file
-	[ResourceManager writeDiagnosticData:[graphViz dataUsingEncoding:NSUTF8StringEncoding] toFileNamed:@"OXPVerifierStageDependencies.dot"];
+	[ResourceManager writeDiagnosticString:graphViz toFileNamed:@"OXPVerifierStageDependencies.dot"];
 }
 
 @end
@@ -711,10 +714,6 @@ static void NoteVerificationStage(NSString *displayName, NSString *stage)
 static void OpenLogFile(NSString *name)
 {
 	//	Open log file in appropriate application / provide feedback.
-#if !OOLITE_MAC_OS_X
-	// MKW - needed to suppress 'ignoring return value' warning for system() call
-	int ret;
-#endif
 	
 	if ([[NSUserDefaults standardUserDefaults] oo_boolForKey:@"oxp-verifier-open-log" defaultValue:YES])
 	{
@@ -722,11 +721,15 @@ static void OpenLogFile(NSString *name)
 		[[NSWorkspace sharedWorkspace] openFile:OOLogHandlerGetLogPath()];
 #elif OOLITE_WINDOWS
 		// start wordpad (for historical reasons wordpad is called write from the command prompt)
-		ret = system([[NSString stringWithFormat:@"write \"Logs\\%@.log\"", name] UTF8String]);
+		system([[NSString stringWithFormat:@"write \"Logs\\%@.log\"", name] UTF8String]);
 #elif  OOLITE_LINUX
+		// MKW - needed to suppress 'ignoring return value' warning for system() call
+		//		int ret;
+		// CIM - and now the compiler complains about that too... casting return
+		// value to void seems to keep it quiet for now
 		// Nothing to do here, since we dump to stdout instead of to a file.
 		//OOLogOutputHandlerStopLoggingToStdout();
-		ret = system([[NSString stringWithFormat:@"cat \"%@\"", OOLogHandlerGetLogPath()] UTF8String]);
+		(void) system([[NSString stringWithFormat:@"cat \"%@\"", OOLogHandlerGetLogPath()] UTF8String]);
 #else 
 		do {} while (0);
 #endif
