@@ -1,9 +1,9 @@
 /*
 
-Octtree.m
+Octree.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@ MA 02110-1301, USA.
 #import "OOMacroOpenGL.h"
 #import "OODebugFlags.h"
 #import "NSObjectOOExtensions.h"
+#import "OOCollectionExtractors.h"
 
 
 #ifndef NDEBUG
@@ -40,19 +41,35 @@ MA 02110-1301, USA.
 #endif
 
 
+typedef struct
+{
+	GLfloat				radius;
+	const int			*octree;
+	unsigned char		*octree_collision;
+} Octree_details;
+
+
 @interface Octree (Private)
 
 #ifndef OODEBUGLDRAWING_DISABLE
 
-- (void) drawOctreeFromLocation:(int) loc :(GLfloat) scale :(Vector) offset;
-- (void) drawOctreeCollisionFromLocation:(int) loc :(GLfloat) scale :(Vector) offset;
+- (void) drawOctreeFromLocation:(uint32_t)loc :(GLfloat)scale :(Vector)offset;
+- (void) drawOctreeCollisionFromLocation:(uint32_t)loc :(GLfloat)scale :(Vector)offset;
+
+- (BOOL) hasCollision;
+- (void) setHasCollision:(BOOL)value;
 
 #endif
 
+- (Octree_details) octreeDetails;
+
 @end
 
 
-static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GLfloat rad, Vector v0, Vector v1, Vector off, int face_hit);
+static const int change_oct[] = { 0, 1, 2, 4, 3, 5, 6, 7 };	// used to move from nearest to furthest octant
+
+
+static BOOL isHitByLine(const int *octbuffer, unsigned char *collbuffer, int level, GLfloat rad, Vector v0, Vector v1, Vector off, int face_hit);
 static GLfloat volumeOfOctree(Octree_details octree_details, unsigned depthLimit);
 static Vector randomFullNodeFrom(Octree_details details, Vector offset);
 
@@ -63,151 +80,97 @@ static BOOL	isHitByOctree(Octree_details axialDetails, Octree_details otherDetai
 
 - (id) init
 {
-	self = [super init];
-	radius = 0;
-	leafs = 0;
-	octree = malloc(sizeof(int));
-	octree_collision = malloc(sizeof(char));
-	octree[0] = 0;
-	octree_collision[0] = (char)0;
-	hasCollision = NO;
+	// -init makes no sense, since octrees are immutable.
+	[self release];
+	[NSException raise:NSInternalInconsistencyException format:@"Call of invalid initializer %s", __FUNCTION__];
+	return nil;
+}
+
+
+// Designated initializer.
+- (id) initWithData:(NSData *)data
+			 radius:(GLfloat)radius
+{
+	if ((self = [super init]))
+	{
+		_data = [data copy];
+		_radius = radius;
+		
+		NSUInteger nodeCount = [_data length] / sizeof *_octree;
+		NSParameterAssert(nodeCount < UINT32_MAX);
+		_nodeCount = (uint32_t)nodeCount;
+		
+		_octree = [_data bytes];
+		
+		_collisionOctree = calloc(1, _nodeCount);
+		if (_octree == NULL || _collisionOctree == NULL)
+		{
+			[self release];
+			return nil;
+		}
+	}
+	
 	return self;
 }
 
+
+- (id) initWithDictionary:(NSDictionary *)dict
+{
+	NSData *data = [dict objectForKey:@"octree"];
+	if (![data isKindOfClass:[NSData class]] || ([data length] % sizeof (int)) != 0)
+	{
+		// Invalid representation.
+		[self release];
+		return nil;
+	}
+	
+	return [self initWithData:data radius:[dict oo_floatForKey:@"radius"]];
+}
+
+
 - (void) dealloc
 {
-	free(octree);
-	free(octree_collision);
+	DESTROY(_data);
+	free(_collisionOctree);
+	
 	[super dealloc];
 }
 
-- (GLfloat) radius
+
+- (BOOL) hasCollision
 {
-	return radius;
+	return _hasCollision;
 }
 
-- (int) leafs
+
+- (void) setHasCollision:(BOOL)value
 {
-	return leafs;
+	_hasCollision = !!value;
 }
 
-- (int*) octree
-{
-	return octree;
-}
 
-- (BOOL)	hasCollision
+- (Octree_details) octreeDetails
 {
-	return hasCollision;
-}
-
-- (void)	setHasCollision:(BOOL) value
-{
-	hasCollision = value;
-}
-
-- (unsigned char*)	octree_collision
-{
-	return octree_collision;
-}
-
-- (Octree_details)	octreeDetails
-{
-	Octree_details	details;
-	details.octree = octree;
-	details.radius = radius;
-	details.octree_collision = octree_collision;
+	Octree_details details =
+	{
+		.octree = _octree,
+		.radius = _radius,
+		.octree_collision = _collisionOctree
+	};
 	return details;
 }
 
-- (id) initWithRepresentationOfOctree:(GLfloat) octRadius :(NSObject*) octreeArray :(int) leafsize
+
+- (Octree *) octreeScaledBy:(GLfloat)factor
 {
-	self = [super init];
-	
-	radius = octRadius;
-	leafs = leafsize;
-	octree = malloc(leafsize *sizeof(int));
-	octree_collision = malloc(leafsize *sizeof(char));
-	hasCollision = NO;
-	
-	int i;
-	for (i = 0; i< leafsize; i++)
-	{
-		octree[i] = 0;
-		octree_collision[i] = (char)0;
-	}
-	
-	copyRepresentationIntoOctree( octreeArray, octree, 0, 1);
-	
-	return self;
+	// Since octree data is immutable, we can share.
+	return [[[Octree alloc] initWithData:_data radius:_radius * factor] autorelease];
 }
 
-- (id) initWithDictionary:(NSDictionary*) dict
-{
-	self = [super init];
-	
-	radius = [[dict objectForKey:@"radius"] floatValue];
-	leafs = [[dict objectForKey:@"leafs"] intValue];
-	octree = malloc(leafs *sizeof(int));
-	octree_collision = malloc(leafs *sizeof(char));
-	int* data = (int*)[(NSData*)[dict objectForKey:@"octree"] bytes];
-	hasCollision = NO;
-	
-	int i;
-	for (i = 0; i< leafs; i++)
-	{
-		octree[i] = data[i];
-		octree_collision[i] = (char)0;
-	}
-	
-	return self;
-}
 
-- (Octree*) octreeScaledBy:(GLfloat) factor
+static Vector offsetForOctant(int oct, GLfloat r)
 {
-	GLfloat temp = radius;
-	radius *= factor;
-	Octree* result = [[Octree alloc] initWithDictionary:[self dictionaryRepresentation]];
-	radius = temp;
-	return [result autorelease];
-}
-
-int copyRepresentationIntoOctree(NSObject* theRep, int* theBuffer, int atLocation, int nextFreeLocation)
-{
-	if ([theRep isKindOfClass:[NSNumber class]])	// ie. a terminating leaf
-	{
-		if ([(NSNumber*)theRep intValue] != 0)
-		{
-			theBuffer[atLocation] = -1;
-			return nextFreeLocation;
-		}
-		else
-		{
-			theBuffer[atLocation] = 0;
-			return nextFreeLocation;
-		}
-	}
-	if ([theRep isKindOfClass:[NSArray class]])		// ie. a subtree
-	{
-		NSArray* theArray = (NSArray*)theRep;
-		int i;
-		int theNextSpace = nextFreeLocation + 8;
-		for (i = 0; i < 8; i++)
-		{
-			NSObject* rep = [theArray objectAtIndex:i];
-			theNextSpace = copyRepresentationIntoOctree( rep, theBuffer, nextFreeLocation + i, theNextSpace);
-		}
-		theBuffer[atLocation] = nextFreeLocation - atLocation;	// now a relative reference
-		return theNextSpace;
-	}
-	
-	OOLog(@"octree.unarchive.failed", @"**** some error creating octree *****");
-	return nextFreeLocation;
-}
-
-Vector offsetForOctant(int oct, GLfloat r)
-{
-	return make_vector(((GLfloat)0.5 - (GLfloat)((oct >> 2) & 1)) * r, ((GLfloat)0.5 - (GLfloat)((oct >> 1) & 1)) * r, ((GLfloat)0.5 - (GLfloat)(oct & 1)) * r);
+	return make_vector((0.5f - (GLfloat)((oct >> 2) & 1)) * r, (0.5f - (GLfloat)((oct >> 1) & 1)) * r, (0.5f - (GLfloat)(oct & 1)) * r);
 }
 
 
@@ -220,15 +183,15 @@ Vector offsetForOctant(int oct, GLfloat r)
 	OO_ENTER_OPENGL();
 	OOGL(glEnable(GL_BLEND));
 	OOGLBEGIN(GL_LINES);
-	glColor4f(0.4, 0.4, 0.4, 0.5);
+	glColor4f(0.4f, 0.4f, 0.4f, 0.5f);
 	
 	// it's a series of cubes
-	[self drawOctreeFromLocation:0 :radius : kZeroVector];
+	[self drawOctreeFromLocation:0 :_radius : kZeroVector];
 	
 	OOGLEND();
 	
 	OODebugEndWireframe(state);
-	CheckOpenGLErrors(@"Octree after drawing %@", self);
+	OOCheckOpenGLErrors(@"Octree after drawing %@", self);
 }
 
 
@@ -239,14 +202,16 @@ Vector offsetForOctant(int oct, GLfloat r)
 #endif
 
 
-- (void) drawOctreeFromLocation:(int) loc :(GLfloat) scale :(Vector) offset
+- (void) drawOctreeFromLocation:(uint32_t)loc :(GLfloat)scale :(Vector)offset
 {
-	if (octree[loc] == 0)
+	if (_octree[loc] == 0)
+	{
 		return;
+	}
 	
 	OO_ENTER_OPENGL();
 	
-	if (octree[loc] == -1)	// full
+	if (_octree[loc] == -1)	// full
 	{
 		// draw a cube
 		glVertex3f(-scale + offset.x, -scale + offset.y, -scale + offset.z);
@@ -280,29 +245,30 @@ Vector offsetForOctant(int oct, GLfloat r)
 		glVertex3f(scale + offset.x, -scale + offset.y, -scale + offset.z);
 		glVertex3f(scale + offset.x, -scale + offset.y, scale + offset.z);
 	}
-	else if (octree[loc] > 0)
+	else if (_octree[loc] > 0)
 	{
-		GLfloat sc = 0.5 * scale;
-		OCTREE_COLOR( 0.4, 0.4, 0.4, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 0 :sc :make_vector( offset.x - sc, offset.y - sc, offset.z - sc)];
-		OCTREE_COLOR( 0.0, 0.0, 1.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 1 :sc :make_vector( offset.x - sc, offset.y - sc, offset.z + sc)];
-		OCTREE_COLOR( 0.0, 1.0, 0.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 2 :sc :make_vector( offset.x - sc, offset.y + sc, offset.z - sc)];
-		OCTREE_COLOR( 0.0, 1.0, 1.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 3 :sc :make_vector( offset.x - sc, offset.y + sc, offset.z + sc)];
-		OCTREE_COLOR( 1.0, 0.0, 0.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 4 :sc :make_vector( offset.x + sc, offset.y - sc, offset.z - sc)];
-		OCTREE_COLOR( 1.0, 0.0, 1.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 5 :sc :make_vector( offset.x + sc, offset.y - sc, offset.z + sc)];
-		OCTREE_COLOR( 1.0, 1.0, 0.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 6 :sc :make_vector( offset.x + sc, offset.y + sc, offset.z - sc)];
-		OCTREE_COLOR( 1.0, 1.0, 1.0, 0.5);
-		[self drawOctreeFromLocation:	loc + octree[loc] + 7 :sc :make_vector( offset.x + sc, offset.y + sc, offset.z + sc)];
+		GLfloat sc = 0.5f * scale;
+		OCTREE_COLOR(0.4f, 0.4f, 0.4f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 0 :sc :make_vector(offset.x - sc, offset.y - sc, offset.z - sc)];
+		OCTREE_COLOR(0.0f, 0.0f, 1.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 1 :sc :make_vector(offset.x - sc, offset.y - sc, offset.z + sc)];
+		OCTREE_COLOR(0.0f, 1.0f, 0.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 2 :sc :make_vector(offset.x - sc, offset.y + sc, offset.z - sc)];
+		OCTREE_COLOR(0.0f, 1.0f, 1.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 3 :sc :make_vector(offset.x - sc, offset.y + sc, offset.z + sc)];
+		OCTREE_COLOR(1.0f, 0.0f, 0.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 4 :sc :make_vector(offset.x + sc, offset.y - sc, offset.z - sc)];
+		OCTREE_COLOR(1.0f, 0.0f, 1.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 5 :sc :make_vector(offset.x + sc, offset.y - sc, offset.z + sc)];
+		OCTREE_COLOR(1.0f, 1.0f, 0.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 6 :sc :make_vector(offset.x + sc, offset.y + sc, offset.z - sc)];
+		OCTREE_COLOR(1.0f, 1.0f, 1.0f, 0.5f);
+		[self drawOctreeFromLocation:loc + _octree[loc] + 7 :sc :make_vector(offset.x + sc, offset.y + sc, offset.z + sc)];
 	}
 }
 
-BOOL drawTestForCollisions;
+
+static BOOL drawTestForCollisions;
 
 - (void) drawOctreeCollisions
 {
@@ -310,27 +276,33 @@ BOOL drawTestForCollisions;
 	
 	// it's a series of cubes
 	drawTestForCollisions = NO;
-	if (hasCollision)
-		[self drawOctreeCollisionFromLocation:0 :radius : kZeroVector];
-	hasCollision = drawTestForCollisions;
+	if (_hasCollision)
+	{
+		[self drawOctreeCollisionFromLocation:0 :_radius :kZeroVector];
+	}
+	_hasCollision = drawTestForCollisions;
 	
 	OODebugEndWireframe(state);
-	CheckOpenGLErrors(@"Octree after drawing collisions for %@", self);
+	OOCheckOpenGLErrors(@"Octree after drawing collisions for %@", self);
 }
 
-- (void) drawOctreeCollisionFromLocation:(int) loc :(GLfloat) scale :(Vector) offset
+
+- (void) drawOctreeCollisionFromLocation:(uint32_t)loc :(GLfloat)scale :(Vector)offset
 {
-	if (octree[loc] == 0)
+	if (_octree[loc] == 0)
+	{
 		return;
-	if ((octree[loc] != 0)&&(octree_collision[loc] != (unsigned char)0))	// full - draw
+	}
+	
+	if ((_octree[loc] != 0)&&(_collisionOctree[loc] != (unsigned char)0))	// full - draw
 	{
 		OO_ENTER_OPENGL();
 		
-		GLfloat red = (GLfloat)(octree_collision[loc])/255.0;
-		glColor4f( 1.0, 0.0, 0.0, red);	// 50% translucent
+		GLfloat red = (GLfloat)(_collisionOctree[loc])/255.0;
+		glColor4f(1.0, 0.0, 0.0, red);	// 50% translucent
 		
 		drawTestForCollisions = YES;
-		octree_collision[loc]--;
+		_collisionOctree[loc]--;
 		
 		// draw a cube
 		OOGL(glDisable(GL_CULL_FACE));		// face culling
@@ -369,23 +341,23 @@ BOOL drawTestForCollisions;
 			
 		OOGL(glEnable(GL_CULL_FACE));		// face culling
 	}
-	if (octree[loc] > 0)
+	if (_octree[loc] > 0)
 	{
-		GLfloat sc = 0.5 * scale;
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 0 :sc :make_vector( offset.x - sc, offset.y - sc, offset.z - sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 1 :sc :make_vector( offset.x - sc, offset.y - sc, offset.z + sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 2 :sc :make_vector( offset.x - sc, offset.y + sc, offset.z - sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 3 :sc :make_vector( offset.x - sc, offset.y + sc, offset.z + sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 4 :sc :make_vector( offset.x + sc, offset.y - sc, offset.z - sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 5 :sc :make_vector( offset.x + sc, offset.y - sc, offset.z + sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 6 :sc :make_vector( offset.x + sc, offset.y + sc, offset.z - sc)];
-		[self drawOctreeCollisionFromLocation:	loc + octree[loc] + 7 :sc :make_vector( offset.x + sc, offset.y + sc, offset.z + sc)];
+		GLfloat sc = 0.5f * scale;
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 0 :sc :make_vector(offset.x - sc, offset.y - sc, offset.z - sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 1 :sc :make_vector(offset.x - sc, offset.y - sc, offset.z + sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 2 :sc :make_vector(offset.x - sc, offset.y + sc, offset.z - sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 3 :sc :make_vector(offset.x - sc, offset.y + sc, offset.z + sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 4 :sc :make_vector(offset.x + sc, offset.y - sc, offset.z - sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 5 :sc :make_vector(offset.x + sc, offset.y - sc, offset.z + sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 6 :sc :make_vector(offset.x + sc, offset.y + sc, offset.z - sc)];
+		[self drawOctreeCollisionFromLocation:loc + _octree[loc] + 7 :sc :make_vector(offset.x + sc, offset.y + sc, offset.z + sc)];
 	}
 }
 #endif // OODEBUGLDRAWING_DISABLE
 
 
-OOINLINE BOOL isHitByLineSub(int* octbuffer, unsigned char* collbuffer, int nextLevel, GLfloat rad, GLfloat rd2, Vector v0, Vector v1, int octantMask)
+static BOOL isHitByLineSub(const int *octbuffer, unsigned char *collbuffer, int nextLevel, GLfloat rad, GLfloat rd2, Vector v0, Vector v1, int octantMask)
 {
 	if (octbuffer[nextLevel + octantMask])
 	{
@@ -396,15 +368,15 @@ OOINLINE BOOL isHitByLineSub(int* octbuffer, unsigned char* collbuffer, int next
 }
 
 
-BOOL hasCollided = NO;
-GLfloat hit_dist = 0.0;
-static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GLfloat rad, Vector v0, Vector v1, Vector off, int face_hit)
+static BOOL hasCollided = NO;
+static GLfloat hit_dist = 0.0;
+static BOOL isHitByLine(const int *octbuffer, unsigned char *collbuffer, int level, GLfloat rad, Vector v0, Vector v1, Vector off, int face_hit)
 {
 	// displace the line by the offset
-	Vector u0 = make_vector( v0.x + off.x, v0.y + off.y, v0.z + off.z);
-	Vector u1 = make_vector( v1.x + off.x, v1.y + off.y, v1.z + off.z);
+	Vector u0 = make_vector(v0.x + off.x, v0.y + off.y, v0.z + off.z);
+	Vector u1 = make_vector(v1.x + off.x, v1.y + off.y, v1.z + off.z);
 	
-	OctreeDebugLog(@"DEBUG octant: [%d] radius: %.2f vs. line: ( %.2f, %.2f, %.2f) - ( %.2f, %.2f, %.2f)", 
+	OctreeDebugLog(@"DEBUG octant: [%d] radius: %.2f vs. line: (%.2f, %.2f, %.2f) - (%.2f, %.2f, %.2f)",
 		level, rad, u0.x, u0.y, u0.z, u1.x, u1.y, u1.z);
 
 	if (octbuffer[level] == 0)
@@ -417,13 +389,13 @@ static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GL
 	{
 		OctreeDebugLog(@"DEBUG Hit a solid octant: [%d]", level);
 		collbuffer[level] = 2;	// green
-		hit_dist = sqrt( u0.x * u0.x + u0.y * u0.y + u0.z * u0.z);
+		hit_dist = sqrt(u0.x * u0.x + u0.y * u0.y + u0.z * u0.z);
 		return YES;
 	}
 	
 	int faces = face_hit;
 	if (faces == 0)
-		faces = lineCubeIntersection( u0, u1, rad);
+		faces = lineCubeIntersection(u0, u1, rad);
 
 	if (faces == 0)
 	{
@@ -435,7 +407,7 @@ static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GL
 	
 	if (faces > 0)
 	{
-		Vector vi = lineIntersectionWithFace( u0, u1, faces, rad);
+		Vector vi = lineIntersectionWithFace(u0, u1, faces, rad);
 		
 		if (CUBE_FACE_FRONT & faces)
 			octantIntersected = ((vi.x < 0.0)? 1: 5) + ((vi.y < 0.0)? 0: 2);
@@ -452,7 +424,7 @@ static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GL
 		if (CUBE_FACE_BOTTOM & faces)
 			octantIntersected = ((vi.x < 0.0)? 0: 4) + ((vi.z < 0.0)? 0: 1);
 
-		OctreeDebugLog(@"----> found intersection with face 0x%2x of cube of radius %.2f at ( %.2f, %.2f, %.2f) octant:%d",
+		OctreeDebugLog(@"----> found intersection with face 0x%2x of cube of radius %.2f at (%.2f, %.2f, %.2f) octant:%d",
 				faces, rad, vi.x, vi.y, vi.z, octantIntersected);
 	}
 	else
@@ -468,7 +440,7 @@ static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GL
 	
 	int nextLevel = level + octbuffer[level];
 		
-	GLfloat rd2 = 0.5 * rad;
+	GLfloat rd2 = 0.5f * rad;
 	
 	// first test the intersected octant, then the three adjacent, then the next three adjacent, the finally the diagonal octant
 	int oct0, oct1, oct2, oct3;	// test oct0 then oct1, oct2, oct3 then (7 - oct1), (7 - oct2), (7 - oct3) then (7 - oct0)
@@ -503,33 +475,31 @@ static BOOL isHitByLine(int* octbuffer, unsigned char* collbuffer, int level, GL
 	return NO;
 }
 
-- (GLfloat) isHitByLine: (Vector) v0: (Vector) v1
+- (GLfloat) isHitByLine:(Vector)v0 :(Vector)v1
 {
-	int i;
-	for (i = 0; i< leafs; i++) octree_collision[i] = (char)0;
+	memset(_collisionOctree, 0, _nodeCount * sizeof *_collisionOctree);
 	hasCollided = NO;
 	
-	if (isHitByLine(octree, octree_collision, 0, radius, v0, v1, kZeroVector, 0))
+	if (isHitByLine(_octree, _collisionOctree, 0, _radius, v0, v1, kZeroVector, 0))
 	{
 		OctreeDebugLog(@"DEBUG Hit at distance %.2f", hit_dist);
-		hasCollision = hasCollided;
+		_hasCollision = hasCollided;
 		return hit_dist;
 	}
 	else
 	{
-		OctreeDebugLog(@"DEBUG Missed!", hit_dist);
-		hasCollision = hasCollided;
-		return 0.0;
+		OctreeDebugLog(@"DEBUG Missed!");
+		_hasCollision = hasCollided;
+		return 0.0f;
 	}
 }
 
-int change_oct[] = {	0,	1,	2,	4,	3,	5,	6,	7};	// used to move from nearest to furthest octant
 
-BOOL	isHitByOctree(	Octree_details axialDetails,
-						Octree_details otherDetails, Vector otherPosition, Triangle other_ijk)
+static BOOL isHitByOctree(Octree_details axialDetails,
+						  Octree_details otherDetails, Vector otherPosition, Triangle other_ijk)
 {
-	int*	axialBuffer = axialDetails.octree;
-	int*	otherBuffer = otherDetails.octree;
+	const int *axialBuffer = axialDetails.octree;
+	const int *otherBuffer = otherDetails.octree;
 
 	if (axialBuffer[0] == 0)
 	{
@@ -565,8 +535,8 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 	}
 	else	// test axial sphere against other cube
 	{
-		Vector	d2 = make_vector( - otherPosition.x, - otherPosition.y, -otherPosition.z);
-		Vector	axialPosition = resolveVectorInIJK( d2, other_ijk);
+		Vector	d2 = vector_flip(otherPosition);
+		Vector	axialPosition = resolveVectorInIJK(d2, other_ijk);
 		if ((axialPosition.x + axialRadius < -otherRadius)||(axialPosition.x - axialRadius > otherRadius)||
 			(axialPosition.y + axialRadius < -otherRadius)||(axialPosition.y - axialRadius > otherRadius)||
 			(axialPosition.z + axialRadius < -otherRadius)||(axialPosition.z - axialRadius > otherRadius))
@@ -577,9 +547,8 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 	}
 	
 	// from here on, this Octree and the other Octree are considered to be intersecting
-	Octree_details	nextDetails;	// for subdivision (may not be required)
-	unsigned char*	axialCollisionBuffer = axialDetails.octree_collision;
-	unsigned char*	otherCollisionBuffer = otherDetails.octree_collision;
+	unsigned char	*axialCollisionBuffer = axialDetails.octree_collision;
+	unsigned char	*otherCollisionBuffer = otherDetails.octree_collision;
 	if (axialBuffer[0] == -1)
 	{
 		// we are SOLID - is the other octree?
@@ -602,12 +571,14 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 		// work out the nearest octant to the axial octree
 		int	nearest_oct = ((otherPosition.x > 0.0)? 0:4)|((otherPosition.y > 0.0)? 0:2)|((otherPosition.z > 0.0)? 0:1);
 		
-		int				nextLevel			= otherBuffer[0];
-		int*			nextBuffer			= &otherBuffer[nextLevel];
-		unsigned char*	nextCollisionBuffer	= &otherCollisionBuffer[nextLevel];
-		Vector	voff, nextPosition;
-		nextDetails.radius = 0.5 * otherRadius;
-		int	i, oct;
+		int				nextLevel = otherBuffer[0];
+		const int		*nextBuffer = &otherBuffer[nextLevel];
+		unsigned char	*nextCollisionBuffer = &otherCollisionBuffer[nextLevel];
+		Octree_details	nextDetails;
+		Vector			voff, nextPosition;
+		unsigned		i, oct;
+		
+		nextDetails.radius = 0.5f * otherRadius;
 		for (i = 0; i < 8; i++)
 		{
 			oct = nearest_oct ^ change_oct[i];	// work from nearest to furthest
@@ -616,11 +587,9 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 				nextDetails.octree = &nextBuffer[oct];
 				nextDetails.octree_collision = &nextCollisionBuffer[oct];
 				
-				voff = resolveVectorInIJK( offsetForOctant( oct, otherRadius), other_ijk);
-				nextPosition.x = otherPosition.x - voff.x;
-				nextPosition.y = otherPosition.y - voff.y;
-				nextPosition.z = otherPosition.z - voff.z;
-				if (isHitByOctree(	axialDetails, nextDetails, nextPosition, other_ijk))	// test octant
+				voff = resolveVectorInIJK(offsetForOctant(oct, otherRadius), other_ijk);
+				nextPosition = vector_subtract(otherPosition, voff);
+				if (isHitByOctree(axialDetails, nextDetails, nextPosition, other_ijk))	// test octant
 					return YES;
 			}
 		}
@@ -638,12 +607,14 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 	// work out the nearest octant to the other octree
 	int	nearest_oct = ((otherPosition.x > 0.0)? 4:0)|((otherPosition.y > 0.0)? 2:0)|((otherPosition.z > 0.0)? 1:0);
 	
-	int		nextLevel = axialBuffer[0];
-	int*	nextBuffer = &axialBuffer[nextLevel];
-	unsigned char* nextCollisionBuffer = &axialCollisionBuffer[nextLevel];
-	nextDetails.radius = 0.5 * axialRadius;
-	Vector	voff, nextPosition;
-	int		i, oct;
+	int				nextLevel = axialBuffer[0];
+	const int		*nextBuffer = &axialBuffer[nextLevel];
+	unsigned char	*nextCollisionBuffer = &axialCollisionBuffer[nextLevel];
+	Vector			nextPosition;
+	Octree_details	nextDetails;
+	unsigned		i, oct;
+	
+	nextDetails.radius = 0.5f * axialRadius;
 	for (i = 0; i < 8; i++)
 	{
 		oct = nearest_oct ^ change_oct[i];	// work from nearest to furthest
@@ -652,31 +623,32 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 			nextDetails.octree = &nextBuffer[oct];
 			nextDetails.octree_collision = &nextCollisionBuffer[oct];
 			
-			voff = offsetForOctant(oct, axialRadius);
-			nextPosition.x = otherPosition.x + voff.x;
-			nextPosition.y = otherPosition.y + voff.y;
-			nextPosition.z = otherPosition.z + voff.z;
-			if (isHitByOctree(	nextDetails, otherDetails, nextPosition, other_ijk))
+			nextPosition = vector_add(otherPosition, offsetForOctant(oct, axialRadius));
+			if (isHitByOctree(nextDetails, otherDetails, nextPosition, other_ijk))
+			{
 				return YES;	// test octant
+			}
 		}
 	}
 	// otherwise we're done!
 	return NO;
 }
 
-- (BOOL) isHitByOctree:(Octree*) other withOrigin: (Vector) v0 andIJK: (Triangle) ijk
+
+- (BOOL) isHitByOctree:(Octree *)other withOrigin:(Vector)v0 andIJK:(Triangle)ijk
 {
 	if (other == nil)  return NO;
 	
-	BOOL hit = isHitByOctree( [self octreeDetails], [other octreeDetails], v0, ijk);
+	BOOL hit = isHitByOctree([self octreeDetails], [other octreeDetails], v0, ijk);
 	
-	hasCollision = hasCollision | hit;
-	[other setHasCollision: [other hasCollision] | hit];
+	_hasCollision = _hasCollision || hit;
+	[other setHasCollision: [other hasCollision] || hit];
 	
 	return hit; 
 }
 
-- (BOOL) isHitByOctree:(Octree*) other withOrigin: (Vector) v0 andIJK: (Triangle) ijk andScales: (GLfloat) s1: (GLfloat) s2
+
+- (BOOL) isHitByOctree:(Octree *)other withOrigin:(Vector)v0 andIJK:(Triangle)ijk andScales:(GLfloat) s1 :(GLfloat)s2
 {
 	Octree_details details1 = [self octreeDetails];
 	Octree_details details2 = [other octreeDetails];
@@ -684,42 +656,27 @@ BOOL	isHitByOctree(	Octree_details axialDetails,
 	details1.radius *= s1;
 	details2.radius *= s2;
 	
-	BOOL hit = isHitByOctree( details1, details2, v0, ijk);
+	BOOL hit = isHitByOctree(details1, details2, v0, ijk);
 	
-	hasCollision = hasCollision | hit;
-	[other setHasCollision: [other hasCollision] | hit];
+	_hasCollision = _hasCollision || hit;
+	[other setHasCollision: [other hasCollision] || hit];
 	
 	return hit; 
 }
 
 
-
 - (NSDictionary *) dictionaryRepresentation
 {
 	return [NSDictionary dictionaryWithObjectsAndKeys:
-		[NSNumber numberWithFloat:radius],	@"radius",
-		[NSNumber numberWithInt:leafs],		@"leafs",
-		[NSData dataWithBytes:(const void *)octree length: leafs * sizeof(int)], @"octree",
+		_data, @"octree",
+		[NSNumber numberWithFloat:_radius],	@"radius",
 		nil];
 }
 
-- (GLfloat) volume
-{
-	/*	For backwards compatibility, limit octree iteration for volume
-		calculation to five levels. Raising the limit means lower calculated
-		volumes for large ships.
-		
-		EMMSTRAN: consider raising the limit but fudging mass lock calculations
-		to compensate. See http://aegidian.org/bb/viewtopic.php?f=3&t=9176 .
-		Then again, five levels of iteration might be fine.
-		-- Ahruman 2011-03-10
-	*/
-	return volumeOfOctree([self octreeDetails], 5);
-}
 
 static GLfloat volumeOfOctree(Octree_details octree_details, unsigned depthLimit)
 {
-	int		*octBuffer = octree_details.octree;
+	const int *octBuffer = octree_details.octree;
 	GLfloat octRadius = octree_details.radius;
 	
 	if (octBuffer[0] == 0)
@@ -733,15 +690,15 @@ static GLfloat volumeOfOctree(Octree_details octree_details, unsigned depthLimit
 	}
 	depthLimit--;
 	
-	// we are not empty or solid
-	// we sum the volume of each of our octants
+	// We are not empty or solid.
+	// We sum the volume of each of our octants.
 	GLfloat			sumVolume = 0.0f;
-	Octree_details	nextDetails;	// for subdivision (may not be required)
+	Octree_details	nextDetails;
 	int				nextLevel = octBuffer[0];
-	int				*nextBuffer = &octBuffer[nextLevel];
-	int				i;
+	const int		*nextBuffer = &octBuffer[nextLevel];
+	unsigned		i;
 	
-	nextDetails.radius = 0.5 * octRadius;
+	nextDetails.radius = 0.5f * octRadius;
 	nextDetails.octree_collision = NULL;	// Placate static analyzer
 	
 	for (i = 0; i < 8; i++)
@@ -755,22 +712,38 @@ static GLfloat volumeOfOctree(Octree_details octree_details, unsigned depthLimit
 	return sumVolume;
 }
 
+
+- (GLfloat) volume
+{
+	/*	For backwards compatibility, limit octree iteration for volume
+	 calculation to five levels. Raising the limit means lower calculated
+	 volumes for large ships.
+	 
+	 EMMSTRAN: consider raising the limit but fudging mass lock calculations
+	 to compensate. See http://aegidian.org/bb/viewtopic.php?f=3&t=9176 .
+	 Then again, five levels of iteration might be fine.
+	 -- Ahruman 2011-03-10
+	 */
+	return volumeOfOctree([self octreeDetails], 5);
+}
+
+
 static Vector randomFullNodeFrom(Octree_details details, Vector offset)
 {
-	int		*octBuffer = details.octree;
+	const int *octBuffer = details.octree;
 	GLfloat octRadius = details.radius;
 	
 	if (octBuffer[0] == 0)  return offset;
 	if (octBuffer[0] == -1)  return offset;
 	
-	// we are not empty or solid
-	// we sum the volume of each of our octants
-	Octree_details	nextDetails;	// for subdivision (may not be required)
+	// We are not empty or solid.
+	// Pick a location from a random octant.
+	Octree_details	nextDetails;
 	int				nextLevel = octBuffer[0];
-	int				*nextBuffer = &octBuffer[nextLevel];
-	int				i, oct;
+	const int		*nextBuffer = &octBuffer[nextLevel];
+	unsigned		i, oct;
 	
-	nextDetails.radius = 0.5 * octRadius;
+	nextDetails.radius = 0.5f * octRadius;
 	nextDetails.octree_collision = NULL;
 	oct = Ranrot() & 7;
 	
@@ -780,10 +753,7 @@ static Vector randomFullNodeFrom(Octree_details details, Vector offset)
 		if (nextBuffer[octant])	// don't test empty octants
 		{
 			nextDetails.octree = &nextBuffer[octant];
-			Vector voff = offsetForOctant(octant, octRadius);
-			voff.x += offset.x;
-			voff.y += offset.y;
-			voff.z += offset.z;
+			Vector voff = vector_add(offset, offsetForOctant(octant, octRadius));
 			return randomFullNodeFrom(nextDetails, voff);
 		}
 	}
@@ -800,8 +770,194 @@ static Vector randomFullNodeFrom(Octree_details details, Vector offset)
 #ifndef NDEBUG
 - (size_t) totalSize
 {
-	return [self oo_objectSize] + leafs * (sizeof *octree + sizeof *octree_collision);
+	return [self oo_objectSize] + _nodeCount * [_data oo_objectSize] + [_data length] + _nodeCount * sizeof *_collisionOctree;
 }
 #endif
 
 @end
+
+
+enum
+{
+	/*
+		Initial capacity of OOOctreeBuilder. 16 KibiEntries is enough to handle
+		all but 2 built-in models; I expect similar results for OXPs, since
+		geometry detail has limited effect on the octree. Large models with
+		higher depth are more likely to need growth.
+	*/
+	kMinimumBuilderCapacity = 16 << 10
+};
+
+
+@implementation OOOctreeBuilder
+
+// Extract top of state stack.
+OOINLINE struct OOOctreeBuildState *State(OOOctreeBuilder *self)
+{
+	return &self->_stateStack[self->_level];
+}
+
+
+// SetNode(): set the value of an entry in _octree, making space if needed.
+static void SetNode_slow(OOOctreeBuilder *self, uint32_t index, int value) NO_INLINE_FUNC;
+OOINLINE void SetNode(OOOctreeBuilder *self, uint32_t index, int value)
+{
+	if (index < self->_capacity)
+	{
+		self->_octree[index] = value;
+	}
+	else
+	{
+		SetNode_slow(self, index, value);
+	}
+}
+
+
+// InsertNode(): set the node at the current insertion point, and increment insertion point.
+OOINLINE void InsertNode(OOOctreeBuilder *self, int value)
+{
+	NSCAssert(State(self)->remaining > 0, @"Attempt to add node to a full parent in octree builder.");
+	State(self)->remaining--;
+	
+	SetNode(self, State(self)->insertionPoint++, value);
+}
+
+
+- (id) init
+{
+	if ((self = [super init]))
+	{
+		_capacity = kMinimumBuilderCapacity;
+		_octree = malloc(_capacity * sizeof *_octree);
+		if (_octree == NULL)
+		{
+			[self release];
+			return nil;
+		}
+		
+		/*
+			We're initially inserting into the root slot, which must exist and
+			takes a single node.
+		*/
+		_nodeCount = 1;
+		State(self)->remaining = 1;
+	}
+	return self;
+}
+
+
+- (void) dealloc
+{
+	free(_octree);
+	
+	[super dealloc];
+}
+
+
+- (Octree *) buildOctreeWithRadius:(GLfloat)radius
+{
+	NSAssert(State(self)->remaining == 0 && _level == 0, @"Attempt to produce octree from an octree builder in an incomplete state.");
+	
+	size_t dataSize = _nodeCount * sizeof *_octree;
+	int *resized = realloc(_octree, dataSize);
+	if (resized == NULL)  resized = _octree;
+	
+	/*	Hand over the bytes to the data object, which will be used directly by
+		the Octree.
+	*/
+	NSData *data = [NSData dataWithBytesNoCopy:resized
+										length:dataSize
+								  freeWhenDone:YES];
+	
+	_octree = NULL;
+	_nodeCount = 0;
+	_capacity = 0;
+	
+	return [[[Octree alloc] initWithData:data radius:radius] autorelease];
+}
+
+
+- (void) writeSolid
+{
+	InsertNode(self, -1);
+}
+
+
+- (void) writeEmpty
+{
+	InsertNode(self, 0);
+}
+
+
+- (void) beginInnerNode
+{
+	NSAssert(_level < kMaxOctreeDepth, @"Attempt to build octree exceeding maximum depth.");
+	
+	// Insert relative offset to next free space.
+	uint32_t newInsertionPoint = _nodeCount;
+	InsertNode(self, (int)_nodeCount - State(self)->insertionPoint);
+	
+	/*
+		Leave space for eight nodes.
+		
+		NOTE: this may leave memory uninitialized or leave _nodeCount pointing
+		past the end of the buffer. A valid sequence of eight child insertions
+		will fix both these problems by writing to all of the new slots.
+	*/
+	_nodeCount += 8;
+	
+	// Push state and set up new "stack frame".
+	_level++;
+	State(self)->insertionPoint = newInsertionPoint;
+	State(self)->remaining = 8;
+}
+
+
+- (void) endInnerNode
+{
+	NSAssert(State(self)->remaining == 0, @"Attempt to end an inner octree node with fewer than eight children.");
+	NSAssert1(_level > 0, @"Unbalanced call to %s", __FUNCTION__);
+	
+	_level--;
+	
+	/*
+		Check if the last eight nodes are solid. If so, we just inserted an
+		entirely solid subtree and can fold it into a single solid node. An
+		entirely solid subtree will always use the last eight nodes (any solid
+		subtrees within a child node will have been folded already), so this
+		simple approach to folding will produce an optimal result.
+		
+		We could do the same for empty nodes, but OOMeshToOctreeConverter will
+		never recurse into an empty subtree.
+	*/
+	
+	NSAssert(_nodeCount > 8, @"After ending an inner node, there must be at least eight nodes in buffer.");
+	for (uint_fast32_t node = _nodeCount - 8; node < _nodeCount; node++)
+	{
+		if (_octree[node] != -1)  return;
+	}
+	
+	// If we got here, subtree is solid; fold it into a solid node.
+	_octree[State(self)->insertionPoint - 1] = -1;
+	_nodeCount -= 8;
+}
+
+
+// Slow path for SetNode() when writing beyond _capacity: expand, then perform set.
+static void SetNode_slow(OOOctreeBuilder *self, uint32_t index, int value)
+{
+	uint32_t newCapacity = MAX(self->_capacity * 2, (uint32_t)kMinimumBuilderCapacity);
+	newCapacity = MAX(newCapacity, index + 1);
+	
+	int *newBuffer = realloc(self->_octree, newCapacity * sizeof *newBuffer);
+	if (EXPECT_NOT(newBuffer == NULL))
+	{
+		[NSException raise:NSMallocException format:@"Failed to allocate memory for octree."];
+	}
+	
+	self->_octree = newBuffer;
+	self->_capacity = newCapacity;
+	self->_octree[index] = value;
+}
+
+@end
