@@ -3,7 +3,7 @@
 ResourceManager.m
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,6 +34,8 @@ MA 02110-1301, USA.
 #import "MyOpenGLView.h"
 #import "OOCollectionExtractors.h"
 #import "OOLogOutputHandler.h"
+#import "NSFileManagerOOExtensions.h"
+#import "OldSchoolPropertyListWriting.h"
 
 #import "OOJSScript.h"
 #import "OOPListScript.h"
@@ -83,7 +85,7 @@ static NSMutableDictionary *sStringCache;
 + (NSString *) errors
 {
 	NSArray					*error = nil;
-	unsigned				i, count;
+	NSUInteger				i, count;
 	NSMutableArray			*result = nil;
 	NSString				*errStr = nil;
 	
@@ -234,10 +236,12 @@ static NSMutableDictionary *sStringCache;
 + (NSArray *)paths
 {
 	if (EXPECT_NOT(sSearchPaths == nil))
+	{
 		if (!sUseAddOns)
 		{
 			sSearchPaths = [[NSMutableArray alloc] init];
 		}
+	}
 	return sUseAddOns ? [self pathsWithAddOns] : (NSArray *)[NSArray arrayWithObject:[self builtInPath]];
 }
 
@@ -442,13 +446,12 @@ static NSMutableDictionary *sStringCache;
 	NSString			*path = nil;
 	id					modDate = nil;
 	
-	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"always-flush-cache"])
+	if (EXPECT_NOT([[NSUserDefaults standardUserDefaults] boolForKey:@"always-flush-cache"]))
 	{
 		OOLog(kOOLogCacheExplicitFlush, @"Cache explicitly flushed with always-flush-cache preference. Rebuilding from scratch.");
 		upToDate = NO;
 	}
-	
-	if (upToDate && [MyOpenGLView pollShiftKey])
+	else if ([MyOpenGLView pollShiftKey])
 	{
 		OOLog(kOOLogCacheExplicitFlush, @"Cache explicitly flushed with shift key. Rebuilding from scratch.");
 		upToDate = NO;
@@ -467,7 +470,7 @@ static NSMutableDictionary *sStringCache;
 	modDates = [NSMutableArray arrayWithCapacity:[searchPaths count]];
 	for (pathEnum = [searchPaths objectEnumerator]; (path = [pathEnum nextObject]); )
 	{
-		modDate = [[fmgr fileAttributesAtPath:path traverseLink:YES] objectForKey:NSFileModificationDate];
+		modDate = [[fmgr oo_fileAttributesAtPath:path traverseLink:YES] objectForKey:NSFileModificationDate];
 		if (modDate != nil)
 		{
 			// Converts to double because I'm not sure the cache can deal with dates under GNUstep.
@@ -532,7 +535,7 @@ static NSMutableDictionary *sStringCache;
 			break;
 		
 		default:
-			OOLog(kOOLogParameterError, @"Unknown dictionary merge mode %u for %@. (This is an internal programming error, please report it.)", fileName, mergeMode);
+			OOLog(kOOLogParameterError, @"Unknown dictionary merge mode %u for %@. (This is an internal programming error, please report it.)", mergeMode, fileName);
 			return nil;
 	}
 	
@@ -703,7 +706,7 @@ static NSMutableDictionary *sStringCache;
 // the latest entry replaces the earliest.
 + (void) handleEquipmentListMerging: (NSMutableArray *)arrayToProcess forLookupIndex:(unsigned)lookupIndex
 {
-	OOUInteger i,j,k;
+	NSUInteger i,j,k;
 	NSMutableArray *refArray = [arrayToProcess objectAtIndex:[arrayToProcess count] - 1];
 	
 	// Any change to arrayRef will directly modify arrayToProcess.
@@ -712,10 +715,10 @@ static NSMutableDictionary *sStringCache;
 	{
 		for (j = 0; j < [arrayToProcess count] - 1; j++)
 		{
-			OOUInteger count = [[arrayToProcess oo_arrayAtIndex:j] count];
+			NSUInteger count = [[arrayToProcess oo_arrayAtIndex:j] count];
 			if (count == 0)  continue;
 			
-			for (k=0; k < count - 1; k++)
+			for (k=0; k < count; k++)
 			{
 				id processValue = [[[arrayToProcess oo_arrayAtIndex:j] oo_arrayAtIndex:k] oo_objectAtIndex:lookupIndex defaultValue:nil];
 				id refValue = [[refArray oo_arrayAtIndex:i] oo_objectAtIndex:lookupIndex defaultValue:nil];
@@ -734,12 +737,11 @@ static NSMutableDictionary *sStringCache;
 
 + (NSDictionary *) whitelistDictionary
 {
-	static id				whitelistDictionary = nil;
-	NSString				*path = nil;
+	static id whitelistDictionary = nil;
 	
 	if (whitelistDictionary == nil)
 	{
-		path = [[[ResourceManager builtInPath] stringByAppendingPathComponent:@"Config"] stringByAppendingPathComponent:@"whitelist.plist"];
+		NSString *path = [[[ResourceManager builtInPath] stringByAppendingPathComponent:@"Config"] stringByAppendingPathComponent:@"whitelist.plist"];
 		whitelistDictionary = [NSDictionary dictionaryWithContentsOfFile:path];
 		if (whitelistDictionary == nil)  whitelistDictionary = [NSNull null];
 		
@@ -751,6 +753,135 @@ static NSMutableDictionary *sStringCache;
 }
 
 
+static NSString *LogClassKeyRoot(NSString *key)
+{
+	NSRange dot = [key rangeOfString:@"."];
+	if (dot.location != NSNotFound)
+	{
+		return [key substringToIndex:dot.location];
+	}
+	else
+	{
+		return key;
+	}
+}
+
+
++ (NSDictionary *) logControlDictionary
+{
+	// Load built-in copy of logcontrol.plist.
+	NSString *path = [[[ResourceManager builtInPath] stringByAppendingPathComponent:@"Config"]
+					  stringByAppendingPathComponent:@"logcontrol.plist"];
+	NSMutableDictionary *logControl = [NSMutableDictionary dictionaryWithDictionary:OODictionaryFromFile(path)];
+	if (logControl == nil)  logControl = [NSMutableDictionary dictionary];
+	
+	// Build list of root log message classes that appear in the built-in list.
+	NSMutableSet *coreRoots = [NSMutableSet set];
+	NSString *key = nil;
+	foreachkey(key, logControl)
+	{
+		[coreRoots addObject:LogClassKeyRoot(key)];
+	}
+	
+	NSArray *rootPaths = [self rootPaths];
+	NSString *configPath = nil;
+	NSDictionary *dict = nil;
+	
+	// Look for logcontrol.plists inside OXPs (but not in root paths). These are not allowed to define keys in hierarchies used by the build-in one.
+	NSEnumerator *pathEnum = [self pathEnumerator];
+	while ((path = [pathEnum nextObject]))
+	{
+		if ([rootPaths containsObject:path])  continue;
+		
+		configPath = [[path stringByAppendingPathComponent:@"Config"]
+					  stringByAppendingPathComponent:@"logcontrol.plist"];
+		dict = OODictionaryFromFile(configPath);
+		if (dict == nil)
+		{
+			configPath = [path stringByAppendingPathComponent:@"logcontrol.plist"];
+			dict = OODictionaryFromFile(configPath);
+		}
+		foreachkey (key, dict)
+		{
+			if (![coreRoots containsObject:LogClassKeyRoot(key)])
+			{
+				[logControl setObject:[dict objectForKey:key] forKey:key];
+			}
+		}
+	}
+	
+	// Now, look for logcontrol.plists in root paths, i.e. not within OXPs. These are allowed to override the built-in copy.
+	pathEnum = [rootPaths objectEnumerator];
+	while ((path = [pathEnum nextObject]))
+	{
+		configPath = [[path stringByAppendingPathComponent:@"Config"]
+					  stringByAppendingPathComponent:@"logcontrol.plist"];
+		dict = OODictionaryFromFile(configPath);
+		if (dict == nil)
+		{
+			configPath = [path stringByAppendingPathComponent:@"logcontrol.plist"];
+			dict = OODictionaryFromFile(configPath);
+		}
+		foreachkey (key, dict)
+		{
+			[logControl setObject:[dict objectForKey:key] forKey:key];
+		}
+	}
+	
+	// Finally, look in preferences, which can override all of the above.
+	dict = [[NSUserDefaults standardUserDefaults] dictionaryForKey:@"logging-enable"];
+	if (dict != nil)  [logControl addEntriesFromDictionary:dict];
+	
+	return logControl;
+}
+
+
++ (NSDictionary *) shaderBindingTypesDictionary
+{
+	static id shaderBindingTypesDictionary = nil;
+	
+	if (shaderBindingTypesDictionary == nil)
+	{
+		NSAutoreleasePool *pool = [NSAutoreleasePool new];
+		
+		NSString *path = [[[ResourceManager builtInPath] stringByAppendingPathComponent:@"Config"] stringByAppendingPathComponent:@"shader-uniform-bindings.plist"];
+		NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithContentsOfFile:path];
+		NSArray *keys = [dict allKeys];
+		
+		// Resolve all $inherit keys.
+		unsigned changeCount = 0;
+		do {
+			changeCount = 0;
+			NSString *key = nil;
+			foreach (key, keys)
+			{
+				NSDictionary *value = [dict oo_dictionaryForKey:key];
+				NSString *inheritKey = [value oo_stringForKey:@"$inherit"];
+				if (inheritKey != nil)
+				{
+					changeCount++;
+					NSMutableDictionary *mutableValue = [[value mutableCopy] autorelease];
+					[mutableValue removeObjectForKey:@"$inherit"];
+					NSDictionary *inherited = [dict oo_dictionaryForKey:inheritKey];
+					if (inherited != nil)
+					{
+						[mutableValue addEntriesFromDictionary:inherited];
+					}
+					
+					[dict setObject:[[mutableValue copy] autorelease] forKey:key];
+				}
+			}
+		} while (changeCount != 0);
+		
+		shaderBindingTypesDictionary = [dict copy];
+		
+		[pool release];
+	}
+	
+	return shaderBindingTypesDictionary;
+}
+
+
 + (NSString *) pathForFileNamed:(NSString *)fileName inFolder:(NSString *)folderName
 {
 	return [self pathForFileNamed:fileName inFolder:folderName cache:YES];
@@ -912,13 +1043,16 @@ static NSMutableDictionary *sStringCache;
 	NSString					*name = nil;
 	NSAutoreleasePool			*pool = nil;
 	
+	OOLog(@"script.load.world.begin", @"Loading world scripts...");
+	
 	loadedScripts = [NSMutableDictionary dictionary];
 	paths = [ResourceManager paths];
 	for (pathEnum = [paths objectEnumerator]; (path = [pathEnum nextObject]); )
 	{
 		pool = [[NSAutoreleasePool alloc] init];
 		
-		NS_DURING
+		@try
+		{
 			results = [OOScript worldScriptsAtPath:[path stringByAppendingPathComponent:@"Config"]];
 			if (results == nil) results = [OOScript worldScriptsAtPath:path];
 			if (results != nil)
@@ -930,17 +1064,19 @@ static NSMutableDictionary *sStringCache;
 					else  OOLog(@"script.load.unnamed", @"Discarding anonymous script %@", script);
 				}
 			}
-		NS_HANDLER
-			OOLog(@"script.load.exception", @"***** %s encountered exception %@ (%@) while trying to load script from %@ -- ignoring this location.", __PRETTY_FUNCTION__, [localException name], [localException reason], path);
+		}
+		@catch (NSException *exception)
+		{
+			OOLog(@"script.load.exception", @"***** %s encountered exception %@ (%@) while trying to load script from %@ -- ignoring this location.", __PRETTY_FUNCTION__, [exception name], [exception reason], path);
 			// Ignore exception and keep loading other scripts.
-		NS_ENDHANDLER
+		}
 		
 		[pool release];
 	}
 	
 	if (OOLogWillDisplayMessagesInClass(@"script.load.world.listAll"))
 	{
-		unsigned count = [loadedScripts count];
+		NSUInteger count = [loadedScripts count];
 		if (count != 0)
 		{
 			NSMutableArray		*displayNames = nil;
@@ -956,7 +1092,7 @@ static NSMutableDictionary *sStringCache;
 			}
 			
 			displayString = [[displayNames sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n    "];
-			OOLog(@"script.load.world.listAll", @"Loaded %u world scripts:\n    %@", count, displayString);
+			OOLog(@"script.load.world.listAll", @"Loaded %lu world scripts:\n    %@", count, displayString);
 		}
 		else
 		{
@@ -970,14 +1106,46 @@ static NSMutableDictionary *sStringCache;
 
 + (BOOL) writeDiagnosticData:(NSData *)data toFileNamed:(NSString *)name
 {
-	NSString			*basePath = nil;
-	
 	if (data == nil || name == nil)  return NO;
 	
-	basePath = [self diagnosticFileLocation];
-	if (basePath == nil)  return NO;
+	NSString *directory = [self diagnosticFileLocation];
+	if (directory == nil)  return NO;
 	
-	return [data writeToFile:[basePath stringByAppendingPathComponent:name] atomically:YES];
+	NSArray *nameComponents = [name componentsSeparatedByString:@"/"];
+	NSUInteger count = [nameComponents count];
+	if (count > 1)
+	{
+		name = [nameComponents lastObject];
+		
+		for (NSUInteger i = 0; i < count - 1; i++)
+		{
+			NSString *component = [nameComponents objectAtIndex:i];
+			if ([component hasPrefix:@"."])
+			{
+				component = [@"!" stringByAppendingString:[component substringFromIndex:1]];
+			}
+			directory = [directory stringByAppendingPathComponent:component];
+			[[NSFileManager defaultManager] oo_createDirectoryAtPath:directory attributes:nil];
+		}
+	}
+	
+	return [data writeToFile:[directory stringByAppendingPathComponent:name] atomically:YES];
+}
+
+
++ (BOOL) writeDiagnosticString:(NSString *)string toFileNamed:(NSString *)name
+{
+	return [self writeDiagnosticData:[string dataUsingEncoding:NSUTF8StringEncoding] toFileNamed:name];
+}
+
+
++ (BOOL) writeDiagnosticPList:(id)plist toFileNamed:(NSString *)name
+{
+	NSData *data = [plist oldSchoolPListFormatWithErrorDescription:NULL];
+	if (data == nil)  [NSPropertyListSerialization dataFromPropertyList:plist format:NSPropertyListXMLFormat_v1_0 errorDescription:NULL];
+	if (data == nil)  return NO;
+	
+	return [self writeDiagnosticData:data toFileNamed:name];
 }
 
 
@@ -1002,7 +1170,7 @@ static NSMutableDictionary *sStringCache;
 	if (!exists)
 	{
 		if (!inCreate) return NO;
-		if (![fmgr createDirectoryAtPath:inPath attributes:nil])
+		if (![fmgr oo_createDirectoryAtPath:inPath attributes:nil])
 		{
 			OOLog(@"resourceManager.write.buildPath.failed", @"Could not create folder %@.", inPath);
 			return NO;
