@@ -3,7 +3,7 @@
 OOJSEngineTimeManagement.h
 
 
-Copyright (C) 2010-2011 Jens Ayton
+Copyright (C) 2010-2013 Jens Ayton
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -630,7 +630,7 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 							   [self profilerOverhead] * 1000.0];
 	
 	NSArray *profileEntries = [self profileEntries];
-	OOUInteger i, count = [profileEntries count];
+	NSUInteger i, count = [profileEntries count];
 	if (count != 0)
 	{
 		[result appendString:@"\n                                                        NAME  T  COUNT    TOTAL     SELF  TOTAL%   SELF%  SELFMAX"];
@@ -843,8 +843,8 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 	_hitCount++;
 	_totalTimeSum += totalTime;
 	_selfTimeSum += selfTime;
-	_totalTimeMax = OOMax_f(_totalTimeMax, totalTime);
-	_selfTimeMax = OOMax_f(_selfTimeMax, selfTime);
+	_totalTimeMax = fmax(_totalTimeMax, totalTime);
+	_selfTimeMax = fmax(_selfTimeMax, selfTime);
 }
 
 
@@ -889,7 +889,7 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 }
 
 
-- (OOUInteger) hitCount
+- (NSUInteger) hitCount
 {
 	return _hitCount;
 }
@@ -985,7 +985,7 @@ static void UpdateProfileForFrame(OOHighResTimeValue now, OOJSProfileStackFrame
 {
 	return [NSDictionary dictionaryWithObjectsAndKeys:
 			_function, @"name",
-			[NSNumber numberWithUnsignedInt:[self hitCount]], @"hitCount",
+			[NSNumber numberWithUnsignedInteger:[self hitCount]], @"hitCount",
 			[NSNumber numberWithDouble:[self totalTimeSum]], @"totalTimeSum",
 			[NSNumber numberWithDouble:[self selfTimeSum]], @"selfTimeSum",
 			[NSNumber numberWithDouble:[self totalTimeAverage]], @"totalTimeAverage",
