@@ -3,7 +3,7 @@
 OOJSPlayerShip.h
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -27,10 +27,12 @@ MA 02110-1301, USA.
 #import "OOJSEntity.h"
 #import "OOJSShip.h"
 #import "OOJSVector.h"
+#import "OOJSQuaternion.h"
 #import "OOJavaScriptEngine.h"
 #import "EntityOOJavaScriptExtensions.h"
 
 #import "PlayerEntity.h"
+#import "PlayerEntityControls.h"
 #import "PlayerEntityContracts.h"
 #import "PlayerEntityScriptMethods.h"
 #import "PlayerEntityLegacyScriptEngine.h"
@@ -59,9 +61,18 @@ static JSBool PlayerShipDisengageAutopilot(JSContext *context, uintN argc, jsval
 static JSBool PlayerShipAwardEquipmentToCurrentPylon(JSContext *context, uintN argc, jsval *vp);
 static JSBool PlayerShipAddPassenger(JSContext *context, uintN argc, jsval *vp);
 static JSBool PlayerShipRemovePassenger(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipAddParcel(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipRemoveParcel(JSContext *context, uintN argc, jsval *vp);
 static JSBool PlayerShipAwardContract(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipRemoveContract(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipSetCustomView(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipResetCustomView(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipTakeInternalDamage(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipBeginHyperspaceCountdown(JSContext *context, uintN argc, jsval *vp);
+static JSBool PlayerShipCancelHyperspaceCountdown(JSContext *context, uintN argc, jsval *vp);
 
-static BOOL ValidateContracts(JSContext *context, uintN argc, jsval *vp, BOOL isCargo, OOSystemID *start, OOSystemID *destination, double *eta, double *fee);
+
+static BOOL ValidateContracts(JSContext *context, uintN argc, jsval *vp, BOOL isCargo, OOSystemID *start, OOSystemID *destination, double *eta, double *fee, double *premium, NSString *functionName);
 
 
 static JSClass sPlayerShipClass =
@@ -88,6 +99,8 @@ enum
 	kPlayerShip_aftShieldRechargeRate,			// aft shield recharge rate, positive float, read-only
 	kPlayerShip_compassMode,					// compass mode, string, read-only
 	kPlayerShip_compassTarget,					// object targeted by the compass, entity, read-only
+	kPlayerShip_currentWeapon,					// shortcut property to _aftWeapon, etc. overrides kShip generic version
+	kPlayerShip_crosshairs,						// custom plist file defining crosshairs
 	kPlayerShip_cursorCoordinates,				// cursor coordinates (unscaled), Vector3D, read only
 	kPlayerShip_cursorCoordinatesInLY,			// cursor coordinates (in LY), Vector3D, read only
 	kPlayerShip_docked,							// docked, boolean, read-only
@@ -102,24 +115,33 @@ enum
 	kPlayerShip_galaxyCoordinatesInLY,			// galaxy coordinates (in LY), Vector3D, read only
 	kPlayerShip_hud,							// hud name identifier, string, read/write
 	kPlayerShip_hudHidden,						// hud visibility, boolean, read/write
+	kPlayerShip_hyperspaceSpinTime,                         // hyperspace spin time, read only
 	kPlayerShip_maxAftShield,					// maximum aft shield charge level, positive float, read-only
 	kPlayerShip_maxForwardShield,				// maximum forward shield charge level, positive float, read-only
+	kPlayerShip_missilesOnline,      // bool (false for ident mode, true for missile mode)
+	kPlayerShip_pitch,							// pitch (overrules Ship)
+	kPlayerShip_price,							// idealised trade-in value decicredits, positive int, read-only
 	kPlayerShip_reticleTargetSensitive,			// target box changes color when primary target in crosshairs, boolean, read/write
+	kPlayerShip_roll,							// roll (overrules Ship)
 	kPlayerShip_scoopOverride,					// Scooping
+	kPlayerShip_serviceLevel,					// servicing level, positive int 75-100, read-only
 	kPlayerShip_specialCargo,					// special cargo, string, read-only
-	kPlayerShip_targetSystem,					// target system id, int, read-only
+	kPlayerShip_targetSystem,					// target system id, int, read-write
 	kPlayerShip_viewDirection,					// view direction identifier, string, read-only
 	kPlayerShip_weaponsOnline,					// weapons online status, boolean, read-only
+	kPlayerShip_yaw,							// yaw (overrules Ship)
 };
 
 
 static JSPropertySpec sPlayerShipProperties[] =
 {
-	// JS name						ID									flags
+	// JS name							ID											flags
 	{ "aftShield",						kPlayerShip_aftShield,						OOJS_PROP_READWRITE_CB },
 	{ "aftShieldRechargeRate",			kPlayerShip_aftShieldRechargeRate,			OOJS_PROP_READONLY_CB },
 	{ "compassMode",					kPlayerShip_compassMode,					OOJS_PROP_READONLY_CB },
 	{ "compassTarget",					kPlayerShip_compassTarget,					OOJS_PROP_READONLY_CB },
+	{ "currentWeapon",					kPlayerShip_currentWeapon,					OOJS_PROP_READWRITE_CB },
+	{ "crosshairs",						kPlayerShip_crosshairs,						OOJS_PROP_READWRITE_CB },
 	{ "cursorCoordinates",				kPlayerShip_cursorCoordinates,				OOJS_PROP_READONLY_CB },
 	{ "cursorCoordinatesInLY",			kPlayerShip_cursorCoordinatesInLY,			OOJS_PROP_READONLY_CB },
 	{ "docked",							kPlayerShip_docked,							OOJS_PROP_READONLY_CB },
@@ -134,15 +156,22 @@ static JSPropertySpec sPlayerShipProperties[] =
 	{ "galaxyCoordinatesInLY",			kPlayerShip_galaxyCoordinatesInLY,			OOJS_PROP_READONLY_CB },
 	{ "hud",							kPlayerShip_hud,							OOJS_PROP_READWRITE_CB },
 	{ "hudHidden",						kPlayerShip_hudHidden,						OOJS_PROP_READWRITE_CB },
+	{ "hyperspaceSpinTime",                         kPlayerShip_hyperspaceSpinTime,                         OOJS_PROP_READONLY_CB },
 	// manifest defined in OOJSManifest.m
 	{ "maxAftShield",					kPlayerShip_maxAftShield,					OOJS_PROP_READONLY_CB },
 	{ "maxForwardShield",				kPlayerShip_maxForwardShield,				OOJS_PROP_READONLY_CB },
+	{ "missilesOnline",      kPlayerShip_missilesOnline,      OOJS_PROP_READONLY_CB },
+	{ "price",							kPlayerShip_price,							OOJS_PROP_READONLY_CB },
+	{ "pitch",							kPlayerShip_pitch,							OOJS_PROP_READONLY_CB },
 	{ "reticleTargetSensitive",			kPlayerShip_reticleTargetSensitive,			OOJS_PROP_READWRITE_CB },
+	{ "roll",							kPlayerShip_roll,							OOJS_PROP_READONLY_CB },
 	{ "scoopOverride",					kPlayerShip_scoopOverride,					OOJS_PROP_READWRITE_CB },
+	{ "serviceLevel",					kPlayerShip_serviceLevel,					OOJS_PROP_READWRITE_CB },
 	{ "specialCargo",					kPlayerShip_specialCargo,					OOJS_PROP_READONLY_CB },
-	{ "targetSystem",					kPlayerShip_targetSystem,					OOJS_PROP_READONLY_CB },
+	{ "targetSystem",					kPlayerShip_targetSystem,					OOJS_PROP_READWRITE_CB },
 	{ "viewDirection",					kPlayerShip_viewDirection,					OOJS_PROP_READONLY_CB },
 	{ "weaponsOnline",					kPlayerShip_weaponsOnline,					OOJS_PROP_READONLY_CB },
+	{ "yaw",							kPlayerShip_yaw,							OOJS_PROP_READONLY_CB },
 	{ 0 }			
 };
 
@@ -150,14 +179,22 @@ static JSPropertySpec sPlayerShipProperties[] =
 static JSFunctionSpec sPlayerShipMethods[] =
 {
 	// JS name						Function							min args
+	{ "addParcel",   					PlayerShipAddParcel,						0 },
 	{ "addPassenger",					PlayerShipAddPassenger,						0 },
 	{ "awardContract",					PlayerShipAwardContract,					0 },
 	{ "awardEquipmentToCurrentPylon",	PlayerShipAwardEquipmentToCurrentPylon,		1 },
+	{ "beginHyperspaceCountdown",           PlayerShipBeginHyperspaceCountdown,                     0 },
+	{ "cancelHyperspaceCountdown",          PlayerShipCancelHyperspaceCountdown,            0 },
 	{ "disengageAutopilot",				PlayerShipDisengageAutopilot,				0 },
 	{ "engageAutopilotToStation",		PlayerShipEngageAutopilotToStation,			1 },
 	{ "launch",							PlayerShipLaunch,							0 },
 	{ "removeAllCargo",					PlayerShipRemoveAllCargo,					0 },
+	{ "removeContract",					PlayerShipRemoveContract,					2 },
+	{ "removeParcel",                               PlayerShipRemoveParcel,                                         1 },
 	{ "removePassenger",				PlayerShipRemovePassenger,					1 },
+	{ "resetCustomView",				PlayerShipResetCustomView,					0 },
+	{ "setCustomView",					PlayerShipSetCustomView,					2 },
+	{ "takeInternalDamage",				PlayerShipTakeInternalDamage,				0 },
 	{ "useSpecialCargo",				PlayerShipUseSpecialCargo,					1 },
 	{ 0 }
 };
@@ -249,6 +286,9 @@ static JSBool PlayerShipGetProperty(JSContext *context, JSObject *this, jsid pro
 	
 	switch (JSID_TO_INT(propID))
 	{
+		case kPlayerShip_hyperspaceSpinTime:
+			return JS_NewNumberValue(context, [player hyperspaceSpinTime], value);
+                       
 		case kPlayerShip_fuelLeakRate:
 			return JS_NewNumberValue(context, [player fuelLeakRate], value);
 			
@@ -295,6 +335,10 @@ static JSBool PlayerShipGetProperty(JSContext *context, JSObject *this, jsid pro
 			// No distinction made internally
 			return JS_NewNumberValue(context, [player shieldRechargeRate], value);
 			
+		case kPlayerShip_missilesOnline:
+			*value = OOJSValueFromBOOL(![player dialIdentEngaged]);
+			return YES;
+
 		case kPlayerShip_galaxyCoordinates:
 			return NSPointToVectorJSValue(context, [player galaxy_coordinates], value);
 			
@@ -303,7 +347,7 @@ static JSBool PlayerShipGetProperty(JSContext *context, JSObject *this, jsid pro
 			
 		case kPlayerShip_cursorCoordinates:
 			return NSPointToVectorJSValue(context, [player cursor_coordinates], value);
-			
+
 		case kPlayerShip_cursorCoordinatesInLY:
 			return VectorToJSValue(context, OOGalacticCoordinatesFromInternal([player cursor_coordinates]), value);
 			
@@ -326,7 +370,11 @@ static JSBool PlayerShipGetProperty(JSContext *context, JSObject *this, jsid pro
 		case kPlayerShip_hud:
 			result = [[player hud] hudName];
 			break;
-			
+
+		case kPlayerShip_crosshairs:
+			result = [[player hud] crosshairDefinition];
+			break;
+
 		case kPlayerShip_hudHidden:
 			*value = OOJSValueFromBOOL([[player hud] isHidden]);
 			return YES;
@@ -338,7 +386,29 @@ static JSBool PlayerShipGetProperty(JSContext *context, JSObject *this, jsid pro
 		case kPlayerShip_viewDirection:
 			*value = OOJSValueFromViewID(context, [UNIVERSE viewDirection]);
 			return YES;
+
+		case kPlayerShip_currentWeapon:
+			result = [player weaponTypeForFacing:[player currentWeaponFacing]];
+			break;
 		
+	  case kPlayerShip_price:
+			return JS_NewNumberValue(context, [UNIVERSE tradeInValueForCommanderDictionary:[player commanderDataDictionary]], value);
+
+	  case kPlayerShip_serviceLevel:
+			return JS_NewNumberValue(context, [player tradeInFactor], value);
+
+			// make roll, pitch, yaw reported to JS use same +/- convention as
+			// for NPC ships
+		case kPlayerShip_pitch:
+			return JS_NewNumberValue(context, -[player flightPitch], value);
+
+		case kPlayerShip_roll:
+			return JS_NewNumberValue(context, -[player flightRoll], value);
+
+		case kPlayerShip_yaw:
+			return JS_NewNumberValue(context, -[player flightYaw], value);
+
+
 		default:
 			OOJSReportBadPropertySelector(context, this, propID, sPlayerShipProperties);
 	}
@@ -361,6 +431,7 @@ static JSBool PlayerShipSetProperty(JSContext *context, JSObject *this, jsid pro
 	PlayerEntity				*player = OOPlayerForScripting();
 	jsdouble					fValue;
 	JSBool						bValue;
+	int32						iValue;
 	NSString					*sValue = nil;
 	OOGalacticHyperspaceBehaviour ghBehaviour;
 	Vector						vValue;
@@ -448,6 +519,26 @@ static JSBool PlayerShipSetProperty(JSContext *context, JSObject *this, jsid pro
 			}
 			break;
 			
+		case kPlayerShip_crosshairs:
+			sValue = OOStringFromJSValue(context, *value);
+			if (sValue == nil)
+			{
+				// reset HUD back to its plist settings
+				NSString *hud = [[[player hud] hudName] retain];
+				[player switchHudTo:hud];
+				[hud release];
+				return YES;
+			}
+			else
+			{
+				if (![[player hud] setCrosshairDefinition:sValue])
+				{
+					OOJSReportWarning(context, @"Crosshair definition file %@ not found or invalid", sValue);
+				}
+				return YES;
+			}
+			break;
+
 		case kPlayerShip_hudHidden:
 			if (JS_ValueToBoolean(context, *value, &bValue))
 			{
@@ -455,6 +546,61 @@ static JSBool PlayerShipSetProperty(JSContext *context, JSObject *this, jsid pro
 				return YES;
 			}
 			break;
+
+	  case kPlayerShip_serviceLevel:
+			if (JS_ValueToNumber(context, *value, &fValue))
+			{
+				int newLevel = (int)fValue;
+				[player adjustTradeInFactorBy:(newLevel-[player tradeInFactor])];
+				return YES;
+			}
+			
+		case kPlayerShip_currentWeapon:
+		{
+			BOOL exists = NO;
+			sValue = JSValueToEquipmentKeyRelaxed(context, *value, &exists);
+			if (!exists || sValue == nil) 
+			{
+				sValue = @"EQ_WEAPON_NONE";
+			}
+			[player setWeaponMount:[player currentWeaponFacing] toWeapon:sValue];
+			return YES;
+		}
+		
+		case kPlayerShip_targetSystem:
+			/* This first check is essential: if removed, it would be
+			 * possible to make jumps of arbitrary length - CIM */
+			if ([player status] != STATUS_ENTERING_WITCHSPACE)
+			{
+				/* These checks though similar are less important. The
+				 * consequences of allowing jump destination to be set in
+				 * flight are not as severe and do not allow the 7LY limit to
+				 * be broken. Nevertheless, it is not allowed. - CIM */
+				if (EXPECT_NOT([player status] != STATUS_DOCKED && [player status] != STATUS_LAUNCHING))
+				{
+					OOJSReportError(context, @"player.ship.targetSystem is read-only unless called when docked.");
+					return NO;
+				}
+				
+				if (JS_ValueToInt32(context, *value, &iValue))
+				{
+					if (iValue >= 0 && iValue < 256)
+					{ 
+						Random_Seed seed = [UNIVERSE systemSeedForSystemNumber:(OOSystemID)iValue];
+						[player setTargetSystemSeed:seed];
+						return YES;
+					}
+					else
+					{
+						return NO;
+					}
+				}
+			}
+			else
+			{
+				OOJSReportError(context, @"player.ship.targetSystem is read-only unless called when docked.");
+				return NO;
+			}
 		
 		default:
 			OOJSReportBadPropertySelector(context, this, propID, sPlayerShipProperties);
@@ -602,7 +748,7 @@ static JSBool PlayerShipAddPassenger(JSContext *context, uintN argc, jsval *vp)
 	PlayerEntity		*player = OOPlayerForScripting();
 	NSString 			*name = nil;
 	OOSystemID			start = 0, destination = 0;
-	jsdouble			eta = 0.0, fee = 0.0;
+	jsdouble			eta = 0.0, fee = 0.0, advance = 0.0;
 	
 	if (argc < 5)
 	{
@@ -617,12 +763,12 @@ static JSBool PlayerShipAddPassenger(JSContext *context, uintN argc, jsval *vp)
 		return NO;
 	}
 	
-	if (!ValidateContracts(context, argc, vp, NO, &start, &destination, &eta, &fee))  return NO; // always go through validate contracts (cargo)
+	if (!ValidateContracts(context, argc, vp, NO, &start, &destination, &eta, &fee, &advance, @"addPassenger"))  return NO; // always go through validate contracts (passenger)
 	
 	// Ensure there's space.
 	if ([player passengerCount] >= [player passengerCapacity])  OOJS_RETURN_BOOL(NO);
 	
-	BOOL OK = [player addPassenger:name start:start destination:destination eta:eta fee:fee];
+	BOOL OK = [player addPassenger:name start:start destination:destination eta:eta fee:fee advance:advance];
 	OOJS_RETURN_BOOL(OK);
 	
 	OOJS_NATIVE_EXIT
@@ -654,6 +800,65 @@ static JSBool PlayerShipRemovePassenger(JSContext *context, uintN argc, jsval *v
 }
 
 
+// addParcel(description: string, start: int, destination: int, ETA: double, fee: double) : Boolean
+static JSBool PlayerShipAddParcel(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	NSString 			*name = nil;
+	OOSystemID			start = 0, destination = 0;
+	jsdouble			eta = 0.0, fee = 0.0, ignore = 0.0;
+	
+	if (argc < 5)
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"addParcel", argc, OOJS_ARGV, nil, @"name, start, destination, ETA, fee");
+		return NO;
+	}
+	
+	name = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	if (EXPECT_NOT(name == nil))
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"addParcel", 1, &OOJS_ARGV[0], nil, @"string");
+		return NO;
+	}
+	
+	if (!ValidateContracts(context, argc, vp, NO, &start, &destination, &eta, &fee, &ignore, @"addParcel"))  return NO; // always go through validate contracts (passenger/parcel mode)
+	
+	// Ensure there's space.
+	
+	BOOL OK = [player addParcel:name start:start destination:destination eta:eta fee:fee];
+	OOJS_RETURN_BOOL(OK);
+	
+	OOJS_NATIVE_EXIT
+}
+
+
+// removeParcel(description :string)
+static JSBool PlayerShipRemoveParcel(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	NSString			*name = nil;
+	BOOL				OK = YES;
+	
+	if (argc > 0)  name = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	if (EXPECT_NOT(name == nil))
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"removeParcel", MIN(argc, 1U), OOJS_ARGV, nil, @"string");
+		return NO;
+	}
+	
+	OK = [player parcelCount] > 0 && [name length] > 0;
+	if (OK)  OK = [player removeParcel:name];
+	
+	OOJS_RETURN_BOOL(OK);
+	
+	OOJS_NATIVE_EXIT
+}
+
+
 // awardContract(quantity: int, commodity: string, start: int, destination: int, eta: double, fee: double) : Boolean
 static JSBool PlayerShipAwardContract(JSContext *context, uintN argc, jsval *vp)
 {
@@ -663,7 +868,7 @@ static JSBool PlayerShipAwardContract(JSContext *context, uintN argc, jsval *vp)
 	NSString 			*key = nil;
 	int32 				qty = 0;
 	OOSystemID			start = 0, destination = 0;
-	jsdouble			eta = 0.0, fee = 0.0;
+	jsdouble			eta = 0.0, fee = 0.0, premium = 0.0;
 	
 	if (argc < 6)
 	{
@@ -684,23 +889,214 @@ static JSBool PlayerShipAwardContract(JSContext *context, uintN argc, jsval *vp)
 		return NO;
 	}
 	
-	if (!ValidateContracts(context, argc, vp, YES, &start, &destination, &eta, &fee))  return NO; // always go through validate contracts (cargo)
+	if (!ValidateContracts(context, argc, vp, YES, &start, &destination, &eta, &fee, &premium, @"awardContract"))  return NO; // always go through validate contracts (cargo)
 	
-	BOOL OK = [player awardContract:qty commodity:key start:start destination:destination eta:eta fee:fee];
+	BOOL OK = [player awardContract:qty commodity:key start:start destination:destination eta:eta fee:fee premium:premium];
 	OOJS_RETURN_BOOL(OK);
 	
 	OOJS_NATIVE_EXIT
 }
 
 
-static BOOL ValidateContracts(JSContext *context, uintN argc, jsval *vp, BOOL isCargo, OOSystemID *start, OOSystemID *destination, double *eta, double *fee)
+// removeContract(commodity: string, destination: int)
+static JSBool PlayerShipRemoveContract(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	NSString			*key = nil;
+	int32				dest = 0;
+	
+	if (argc < 2)
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"removeContract", argc, OOJS_ARGV, nil, @"commodity, destination");
+		return NO;
+	}
+	
+	key = OOStringFromJSValue(context, OOJS_ARGV[0]);
+	
+	if (EXPECT_NOT(key == nil))
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"removeContract", 1, &OOJS_ARGV[0], nil, @"string (commodity identifier)");
+		return NO;
+	}
+	
+	if (!JS_ValueToInt32(context, OOJS_ARGV[1], &dest) || dest < 0 || dest > 255)
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"removeContract", 1, &OOJS_ARGV[1], nil, @"system ID");
+		return NO;
+	}
+	
+	BOOL OK = [player removeContract:key destination:(unsigned)dest];	
+	OOJS_RETURN_BOOL(OK);
+	
+	OOJS_NATIVE_EXIT
+}
+
+
+// setCustomView(position:vector, orientation:quaternion [, weapon:string])
+static JSBool PlayerShipSetCustomView(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	
+	if (argc < 2)
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"setCustomView", argc, OOJS_ARGV, nil, @"position, orientiation, [weapon]");
+		return NO;
+	}
+
+// must be in custom view
+	if ([UNIVERSE viewDirection] != VIEW_CUSTOM) 
+	{
+		OOJSReportError(context, @"PlayerShip.setCustomView only works when custom view is active.");
+		return NO;
+	}
+
+	NSMutableDictionary			*viewData = [NSMutableDictionary dictionaryWithCapacity:3];
+
+	Vector position = kZeroVector;
+	BOOL gotpos = JSValueToVector(context, OOJS_ARGV[0], &position);
+	if (!gotpos)
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"setCustomView", argc, OOJS_ARGV, nil, @"position, orientiation, [weapon]");
+		return NO;
+	}
+	NSString *positionstr = [[NSString alloc] initWithFormat:@"%f %f %f",position.x,position.y,position.z];   
+
+	Quaternion orientation = kIdentityQuaternion;
+	BOOL gotquat = JSValueToQuaternion(context, OOJS_ARGV[1], &orientation);
+	if (!gotquat)
+	{
+		OOJSReportBadArguments(context, @"PlayerShip", @"setCustomView", argc, OOJS_ARGV, nil, @"position, orientiation, [weapon]");
+		return NO;
+	}
+	NSString *orientationstr = [[NSString alloc] initWithFormat:@"%f %f %f %f",orientation.w,orientation.x,orientation.y,orientation.z];
+
+	[viewData setObject:positionstr forKey:@"view_position"];
+	[viewData setObject:orientationstr forKey:@"view_orientation"];
+
+	if (argc > 2)
+	{
+		NSString* facing = OOStringFromJSValue(context,OOJS_ARGV[2]);
+		[viewData setObject:facing forKey:@"weapon_facing"];
+	} 
+
+	[player setCustomViewDataFromDictionary:viewData];
+	[player noteSwitchToView:VIEW_CUSTOM fromView:VIEW_CUSTOM];
+
+	[positionstr release];
+	[orientationstr release];
+
+	OOJS_RETURN_BOOL(YES);
+	OOJS_NATIVE_EXIT
+}
+
+
+// resetCustomView()
+static JSBool PlayerShipResetCustomView(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	
+// must be in custom view
+	if ([UNIVERSE viewDirection] != VIEW_CUSTOM) 
+	{
+		OOJSReportError(context, @"PlayerShip.setCustomView only works when custom view is active.");
+		return NO;
+	}
+
+	[player resetCustomView];
+	[player noteSwitchToView:VIEW_CUSTOM fromView:VIEW_CUSTOM];
+
+	OOJS_RETURN_BOOL(YES);
+	OOJS_NATIVE_EXIT
+}
+
+// takeInternalDamage()
+static JSBool PlayerShipTakeInternalDamage(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	
+	BOOL took = [player takeInternalDamage];
+
+	OOJS_RETURN_BOOL(took);
+	OOJS_NATIVE_EXIT
+}
+
+
+// beginHyperspaceCountdown([int: spin_time])
+static JSBool PlayerShipBeginHyperspaceCountdown(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+	
+	PlayerEntity		*player = OOPlayerForScripting();
+	int32                           spin_time;
+	int32                           witchspaceSpinUpTime = 0;
+	BOOL begun = NO;
+	if (argc < 1) 
+	{
+		witchspaceSpinUpTime = 0;
+	}
+	else
+	{
+		if (!JS_ValueToInt32(context, OOJS_ARGV[0], &spin_time) || spin_time < 5 || spin_time > 60)
+		{
+			OOJSReportBadArguments(context, @"PlayerShip", @"beginHyperspaceCountdown", 1, &OOJS_ARGV[0], nil, @"between 5 and 60 seconds");
+			return NO;
+		}
+		if (spin_time < 5) 
+		{
+			witchspaceSpinUpTime = 5;
+		}
+		else
+		{
+			witchspaceSpinUpTime = spin_time;
+		}
+	}
+	if ([player hasHyperspaceMotor] && [player status] == STATUS_IN_FLIGHT && [player witchJumpChecklist:false])
+	{
+		[player beginWitchspaceCountdown:witchspaceSpinUpTime];
+		begun = YES;
+	}
+	OOJS_RETURN_BOOL(begun);
+	OOJS_NATIVE_EXIT
+}
+
+
+// cancelHyperspaceCountdown()
+static JSBool PlayerShipCancelHyperspaceCountdown(JSContext *context, uintN argc, jsval *vp)
+{
+	OOJS_NATIVE_ENTER(context)
+       
+	PlayerEntity            *player = OOPlayerForScripting();
+       
+	BOOL cancelled = NO;
+	if ([player hasHyperspaceMotor] && [player status] == STATUS_WITCHSPACE_COUNTDOWN)
+	{
+		[player cancelWitchspaceCountdown];
+		[player setJumpType:false];
+		cancelled = YES;
+	}
+
+	OOJS_RETURN_BOOL(cancelled);
+	OOJS_NATIVE_EXIT
+		
+}
+
+
+
+static BOOL ValidateContracts(JSContext *context, uintN argc, jsval *vp, BOOL isCargo, OOSystemID *start, OOSystemID *destination, double *eta, double *fee, double *premium, NSString *functionName)
 {
 	OOJS_PROFILE_ENTER
 	
 	NSCParameterAssert(context != NULL && vp != NULL && start != NULL && destination != NULL && eta != NULL && fee != NULL);
 	
 	unsigned		offset = isCargo ? 2 : 1;
-	NSString		*functionName = isCargo ? @"awardContract" : @"addPassenger";
 	jsdouble		fValue;
 	int32			iValue;
 	
@@ -732,6 +1128,11 @@ static BOOL ValidateContracts(JSContext *context, uintN argc, jsval *vp, BOOL is
 		return NO;
 	}
 	*fee = fValue;
+
+	if (argc > offset+4 && JS_ValueToNumber(context, OOJS_ARGV[offset + 4], &fValue) && isfinite(fValue) && fValue >= 0.0)
+	{
+		*premium = fValue;
+	}
 	
 	return YES;
 	
