@@ -3,7 +3,7 @@
 OOJSScript.m
 
 JavaScript support for Oolite
-Copyright (C) 2007-2011 David Taylor and Jens Ayton.
+Copyright (C) 2007-2013 David Taylor and Jens Ayton.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -68,12 +68,15 @@ static JSScript *ScriptWithCompiledData(JSContext *context, NSData *data);
 static NSString *StrippedName(NSString *string);
 
 
+static JSBool ScriptAddProperty(JSContext *context, JSObject *this, jsid propID, jsval *value);
+
+
 static JSClass sScriptClass =
 {
 	"Script",
 	JSCLASS_HAS_PRIVATE,
 	
-	JS_PropertyStub,
+	ScriptAddProperty,
 	JS_PropertyStub,
 	JS_PropertyStub,
 	JS_StrictPropertyStub,
@@ -360,10 +363,7 @@ static JSFunctionSpec sScriptMethods[] =
 
 - (void)runWithTarget:(Entity *)target
 {
-	JSContext *context = OOJSAcquireContext();
-	jsval arg = OOJSValueFromEntityStatus(context, [PLAYER status]);
-	[self callMethod:OOJSID("tickle") inContext:context withArguments:&arg count:1 result:NULL];
-	OOJSRelinquishContext(context);
+	
 }
 
 
@@ -392,8 +392,8 @@ static JSFunctionSpec sScriptMethods[] =
 			JS_ClearPendingException(context);
 		}
 		
-		OOLog(@"script.trace.javaScript", @"Calling [%@].%@()", [self name], OOStringFromJSID(methodID));
-		OOLogIndentIf(@"script.trace.javaScript");
+		OOLog(@"script.javaScript.call", @"Calling [%@].%@()", [self name], OOStringFromJSID(methodID));
+		OOLogIndentIf(@"script.javaScript.call");
 #endif
 		
 		// Push self on stack of running scripts.
@@ -419,7 +419,7 @@ static JSFunctionSpec sScriptMethods[] =
 		sRunningStack = stackElement.back;
 		
 #ifndef NDEBUG
-		OOLogOutdentIf(@"script.trace.javaScript");
+		OOLogOutdentIf(@"script.javaScript.call");
 #endif
 	}
 	
@@ -610,6 +610,24 @@ void InitOOJSScript(JSContext *context, JSObject *global)
 }
 
 
+static JSBool ScriptAddProperty(JSContext *context, JSObject *this, jsid propID, jsval *value)
+{
+	// Complain about attempts to set the property tickle.
+	if (JSID_IS_STRING(propID))
+	{
+		JSString *propName = JSID_TO_STRING(propID);
+		JSBool match;
+		if (JS_StringEqualsAscii(context, propName, "tickle", &match) && match)
+		{
+			OOJSScript *thisScript = OOJSNativeObjectOfClassFromJSObject(context, this, [OOJSScript class]);
+			OOJSReportWarning(context, @"Script %@ appears to use the tickle() event handler, which is no longer supported.", [thisScript name]);
+		}
+	}
+	
+	return YES;
+}
+
+
 static void AddStackToArrayReversed(NSMutableArray *array, RunningStack *stack)
 {
 	if (stack != NULL)
@@ -704,7 +722,10 @@ static JSScript *ScriptWithCompiledData(JSContext *context, NSData *data)
 	xdr = JS_XDRNewMem(context, JSXDR_DECODE);
 	if (xdr != NULL)
 	{
-		JS_XDRMemSetData(xdr, (void *)[data bytes], [data length]);
+		NSUInteger length = [data length];
+		if (EXPECT_NOT(length > UINT32_MAX))  return NULL;
+		
+		JS_XDRMemSetData(xdr, (void *)[data bytes], (uint32_t)length);
 		if (!JS_XDRScript(xdr, &result))  result = NULL;
 		
 		JS_XDRMemSetData(xdr, NULL, 0);	// Don't let it be freed by XDRDestroy
