@@ -3,7 +3,7 @@
 OOJavaScriptEngine.m
 
 JavaScript support for Oolite
-Copyright (C) 2007-2011 David Taylor and Jens Ayton.
+Copyright (C) 2007-2013 David Taylor and Jens Ayton.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,6 +36,7 @@ MA 02110-1301, USA.
 #import "ResourceManager.h"
 #import "NSNumberOOExtensions.h"
 #import "OOConstToJSString.h"
+#import "OOVisualEffectEntity.h"
 
 #import "OOJSGlobal.h"
 #import "OOJSMissionVariables.h"
@@ -45,6 +46,8 @@ MA 02110-1301, USA.
 #import "OOJSEntity.h"
 #import "OOJSShip.h"
 #import "OOJSStation.h"
+#import "OOJSDock.h"
+#import "OOJSVisualEffect.h"
 #import "OOJSPlayer.h"
 #import "OOJSPlayerShip.h"
 #import "OOJSManifest.h"
@@ -233,7 +236,7 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 
 @implementation OOJavaScriptEngine
 
-+ (OOJavaScriptEngine *)sharedEngine
++ (OOJavaScriptEngine *) sharedEngine
 {
 	if (sSharedEngine == nil)  sSharedEngine = [[self alloc] init];
 	
@@ -270,7 +273,7 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 	assert(sizeof(jschar) == sizeof(unichar));
 	
 	// initialize the JS run time, and return result in runtime.
-	_runtime = JS_NewRuntime(8L * 1024L * 1024L);
+	_runtime = JS_NewRuntime(32L * 1024L * 1024L);
 	
 	// if runtime creation failed, end the program here.
 	if (_runtime == NULL)
@@ -344,6 +347,8 @@ static void ReportJSError(JSContext *context, const char *message, JSErrorReport
 	InitOOJSEntity(gOOJSMainThreadContext, _globalObject);
 	InitOOJSShip(gOOJSMainThreadContext, _globalObject);
 	InitOOJSStation(gOOJSMainThreadContext, _globalObject);
+	InitOOJSDock(gOOJSMainThreadContext, _globalObject);
+	InitOOJSVisualEffect(gOOJSMainThreadContext, _globalObject);
 	InitOOJSPlayer(gOOJSMainThreadContext, _globalObject);
 	InitOOJSPlayerShip(gOOJSMainThreadContext, _globalObject);
 	InitOOJSManifest(gOOJSMainThreadContext, _globalObject);
@@ -719,7 +724,8 @@ void OOJSDumpStack(JSContext *context)
 {
 	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 	
-	NS_DURING
+	@try
+	{
 		JSStackFrame	*frame = NULL;
 		unsigned		idx = 0;
 		unsigned		skip = sErrorHandlerStackSkip;
@@ -833,19 +839,21 @@ void OOJSDumpStack(JSContext *context)
 				JS_PutPropertyDescArray(context, &properties);
 			}
 		}
-	NS_HANDLER
-		OOLog(kOOLogException, @"Exception during JavaScript stack trace: %@:%@", [localException name], [localException reason]);
-	NS_ENDHANDLER
+	}
+	@catch (NSException *exception)
+	{
+		OOLog(kOOLogException, @"Exception during JavaScript stack trace: %@:%@", [exception name], [exception reason]);
+	}
 	
 	[pool release];
 }
 
 
 static const char *sConsoleScriptName;	// Lifetime is lifetime of script object, which is forever.
-static OOUInteger sConsoleEvalLineNo;
+static NSUInteger sConsoleEvalLineNo;
 
 
-static void GetLocationNameAndLine(JSContext *context, JSStackFrame *stackFrame, const char **name, OOUInteger *line)
+static void GetLocationNameAndLine(JSContext *context, JSStackFrame *stackFrame, const char **name, NSUInteger *line)
 {
 	NSCParameterAssert(context != NULL && stackFrame != NULL && name != NULL && line != NULL);
 	
@@ -874,9 +882,9 @@ NSString *OOJSDescribeLocation(JSContext *context, JSStackFrame *stackFrame)
 	NSCParameterAssert(context != NULL && stackFrame != NULL);
 	
 	const char	*fileName;
-	OOUInteger	lineNo;
+	NSUInteger	lineNo;
 	GetLocationNameAndLine(context, stackFrame, &fileName, &lineNo);
-	if (fileName == NULL)  return NO;
+	if (fileName == NULL)  return nil;
 	
 	// If this stops working, we probably need to switch to strcmp().
 	if (fileName == sConsoleScriptName && lineNo >= sConsoleEvalLineNo)  return @"<console input>";
@@ -889,7 +897,7 @@ NSString *OOJSDescribeLocation(JSContext *context, JSStackFrame *stackFrame)
 	NSString	*shortFileName = [fileNameObj lastPathComponent];
 	if (![[shortFileName lowercaseString] isEqualToString:@"script.js"])  fileNameObj = shortFileName;
 	
-	return [NSString stringWithFormat:@"%@:%u", fileNameObj, lineNo];
+	return [NSString stringWithFormat:@"%@:%lu", fileNameObj, lineNo];
 }
 
 
@@ -945,8 +953,6 @@ jsid OOJSIDFromString(NSString *string)
 	
 	OOJSRelinquishContext(context);
 	
-	return INTERNED_STRING_TO_JSID(jsString);
-	
 	if (EXPECT(jsString != NULL))  return INTERNED_STRING_TO_JSID(jsString);
 	else  return JSID_VOID;
 }
@@ -992,15 +998,18 @@ void OOJSReportErrorForCaller(JSContext *context, NSString *scriptClass, NSStrin
 	va_list					args;
 	NSString				*msg = nil;
 	
-	NS_DURING
+	@try
+	{
 		va_start(args, format);
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		va_end(args);
 		
 		OOJSReportError(context, @"%@%@", CallerPrefix(scriptClass, function), msg);
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
 	[msg release];
 }
 
@@ -1011,12 +1020,15 @@ void OOJSReportErrorWithArguments(JSContext *context, NSString *format, va_list
 	
 	NSCParameterAssert(JS_IsInRequest(context));
 	
-	NS_DURING
+	@try
+	{
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		JS_ReportError(context, "%s", [msg UTF8String]);
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
 	[msg release];
 }
 
@@ -1058,15 +1070,18 @@ void OOJSReportWarningForCaller(JSContext *context, NSString *scriptClass, NSStr
 	va_list					args;
 	NSString				*msg = nil;
 	
-	NS_DURING
+	@try
+	{
 		va_start(args, format);
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		va_end(args);
 		
 		OOJSReportWarning(context, @"%@%@", CallerPrefix(scriptClass, function), msg);
-	NS_HANDLER
-	// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
+	@catch (id exception)
+	{
+		// Squash any secondary errors during error handling.
+	}
 	[msg release];
 }
 
@@ -1075,12 +1090,15 @@ void OOJSReportWarningWithArguments(JSContext *context, NSString *format, va_lis
 {
 	NSString				*msg = nil;
 	
-	NS_DURING
+	@try
+	{
 		msg = [[NSString alloc] initWithFormat:format arguments:args];
 		JS_ReportWarning(context, "%s", [msg UTF8String]);
-	NS_HANDLER
-	// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
+	@catch (id exception)
+	{
+		// Squash any secondary errors during error handling.
+	}
 	[msg release];
 }
 
@@ -1106,15 +1124,18 @@ void OOJSReportBadPropertyValue(JSContext *context, JSObject *thisObj, jsid prop
 
 void OOJSReportBadArguments(JSContext *context, NSString *scriptClass, NSString *function, uintN argc, jsval *argv, NSString *message, NSString *expectedArgsDescription)
 {
-	NS_DURING
+	@try
+	{
 		if (message == nil)  message = @"Invalid arguments";
 		message = [NSString stringWithFormat:@"%@ %@", message, [NSString stringWithJavaScriptParameters:argv count:argc inContext:context]];
 		if (expectedArgsDescription != nil)  message = [NSString stringWithFormat:@"%@ -- expected %@", message, expectedArgsDescription];
 		
 		OOJSReportErrorForCaller(context, scriptClass, function, @"%@.", message);
-	NS_HANDLER
-	// Squash any secondary errors during error handling.
-	NS_ENDHANDLER
+	}
+	@catch (id exception)
+	{
+		// Squash any secondary errors during error handling.
+	}
 }
 
 
@@ -1168,20 +1189,25 @@ static JSObject *JSArrayFromNSArray(JSContext *context, NSArray *array)
 	OOJS_PROFILE_ENTER
 	
 	JSObject				*result = NULL;
-	unsigned				i;
-	unsigned				count;
-	jsval					value;
 	
 	if (array == nil)  return NULL;
 	
-	NS_DURING
+	@try
+	{
+		NSUInteger fullCount = [array count];
+		if (EXPECT_NOT(fullCount > INT32_MAX))
+		{
+			return NULL;
+		}
+		
+		uint32_t i, count = (int32_t)fullCount;
+		
 		result = JS_NewArrayObject(context, 0, NULL);
 		if (result != NULL)
 		{
-			count = [array count];
 			for (i = 0; i != count; ++i)
 			{
-				value = [[array objectAtIndex:i] oo_jsValueInContext:context];
+				jsval value = [[array objectAtIndex:i] oo_jsValueInContext:context];
 				BOOL OK = JS_SetElement(context, result, i, &value);
 				
 				if (EXPECT_NOT(!OK))
@@ -1191,9 +1217,11 @@ static JSObject *JSArrayFromNSArray(JSContext *context, NSArray *array)
 				}
 			}
 		}
-	NS_HANDLER
+	}
+	@catch (id ex)
+	{
 		result = NULL;
-	NS_ENDHANDLER
+	}
 	
 	return (JSObject *)result;
 	
@@ -1248,7 +1276,8 @@ static JSObject *JSObjectFromNSDictionary(JSContext *context, NSDictionary *dict
 	
 	if (dictionary == nil)  return NULL;
 	
-	NS_DURING
+	@try
+	{
 		result = JS_NewObject(context, NULL, NULL, NULL);	// create object of class Object
 		if (result != NULL)
 		{
@@ -1280,9 +1309,11 @@ static JSObject *JSObjectFromNSDictionary(JSContext *context, NSDictionary *dict
 				if (EXPECT_NOT(!OK))  break;
 			}
 		}
-	NS_HANDLER
+	}
+	@catch (id exception)
+	{
 		OK = NO;
-	NS_ENDHANDLER
+	}
 	
 	if (EXPECT_NOT(!OK))
 	{
@@ -1515,7 +1546,7 @@ NSString *OOStringFromJSString(JSContext *context, JSString *string)
 	{
 		return nil;
 	}
-
+	
 	OOJS_PROFILE_EXIT
 }
 
@@ -1680,19 +1711,7 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 
 @implementation NSString (OOJavaScriptExtensions)
 
-+ (id) stringOrNilWithJavaScriptValue:(jsval)value inContext:(JSContext *)context
-{
-	return OOStringFromJSValue(context, value);
-}
-
-
-+ (id) stringWithJavaScriptValue:(jsval)value inContext:(JSContext *)context
-{
-	return OOStringFromJSValueEvenIfNull(context, value);
-}
-
-
-+ (id) stringWithJavaScriptParameters:(jsval *)params count:(uintN)count inContext:(JSContext *)context
++ (NSString *) stringWithJavaScriptParameters:(jsval *)params count:(uintN)count inContext:(JSContext *)context
 {
 	OOJS_PROFILE_ENTER
 	
@@ -1744,7 +1763,7 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 }
 
 
-+ (id) concatenationOfStringsFromJavaScriptValues:(jsval *)values count:(size_t)count separator:(NSString *)separator inContext:(JSContext *)context
++ (NSString *) concatenationOfStringsFromJavaScriptValues:(jsval *)values count:(size_t)count separator:(NSString *)separator inContext:(JSContext *)context
 {
 	OOJS_PROFILE_ENTER
 	
@@ -1757,7 +1776,7 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 	
 	for (i = 0; i != count; ++i)
 	{
-		element = [NSString stringWithJavaScriptValue:values[i] inContext:context];
+		element = OOStringFromJSValueEvenIfNull(context, values[i]);
 		if (result == nil)  result = [[element mutableCopy] autorelease];
 		else
 		{
@@ -1777,12 +1796,12 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 	OOJS_PROFILE_ENTER
 	
 	NSMutableString			*result = nil;
-	unsigned				i, length;
+	NSUInteger				i, length;
 	unichar					c;
 	NSAutoreleasePool		*pool = nil;
 	
 	length = [self length];
-	result = [NSMutableString stringWithCapacity:[self length]];
+	result = [NSMutableString stringWithCapacity:length];
 	
 	// Not hugely efficient.
 	pool = [[NSAutoreleasePool alloc] init];
@@ -1880,38 +1899,24 @@ NSString *OOJSDescribeValue(JSContext *context, jsval value, BOOL abbreviateObje
 	BOOL					isFloat = NO;
 	long long				longLongValue;
 	
-#if 0
-	/*	Under GNUstep, it is not possible to distinguish between boolean
-		NSNumbers and integer NSNumbers - there is no such distinction.
-		It is better to convert booleans to integers than vice versa.
-	*/
-	if ([self oo_isBoolean])
+	isFloat = [self oo_isFloatingPointNumber];
+	if (!isFloat)
 	{
-		if ([self boolValue])  result = JSVAL_TRUE;
-		else  result = JSVAL_FALSE;
+		longLongValue = [self longLongValue];
+		if (longLongValue < (long long)JSVAL_INT_MIN || (long long)JSVAL_INT_MAX < longLongValue)
+		{
+			// values outside JSVAL_INT range are returned as doubles.
+			isFloat = YES;
+		}
+	}
+	
+	if (isFloat)
+	{
+		if (!JS_NewNumberValue(context, [self doubleValue], &result)) result = JSVAL_VOID;
 	}
 	else
-#endif
 	{
-		isFloat = [self oo_isFloatingPointNumber];
-		if (!isFloat)
-		{
-			longLongValue = [self longLongValue];
-			if (longLongValue < (long long)JSVAL_INT_MIN || (long long)JSVAL_INT_MAX < longLongValue)
-			{
-				// values outside JSVAL_INT range are returned as doubles.
-				isFloat = YES;
-			}
-		}
-		
-		if (isFloat)
-		{
-			if (!JS_NewNumberValue(context, [self doubleValue], &result)) result = JSVAL_VOID;
-		}
-		else
-		{
-			result = INT_TO_JSVAL(longLongValue);
-		}
+		result = INT_TO_JSVAL((int32_t)longLongValue);
 	}
 	
 	return result;
@@ -2221,6 +2226,11 @@ NSDictionary *OOJSDictionaryFromJSObject(JSContext *context, JSObject *object)
 		}
 		else if (JSID_IS_INT(thisID))
 		{
+			/* this causes problems with native functions which expect string keys
+			 * e.g. in mission.runScreen with the 'choices' parameter
+			 * should this instead be making the objKey a string?
+			 * is there anything that relies on the current behaviour?
+			 * - CIM 15/2/13 */
 			objKey = [NSNumber numberWithInt:JSID_TO_INT(thisID)];
 		}
 		else
@@ -2290,8 +2300,7 @@ NSDictionary *OOJSDictionaryFromStringTable(JSContext *context, jsval tableValue
 		
 		if (objKey != nil && !JSVAL_IS_VOID(value))
 		{
-			// Note: we want nulls and undefines included, so not OOStringFromJSValue().
-			objValue = [NSString stringWithJavaScriptValue:value inContext:context];
+			objValue = OOStringFromJSValueEvenIfNull(context, value);
 			
 			if (objValue != nil)
 			{
@@ -2458,7 +2467,7 @@ static id JSArrayConverter(JSContext *context, JSObject *array)
 
 static id JSStringConverter(JSContext *context, JSObject *object)
 {
-	return [NSString stringOrNilWithJavaScriptValue:OBJECT_TO_JSVAL(object) inContext:context];
+	return OOStringFromJSValue(context, OBJECT_TO_JSVAL(object));
 }
 
 
