@@ -4,7 +4,7 @@ OOLegacyScriptWhitelist.m
 
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -41,7 +41,7 @@ struct SanStackElement
 {
 	SanStackElement		*back;
 	NSString			*key;		// Dictionary key; nil for arrays.
-	OOUInteger			index;		// Array index if key is nil.
+	NSUInteger			index;		// Array index if key is nil.
 };
 
 
@@ -74,7 +74,7 @@ static NSArray *OOSanitizeLegacyScriptInternal(NSArray *script, SanStackElement
 	NSMutableArray				*result = nil;
 	NSEnumerator				*statementEnum = nil;
 	id							statement = nil;
-	OOUInteger					index = 0;
+	NSUInteger					index = 0;
 	
 	pool = [[NSAutoreleasePool alloc] init];
 	
@@ -130,7 +130,7 @@ static NSArray *OOSanitizeLegacyScriptConditionsInternal(NSArray *conditions, Sa
 	NSMutableArray				*result = nil;
 	NSArray						*tokens = nil;
 	BOOL						OK = YES;
-	OOUInteger					index = 0;
+	NSUInteger					index = 0;
 	
 	if (OOLegacyConditionsAreSanitized(conditions) || conditions == nil)  return conditions;
 	
@@ -177,7 +177,7 @@ BOOL OOLegacyConditionsAreSanitized(NSArray *conditions)
 static NSArray *SanitizeCondition(NSString *condition, SanStackElement *stack)
 {
 	NSArray						*tokens = nil;
-	OOUInteger					i, tokenCount;
+	NSUInteger					i, tokenCount;
 	OOOperationType				opType;
 	NSString					*selectorString = nil;
 	NSString					*sanitizedSelectorString = nil;
@@ -372,7 +372,7 @@ static NSArray *SanitizeConditionalStatement(NSDictionary *statement, SanStackEl
 static NSArray *SanitizeActionStatement(NSString *statement, SanStackElement *stack, BOOL allowAIMethods)
 {
 	NSMutableArray				*tokens = nil;
-	OOUInteger					tokenCount;
+	NSUInteger					tokenCount;
 	NSString					*rawSelectorString = nil;
 	NSString					*selectorString = nil;
 	NSString					*argument = nil;
@@ -407,14 +407,7 @@ static NSArray *SanitizeActionStatement(NSString *statement, SanStackElement *st
 			argument = [tokens componentsJoinedByString:@" "];
 		}
 		
-#if OOLITE_HAVE_STRING_BY_REPLACING
 		argument = [argument stringByReplacingOccurrencesOfString:@"[credits_number]" withString:@"[_oo_legacy_credits_number]"];
-#else
-		if ([argument rangeOfString:@"[credits_number]"].location != NSNotFound)
-		{
-			argument = [[argument componentsSeparatedByString:@"[credits_number]"] componentsJoinedByString:@"[_oo_legacy_credits_number]"];
-		}
-#endif
 	}
 	
 	return [NSArray arrayWithObjects:[NSNumber numberWithBool:NO], selectorString, argument, nil];
