@@ -5,7 +5,7 @@ Universe.h
 Manages a lot of stuff that isn't managed somewhere else.
 
 Oolite
-Copyright (C) 2004-2011 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -33,6 +33,8 @@ MA 02110-1301, USA.
 #import "OOTypes.h"
 #import "OOSound.h"
 #import "OOJSPropID.h"
+#import "OOStellarBody.h"
+#import "OOEntityWithDrawable.h"
 
 
 #if OOLITE_ESPEAK
@@ -40,8 +42,9 @@ MA 02110-1301, USA.
 #endif
 
 @class	GameController, CollisionRegion, MyOpenGLView, GuiDisplayGen,
-		Entity, ShipEntity, StationEntity, OOPlanetEntity, OOSunEntity,
-		PlayerEntity, OORoleSet;
+	Entity, ShipEntity, StationEntity, OOPlanetEntity, OOSunEntity,
+	OOVisualEffectEntity, PlayerEntity, OORoleSet, WormholeEntity, 
+	DockEntity, OOJSScript;
 
 
 typedef BOOL (*EntityFilterPredicate)(Entity *entity, void *parameter);
@@ -122,12 +125,11 @@ enum
 #define SHIPYARD_KEY_PRICE					@"price"
 #define SHIPYARD_KEY_PERSONALITY			@"personality"
 #define SHIPYARD_KEY_DESCRIPTION			@"description"
+// default passenger berth required space
+#define PASSENGER_BERTH_SPACE				5
 
 #define PLANETINFO_UNIVERSAL_KEY			@"universal"
 
-// Derived constants (MIN_ENTITY_UID, MAX_ENTITY_UID) are defined in OOTypes.h
-#define	UNIVERSE_MAX_ENTITIES				2048
-
 #define OOLITE_EXCEPTION_LOOPING			@"OoliteLoopingException"
 #define OOLITE_EXCEPTION_DATA_NOT_FOUND		@"OoliteDataNotFoundException"
 #define OOLITE_EXCEPTION_FATAL				@"OoliteFatalException"
@@ -140,6 +142,8 @@ enum
 
 #define DEMO_LIGHT_POSITION 5000.0f, 25000.0f, -10000.0f
 
+#define MIN_DISTANCE_TO_BUOY			750.0f // don't add ships within this distance
+#define MIN_DISTANCE_TO_BUOY2			(MIN_DISTANCE_TO_BUOY * MIN_DISTANCE_TO_BUOY)
 
 #ifndef OO_LOCALIZATION_TOOLS
 #define OO_LOCALIZATION_TOOLS	1
@@ -150,16 +154,11 @@ enum
 #endif
 
 
-typedef OOUInteger	OOTechLevelID;		// 0..14, 99 is special. NSNotFound is used, so OOUInteger required.
-typedef uint8_t		OOGovernmentID;		// 0..7
-typedef uint8_t		OOEconomyID;		// 0..7
-
-
 @interface Universe: OOWeakRefObject
 {
 @public
 	// use a sorted list for drawing and other activities
-	Entity					*sortedEntities[UNIVERSE_MAX_ENTITIES];
+	Entity					*sortedEntities[UNIVERSE_MAX_ENTITIES + 1];	// One extra for padding; see -doRemoveEntity:.
 	unsigned				n_entities;
 	
 	int						cursor_row;
@@ -170,7 +169,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	GLfloat					stars_ambient[4];
 	
 @private
-	OOUInteger				_sessionID;
+	NSUInteger				_sessionID;
 	
 	// colors
 	GLfloat					sun_diffuse[4];
@@ -204,7 +203,6 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	
 	BOOL					displayGUI;
 	BOOL					wasDisplayGUI;
-	BOOL					displayCursor;
 	
 	BOOL					autoSaveNow;
 	BOOL					autoSave;
@@ -221,7 +219,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	OOTimeAbsolute			demo_start_time;
 	GLfloat					demo_start_z;
 	int						demo_stage;
-	int						demo_ship_index;
+	NSUInteger				demo_ship_index;
 	NSArray					*demo_ships;
 	
 	GLfloat					main_light_position[4];
@@ -271,8 +269,6 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	
 	NSMutableDictionary		*localPlanetInfoOverrides;
 	
-	NSException				*exception;
-	
 	NSMutableArray			*activeWormholes;
 	
 	NSMutableArray			*characterPool;
@@ -287,7 +283,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	
 #if OOLITE_SPEECH_SYNTH
 #if OOLITE_MAC_OS_X
-	NSSpeechSynthesizer		*speechSynthesizer;		// use this from OS X 10.3 onwards
+	NSSpeechSynthesizer		*speechSynthesizer;
 #elif OOLITE_ESPEAK
 	const espeak_VOICE		**espeak_voices;
 	unsigned int			espeak_voice_count;
@@ -300,15 +296,22 @@ typedef uint8_t		OOEconomyID;		// 0..7
 #endif
 	BOOL					doProcedurallyTexturedPlanets;
 	
+	GLfloat					frustum[6][4];
+	
+	NSMutableDictionary		*conditionScripts;
+	
 	BOOL					_pauseMessage;
 	BOOL					_autoCommLog;
 	BOOL					_permanentCommLog;
+	BOOL					_witchspaceBreakPattern;
+	BOOL					_dockingClearanceProtocolActive;
+	BOOL					_doingStartUp;
 }
 
 - (id)initWithGameView:(MyOpenGLView *)gameView;
 
 // SessionID: a value that's incremented when the game is reset.
-- (OOUInteger) sessionID;
+- (NSUInteger) sessionID;
 
 - (BOOL) doProcedurallyTexturedPlanets;
 - (void) setDoProcedurallyTexturedPlanets:(BOOL) value;
@@ -319,19 +322,22 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (void) reinitAndShowDemo:(BOOL)showDemo;
 
-- (int) entityCount;
+- (BOOL) doingStartUp;	// True during initial game startup (not reset).
+
+- (NSUInteger) entityCount;
 #ifndef NDEBUG
 - (void) debugDumpEntities;
 - (NSArray *) entityList;
 #endif
 
 - (void) pauseGame;
-- (BOOL) isGamePaused;
 
+- (void) carryPlayerOn:(StationEntity*)carrier inWormhole:(WormholeEntity*)wormhole;
 - (void) setUpUniverseFromStation;
 - (void) setUpUniverseFromWitchspace;
 - (void) setUpUniverseFromMisjump;
 - (void) setUpWitchspace;
+- (void) setUpWitchspaceBetweenSystem:(Random_Seed)s1 andSystem:(Random_Seed)s2;
 - (void) setUpSpace;
 - (void) setLighting;
 - (void) forceLightSwitch;
@@ -353,15 +359,24 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (void) witchspaceShipWithPrimaryRole:(NSString *)role;
 - (ShipEntity *) spawnShipWithRole:(NSString *) desc near:(Entity *) entity;
 
+- (OOVisualEffectEntity *) addVisualEffectAt:(Vector)pos withKey:(NSString *)key;
 - (ShipEntity *) addShipAt:(Vector)pos withRole:(NSString *)role withinRadius:(GLfloat)radius;
 - (NSArray *) addShipsAt:(Vector)pos withRole:(NSString *)role quantity:(unsigned)count withinRadius:(GLfloat)radius asGroup:(BOOL)isGroup;
 - (NSArray *) addShipsToRoute:(NSString *)route withRole:(NSString *)role quantity:(unsigned)count routeFraction:(double)routeFraction asGroup:(BOOL)isGroup;
 
 - (BOOL) roleIsPirateVictim:(NSString *)role;
 
+- (void) forceWitchspaceEntries;
 - (void) addWitchspaceJumpEffectForShip:(ShipEntity *)ship;
+- (GLfloat) safeWitchspaceExitDistance;
 
 - (void) setUpBreakPattern:(Vector)pos orientation:(Quaternion)q forDocking:(BOOL)forDocking;
+- (BOOL) witchspaceBreakPattern;
+- (void) setWitchspaceBreakPattern:(BOOL)newValue;
+
+- (BOOL) dockingClearanceProtocolActive;
+- (void) setDockingClearanceProtocolActive:(BOOL)newValue;
+
 - (void) handleGameOver;
 
 - (void) setupIntroFirstGo:(BOOL)justCobra;
@@ -379,9 +394,10 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (StationEntity *) stationFriendlyTo:(ShipEntity *) ship;
 
 - (void) resetBeacons;
-- (ShipEntity *) firstBeacon;
-- (ShipEntity *) lastBeacon;
-- (void) setNextBeacon:(ShipEntity *) beaconShip;
+- (Entity <OOBeaconEntity> *) firstBeacon;
+- (Entity <OOBeaconEntity> *) lastBeacon;
+- (void) setNextBeacon:(Entity <OOBeaconEntity> *) beaconShip;
+- (void) clearBeacon:(Entity <OOBeaconEntity> *) beaconShip;
 
 - (GLfloat *) skyClearColor;
 // Note: the alpha value is also air resistance!
@@ -391,9 +407,13 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (BOOL) breakPatternHide;
 
 - (NSString *) randomShipKeyForRoleRespectingConditions:(NSString *)role;
-- (ShipEntity *) newShipWithRole:(NSString *)role;		// Selects ship using role weights, applies auto_ai, respects conditions
-- (ShipEntity *) newShipWithName:(NSString *)shipKey;	// Does not apply auto_ai or respect conditions
-- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy;	// If usePlayerProxy, non-carriers are instantiated as ProxyPlayerEntity.
+- (ShipEntity *) newShipWithRole:(NSString *)role OO_RETURNS_RETAINED;		// Selects ship using role weights, applies auto_ai, respects conditions
+- (ShipEntity *) newShipWithName:(NSString *)shipKey OO_RETURNS_RETAINED;	// Does not apply auto_ai or respect conditions
+- (ShipEntity *) newSubentityWithName:(NSString *)shipKey OO_RETURNS_RETAINED;	// Does not apply auto_ai or respect conditions
+- (OOVisualEffectEntity *) newVisualEffectWithName:(NSString *)effectKey OO_RETURNS_RETAINED;
+- (DockEntity *) newDockWithName:(NSString *)shipKey OO_RETURNS_RETAINED;	// Does not apply auto_ai or respect conditions
+- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy OO_RETURNS_RETAINED;	// If usePlayerProxy, non-carriers are instantiated as ProxyPlayerEntity.
+- (ShipEntity *) newShipWithName:(NSString *)shipKey usePlayerProxy:(BOOL)usePlayerProxy isSubentity:(BOOL)isSubentity OO_RETURNS_RETAINED;
 
 - (Class) shipClassForShipDictionary:(NSDictionary *)dict;
 
@@ -410,15 +430,15 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (NSArray *) getContainersOfCommodity:(NSString*) commodity_name :(OOCargoQuantity) how_many;
 - (void) fillCargopodWithRandomCargo:(ShipEntity *)cargopod;
 
-- (OOCargoType) getRandomCommodity;
-- (OOCargoQuantity) getRandomAmountOfCommodity:(OOCargoType) co_type;
+- (OOCommodityType) getRandomCommodity;
+- (OOCargoQuantity) getRandomAmountOfCommodity:(OOCommodityType) co_type;
 
-- (NSArray *) commodityDataForType:(OOCargoType)type;
-- (OOCargoType) commodityForName:(NSString *) co_name;
-- (NSString *) symbolicNameForCommodity:(OOCargoType) co_type;
-- (NSString *) displayNameForCommodity:(OOCargoType) co_type;
-- (OOMassUnit) unitsForCommodity:(OOCargoType) co_type;
-- (NSString *) describeCommodity:(OOCargoType) co_type amount:(OOCargoQuantity) co_amount;
+- (NSArray *) commodityDataForType:(OOCommodityType)type;
+- (OOCommodityType) commodityForName:(NSString *)co_name;
+- (NSString *) symbolicNameForCommodity:(OOCommodityType)co_type;
+- (NSString *) displayNameForCommodity:(OOCommodityType)co_type;
+- (OOMassUnit) unitsForCommodity:(OOCommodityType)co_type;
+- (NSString *) describeCommodity:(OOCommodityType)co_type amount:(OOCargoQuantity) co_amount;
 
 - (void) setGameView:(MyOpenGLView *)view;
 - (MyOpenGLView *) gameView;
@@ -426,6 +446,10 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (NSDictionary *) gameSettings;
 
 - (void) drawUniverse;
+
+- (void) defineFrustum;
+- (BOOL) viewFrustumIntersectsSphereAt:(Vector)position withRadius:(GLfloat)radius;
+
 - (void) drawMessage;
 
 // Used to draw subentities. Should be getting this from camera.
@@ -445,10 +469,11 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (Entity*) hazardOnRouteFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(Vector) p2;
 - (Vector) getSafeVectorFromEntity:(Entity *) e1 toDistance:(double)dist fromPoint:(Vector) p2;
 
-- (ShipEntity *) getFirstShipHitByLaserFromShip:(ShipEntity *)srcEntity inView:(OOViewID)viewdir offset:(Vector)offset rangeFound:(GLfloat*)range_ptr;
-- (Entity *) getFirstEntityTargetedByPlayer;
+- (ShipEntity *) firstShipHitByLaserFromShip:(ShipEntity *)srcEntity inDirection:(OOWeaponFacing)direction offset:(Vector)offset gettingRangeFound:(GLfloat*)range_ptr;
+- (Entity *) firstEntityTargetedByPlayer;
+- (Entity *) firstEntityTargetedByPlayerPrecisely;
 
-- (NSArray *) getEntitiesWithinRange:(double)range ofEntity:(Entity *)entity;
+- (NSArray *) entitiesWithinRange:(double)range ofEntity:(Entity *)entity;
 - (unsigned) countShipsWithRole:(NSString *)role inRange:(double)range ofEntity:(Entity *)entity;
 - (unsigned) countShipsWithRole:(NSString *)role;
 - (unsigned) countShipsWithPrimaryRole:(NSString *)role inRange:(double)range ofEntity:(Entity *)entity;
@@ -476,6 +501,10 @@ typedef uint8_t		OOEconomyID;		// 0..7
 									  parameter:(void *)parameter
 										inRange:(double)range
 									   ofEntity:(Entity *)entity;
+- (NSMutableArray *) findVisualEffectsMatchingPredicate:(EntityFilterPredicate)predicate
+									  parameter:(void *)parameter
+										inRange:(double)range
+									   ofEntity:(Entity *)entity;
 - (id) nearestEntityMatchingPredicate:(EntityFilterPredicate)predicate
 							parameter:(void *)parameter
 					 relativeToEntity:(Entity *)entity;
@@ -491,8 +520,9 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (NSString*) collisionDescription;
 - (void) dumpCollisions;
 
-- (void) setViewDirection:(OOViewID) vd;
 - (OOViewID) viewDirection;
+- (void) setViewDirection:(OOViewID)vd;
+- (void) enterGUIViewModeWithMouseInteraction:(BOOL)mouseInteraction;	// Use instead of setViewDirection:VIEW_GUI_DISPLAY
 
 - (NSString *) soundNameForCustomSoundKey:(NSString *)key;
 - (NSDictionary *) screenTextureDescriptorForKey:(NSString *)key;
@@ -569,7 +599,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (Random_Seed) findNeighbouringSystemToCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
 - (Random_Seed) findConnectedSystemAtCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
-- (int) findSystemNumberAtCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
+- (OOSystemID) findSystemNumberAtCoords:(NSPoint) coords withGalaxySeed:(Random_Seed) gal_seed;
 - (NSPoint) findSystemCoordinatesWithPrefix:(NSString *) p_fix;
 - (NSPoint) findSystemCoordinatesWithPrefix:(NSString *) p_fix exactMatch:(BOOL) exactMatch;
 - (BOOL*) systemsFound;
@@ -596,8 +626,8 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (NSString *) shortTimeDescription:(OOTimeDelta) interval;
 
 - (Random_Seed) marketSeed;
-- (NSArray *) passengersForLocalSystemAtTime:(OOTimeAbsolute) current_time;
-- (NSArray *) contractsForLocalSystemAtTime:(OOTimeAbsolute) current_time;
+//- (NSArray *) passengersForLocalSystemAtTime:(OOTimeAbsolute) current_time;
+//- (NSArray *) contractsForLocalSystemAtTime:(OOTimeAbsolute) current_time;
 - (NSArray *) shipsForSaleForSystem:(Random_Seed) s_seed withTL:(OOTechLevelID) specialTL atTime:(OOTimeAbsolute) current_time;
 
 /* Calculate base cost, before depreciation */
@@ -627,9 +657,6 @@ typedef uint8_t		OOEconomyID;		// 0..7
 
 - (void) resetCommsLogColor;
 
-- (void) setDisplayCursor:(BOOL) value;
-- (BOOL) displayCursor;
-
 - (void) setDisplayText:(BOOL) value;
 - (BOOL) displayGUI;
 
@@ -651,7 +678,7 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (OOShaderSetting) shaderEffectsLevel;
 - (BOOL) useShaders;
 
-- (void) handleOoliteException:(NSException*) ooliteException;
+- (void) handleOoliteException:(NSException *)ooliteException;
 
 - (GLfloat)airResistanceFactor;
 
@@ -691,6 +718,10 @@ typedef uint8_t		OOEconomyID;		// 0..7
 - (BOOL) blockJSPlayerShipProps;
 - (void) setBlockJSPlayerShipProps:(BOOL)value;
 
+- (void) loadConditionScripts;
+- (void) addConditionScripts:(NSEnumerator *)scripts;
+- (OOJSScript *) getConditionScript:(NSString *)scriptname;
+
 @end
 
 
@@ -698,20 +729,22 @@ typedef uint8_t		OOEconomyID;		// 0..7
 	The purpose of this is that it makes UNIVERSE essentially a read-only
 	global with zero overhead.
 */
-OOINLINE Universe *GetUniverse(void) INLINE_CONST_FUNC;
-OOINLINE Universe *GetUniverse(void)
+OOINLINE Universe *OOGetUniverse(void) INLINE_CONST_FUNC;
+OOINLINE Universe *OOGetUniverse(void)
 {
 	extern Universe *gSharedUniverse;
 	return gSharedUniverse;
 }
-#define UNIVERSE GetUniverse()
+#define UNIVERSE OOGetUniverse()
 
 
 // Only for use with string literals, and only for looking up strings.
-NSString *DESC_(NSString *key);
-NSString *DESC_PLURAL_(NSString *key, int count);
-#define DESC(key)	(DESC_(key ""))
-#define DESC_PLURAL(key,count)	(DESC_PLURAL_(key, count))
+#define DESC(key)	(OOLookUpDescriptionPRIV(key ""))
+#define DESC_PLURAL(key,count)	(OOLookUpPluralDescriptionPRIV(key "", count))
+
+// Not for direct use.
+NSString *OOLookUpDescriptionPRIV(NSString *key);
+NSString *OOLookUpPluralDescriptionPRIV(NSString *key, NSInteger count);
 
 
 @interface OOSound (OOCustomSounds)
