@@ -3,7 +3,7 @@
 MyOpenGLView.m
 
 Oolite
-Copyright (C) 2004-2012 Giles C Williams and contributors
+Copyright (C) 2004-2013 Giles C Williams and contributors
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,6 +36,7 @@ MA 02110-1301, USA.
 #import "PlanetEntity.h"
 #import "OOGraphicsResetManager.h"
 #import "OOCollectionExtractors.h" // for splash screen settings
+#import "OOFullScreenController.h"
 
 #define kOOLogUnconvertedNSLog @"unclassified.MyOpenGLView"
 
@@ -92,10 +93,9 @@ MA 02110-1301, USA.
 #if OOLITE_WINDOWS
 		// Pre setVideoMode adjustments.
 		NSSize tmp = currentWindowSize;
-		updateContext = NO;	//don't update the window!
-		ShowWindow(SDL_Window,SW_HIDE);
-		MoveWindow(SDL_Window,GetSystemMetrics(SM_CXSCREEN)/2,GetSystemMetrics(SM_CYSCREEN)/2,1,1,TRUE);
-		ShowWindow(SDL_Window,SW_MINIMIZE);
+		ShowWindow(SDL_Window,SW_SHOWMINIMIZED);
+		updateContext = NO;	//don't update the (splash screen) window yet!
+		MoveWindow(SDL_Window,GetSystemMetrics(SM_CXSCREEN)/2,GetSystemMetrics(SM_CYSCREEN)/2,1,1,TRUE); // centre the splash screen
 		
 		// Initialise the SDL surface. (need custom SDL.dll)
 		surface = SDL_SetVideoMode(firstScreen.width, firstScreen.height, 32, videoModeFlags);
@@ -116,13 +116,22 @@ MA 02110-1301, USA.
 		updateContext = YES;
 #endif
 		surface = SDL_SetVideoMode(firstScreen.width, firstScreen.height, 32, videoModeFlags);
-		// blank the surface / go to fullscreen
 		if (!surface) {
 			return;
 		}
-
+		// blank the surface / go to fullscreen
 		[self initialiseGLWithSize: firstScreen];
 	}
+	
+	_gamma = 1.0f;
+	if (SDL_SetGamma(_gamma, _gamma, _gamma) < 0 ) 
+	{
+		char * errStr = SDL_GetError();
+		OOLogWARN(@"gamma.set.failed", @"Could not set gamma: %s", errStr);
+		// CIM: this doesn't seem to necessarily be fatal. Gamma settings
+		// mostly work on mine despite this function failing.
+		//	exit(1);
+	}
 }
 
 
@@ -151,7 +160,7 @@ MA 02110-1301, USA.
 		if ([arg isEqual:@"-nosplash"] || [arg isEqual:@"--nosplash"])
 		{
 			showSplashScreen = NO;
-			break;
+			break;	// -nosplash always trumps -splash
 		}
 		else if ([arg isEqual:@"-splash"] || [arg isEqual:@"--splash"])
 		{
@@ -224,21 +233,39 @@ MA 02110-1301, USA.
 	firstScreen= (fullScreen) ? [self modeAsSize: currentSize] : currentWindowSize;
 	viewSize = firstScreen;	// viewSize must be set prior to splash screen initialization
 	
+	OOLog(@"display.initGL",@"Trying 32-bit depth buffer");
 	[self createSurface];
 	if (surface == NULL)
 	{
-		// Retry, allowing 16-bit contexts.
-		SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
-		SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
-		SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
-		SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
+		// Retry with a 24-bit depth buffer
+		OOLog(@"display.initGL",@"Trying 24-bit depth buffer");
+		SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
 		[self createSurface];
-		
 		if (surface == NULL)
 		{
-			char * errStr = SDL_GetError();
-			OOLogERR(@"display.mode.error", @"Could not create display surface: %s", errStr);
-			exit(1);
+			// Still not working? One last go...
+			// Retry, allowing 16-bit contexts.
+			OOLog(@"display.initGL",@"Trying 16-bit depth buffer");
+			SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
+			SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
+			SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
+			SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
+			[self createSurface];
+			
+			if (surface == NULL)
+			{
+				char * errStr = SDL_GetError();
+				OOLogERR(@"display.mode.error", @"Could not create display surface: %s", errStr);
+#if OOLITE_WINDOWS
+				if (showSplashScreen)
+				{
+					[[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"splash-screen"];
+					OOLogWARN(@"display.mode.conflict",@"Possible incompatibility between the splash screen and video drivers detected.");
+					OOLogWARN(@"display.mode.conflict",@"Oolite will start without showing the splash screen from now on. Override with 'oolite.exe -splash'");
+				}
+#endif
+				exit(1);
+			}
 		}
 	}
 	
@@ -292,8 +319,8 @@ MA 02110-1301, USA.
 	/* MKW 2011.11.11
 	 * Eat all SDL events to gobble up any resize events while the
 	 * splash-screen was visible.  They affected the main window after 1.74.
-	 * TODO Post-MNSR: should really process SDL events while the splash-screen
-	 *                 is being displayed.
+	 * TODO: should really process SDL events while showing the splash-screen
+
 	int numEvents = 0;
 	*/
 	SDL_Event dummyEvent;
@@ -426,6 +453,13 @@ MA 02110-1301, USA.
 }
 
 
+- (void) noteMouseInteractionModeChangedFrom:(OOMouseInteractionMode)oldMode to:(OOMouseInteractionMode)newMode
+{
+	[self autoShowMouse];
+	[self setMouseInDeltaMode:OOMouseInteractionModeIsFlightMode(newMode)];
+}
+
+
 - (BOOL) inFullScreenMode
 {
 	return fullScreen;
@@ -575,13 +609,18 @@ MA 02110-1301, USA.
 	
 	/* MKW 2011.11.11
 	 * According to Marc using the NOFRAME flag causes trouble under Ubuntu 8.04.
-	 * TODO: Investigate & fix!
-	 * surface = SDL_SetVideoMode(dest.w, dest.h, 32, SDL_HWSURFACE | SDL_OPENGL | SDL_NOFRAME);
+	 *
+	 * The current Ubuntu LTS is 10.04, which doesn't seem to have that problem.
+	 * 12.04 LTS is going to be released soon, also without apparent problems.
+	 * Changed to SDL_NOFRAME, throwing caution to the wind - Kaks 2012.03.23
+	 * Took SDL_NOFRAME out, since it still causes strange problems here - cim 2012.04.09
 	 */
 	 surface = SDL_SetVideoMode(dest.w, dest.h, 32, SDL_HWSURFACE | SDL_OPENGL);
 	
   #endif
 	
+	OOSetOpenGLState(OPENGL_STATE_OVERLAY);
+
 	glViewport( 0, 0, dest.w, dest.h);
 	
 	glEnable( GL_TEXTURE_2D );
@@ -656,6 +695,9 @@ MA 02110-1301, USA.
 		SDL_FreeSurface( image );
 	}
 	glDeleteTextures(1, &texture);
+
+	glDisable( GL_TEXTURE_2D );
+	OOVerifyOpenGLState();
 }
 
 - (void) initialiseGLWithSize:(NSSize) v_size
@@ -775,6 +817,13 @@ MA 02110-1301, USA.
 		surface = SDL_SetVideoMode((int)viewSize.width, (int)viewSize.height, 32, videoModeFlags);
 	}
 
+	if (!surface)
+	{
+	  // we should always have a valid surface, but in case we don't
+		OOLogERR(@"display.mode.error",@"Unable to change display mode: %s",SDL_GetError());
+		exit(1);
+	}
+
 	bounds.size.width = surface->w;
 	bounds.size.height = surface->h;
 
@@ -1211,7 +1260,7 @@ MA 02110-1301, USA.
 			case SDL_JOYBUTTONUP:
 			case SDL_JOYBUTTONDOWN:
 			case SDL_JOYHATMOTION:
-				[[OOJoystickManager sharedStickHandler] handleSDLEvent: &event];
+				[(OOSDLJoystickManager*)[OOJoystickManager sharedStickHandler] handleSDLEvent: &event];
 				break;
 				
 			case SDL_MOUSEBUTTONDOWN:
@@ -1928,6 +1977,22 @@ keys[a] = NO; keys[b] = NO; \
 }
 
 
+- (void) setGammaValue: (float) value
+{
+	if (value < 0.2f)  value = 0.2f;
+	if (value > 4.0f)  value = 4.0f;
+	
+	_gamma = value;
+	SDL_SetGamma(_gamma, _gamma, _gamma);
+}
+
+
+- (float) gammaValue
+{
+	return _gamma;
+}
+
+
 + (BOOL)pollShiftKey
 {
 #if OOLITE_WINDOWS
@@ -1946,9 +2011,9 @@ keys[a] = NO; keys[b] = NO; \
 #ifndef NDEBUG
 - (void) dumpRGBAToFileNamed:(NSString *)name
 					   bytes:(uint8_t *)bytes
-					   width:(OOUInteger)width
-					  height:(OOUInteger)height
-					rowBytes:(OOUInteger)rowBytes
+					   width:(NSUInteger)width
+					  height:(NSUInteger)height
+					rowBytes:(NSUInteger)rowBytes
 {
 	if (name == nil || bytes == NULL || width == 0 || height == 0 || rowBytes < width * 4)  return;
 	
@@ -1965,9 +2030,9 @@ keys[a] = NO; keys[b] = NO; \
 
 - (void) dumpRGBToFileNamed:(NSString *)name
 					   bytes:(uint8_t *)bytes
-					   width:(OOUInteger)width
-					  height:(OOUInteger)height
-					rowBytes:(OOUInteger)rowBytes
+					   width:(NSUInteger)width
+					  height:(NSUInteger)height
+					rowBytes:(NSUInteger)rowBytes
 {
 	if (name == nil || bytes == NULL || width == 0 || height == 0 || rowBytes < width * 3)  return;
 	
@@ -1983,9 +2048,9 @@ keys[a] = NO; keys[b] = NO; \
 
 - (void) dumpGrayToFileNamed:(NSString *)name
 					   bytes:(uint8_t *)bytes
-					   width:(OOUInteger)width
-					  height:(OOUInteger)height
-					rowBytes:(OOUInteger)rowBytes
+					   width:(NSUInteger)width
+					  height:(NSUInteger)height
+					rowBytes:(NSUInteger)rowBytes
 {
 	if (name == nil || bytes == NULL || width == 0 || height == 0 || rowBytes < width)  return;
 	
@@ -2001,9 +2066,9 @@ keys[a] = NO; keys[b] = NO; \
 
 - (void) dumpGrayAlphaToFileNamed:(NSString *)name
 					   bytes:(uint8_t *)bytes
-					   width:(OOUInteger)width
-					  height:(OOUInteger)height
-					rowBytes:(OOUInteger)rowBytes
+					   width:(NSUInteger)width
+					  height:(NSUInteger)height
+					rowBytes:(NSUInteger)rowBytes
 {
 	if (name == nil || bytes == NULL || width == 0 || height == 0 || rowBytes < width * 2)  return;
 	
@@ -2020,14 +2085,14 @@ keys[a] = NO; keys[b] = NO; \
 - (void) dumpRGBAToRGBFileNamed:(NSString *)rgbName
 			   andGrayFileNamed:(NSString *)grayName
 						  bytes:(uint8_t *)bytes
-						  width:(OOUInteger)width
-						 height:(OOUInteger)height
-					   rowBytes:(OOUInteger)rowBytes
+						  width:(NSUInteger)width
+						 height:(NSUInteger)height
+					   rowBytes:(NSUInteger)rowBytes
 {
 	if ((rgbName == nil && grayName == nil) || bytes == NULL || width == 0 || height == 0 || rowBytes < width * 4)  return;
 	
 	uint8_t				*rgbBytes, *rgbPx, *grayBytes, *grayPx, *srcPx;
-	OOUInteger			x, y;
+	NSUInteger			x, y;
 	BOOL				trivalAlpha = YES;
 	
 	rgbPx = rgbBytes = malloc(width * height * 3);
